//==========================================================
//	ARTRAY Camera / Capture Module Software Developer Kit
//
//						Version 1.20		2003.9.4
//						Version 1.21		2003.9.18
//						Version 1.22		2003.10.10
//						Version 1.23		2004.1.21
//						Version 1.235		2004.3.19
//						Version 1.24		2004.7.9
//						Version 1.245		2004.8.17
//						Version 1.25		2004.9.1
//						Version 1.26		2004.10.28
//						Version 1.27		2005.3.8
//						Version 1.275		2005.4.18
//						Version 1.276		2005.9.5
//						Version 1.277		2005.12.22
//
//									(C) 2002-2005 Artray
//==========================================================


#ifndef _CARTCAMSDK_
#define _CARTCAMSDK_

#if _MSC_VER > 1000
	#pragma once
#endif


#include <windows.h>



#define WM_GRAPHNOTIFY  WM_APP + 1		// Not used in this version.
#define WM_GRAPHPAINT	WM_APP + 2		// Sent to application when an image is updated.
#define WM_ERROR		WM_APP + 3		// Sent to application when an error occurs


#define  HACAM  DWORD

enum {
	ARTCAM_CAMERATYPE_DS		= 1,	// DirectShow Camera
	ARTCAM_CAMERATYPE_DISABLE1	= 2,	// Invalid
	ARTCAM_CAMERATYPE_USTC		= 3,	// ARTUST
	ARTCAM_CAMERATYPE_CNV		= 4,	// ARTCNV
	ARTCAM_CAMERATYPE_DISABLE2	= 5,	// Invalid
	ARTCAM_CAMERATYPE_130MI		= 6,	// ARTCAM-130MI
	ARTCAM_CAMERATYPE_DISABLE3	= 7,	// Invalid
	ARTCAM_CAMERATYPE_200MI		= 8,	// ARTCAM-200MI
	ARTCAM_CAMERATYPE_300MI		= 9,	// ARTCAM-300MI
	ARTCAM_CAMERATYPE_150P		= 10,	// ARTCAM-150P
	ARTCAM_CAMERATYPE_320P		= 11,	// ARTCAM-320P
	ARTCAM_CAMERATYPE_DISABLE4	= 12,	// Invalid
	ARTCAM_CAMERATYPE_200SH		= 13,	// ARTCAM-200SH
	ARTCAM_CAMERATYPE_098		= 14,	// ARTCAM-098
	ARTCAM_CAMERATYPE_036MI		= 15,	// ARTCAM-036MI
	ARTCAM_CAMERATYPE_500P		= 16,	// ARTCAM-500P
	ARTCAM_CAMERATYPE_150P2		= 17,	// ARTCAM-150PII
};

// Error Code
enum {
	ARTCAMSDK_NOERROR = 0,			// Normal
	ARTCAMSDK_NOT_INITIALIZE,		// Not initialized
	ARTCAMSDK_DISABLEDDEVICE,		// Access to unavailable device was attempted
	ARTCAMSDK_CREATETHREAD,			// Failure of thread creation for capturing
	ARTCAMSDK_CREATEWINDOW,			// Window creation failed
	ARTCAMSDK_OUTOFMEMORY,			// Not enough memory for image transfer/Failure for securing memory
	ARTCAMSDK_CAMERASET,			// Error for camera (device) settings
	ARTCAMSDK_CAMERASIZE,			// Error for camera (device) size settings
	ARTCAMSDK_CAPTURE,				// Capturing failed
	ARTCAMSDK_PARAM,				// Wrong argument
	ARTCAMSDK_DIRECTSHOW,			// Directshow initialization error
	ARTCAMSDK_UNSUPPORTED,			// Not supported
	ARTCAMSDK_UNKNOWN,				// Unknow error
	ARTCAMSDK_CAPTURELOST,			// Device lost
};

// Information for transferring

typedef struct GP_INFO {
	LONG	lSize;
	LONG	lWidth;
	LONG	lHeight;
	LONG	lBpp;
	LONG	lFps;
} *LPGP_INFO;


// Type of image filter
enum {
	ARTCAM_FILTERTYPE_RESERVE = 0,			// Reserved
	ARTCAM_FILTERTYPE_BRIGHTNESS,			// Brightness
	ARTCAM_FILTERTYPE_CONTRAST,				// Contrast
	ARTCAM_FILTERTYPE_HUE,					// Hue
	ARTCAM_FILTERTYPE_SATURATION,			// Saturation
	ARTCAM_FILTERTYPE_SHARPNESS,			// Sharpness
	ARTCAM_FILTERTYPE_BAYER_GAIN_RGB,		// Bayer coversion color gain
	ARTCAM_FILTERTYPE_BAYER_GAIN_R,			// Color gain (red only) for bayer conversion
	ARTCAM_FILTERTYPE_BAYER_GAIN_G,			// Color gain (green only) for bayer conversion
	ARTCAM_FILTERTYPE_BAYER_GAIN_B,			// Color gain (Blue only) for bayer conversionColor gain (blue only) for bayer conversion
	ARTCAM_FILTERTYPE_BAYER_GAIN_AUTO,		// Auto white balance
	ARTCAM_FILTERTYPE_GAMMA,				// Gumma
	ARTCAM_FILTERTYPE_BAYERMODE,			// Bayer conversion mode
	ARTCAM_FILTERTYPE_GLOBAL_GAIN,			// Global gain for camera
	ARTCAM_FILTERTYPE_COLOR_GAIN_R,			// Color gain (red) for camera
	ARTCAM_FILTERTYPE_COLOR_GAIN_G1,		// Color gain (green1) for camera
	ARTCAM_FILTERTYPE_COLOR_GAIN_G2,		// Color gain (green2) for camera
	ARTCAM_FILTERTYPE_COLOR_GAIN_B,			// Color gain (blue) for camera
	ARTCAM_FILTERTYPE_EXPOSURETIME,			// Exposure time (shutter speed)
};

// Compatibility with previous versions
#define ARTCAM_FILTERTYPE_RGB_GAIN		ARTCAM_FILTERTYPE_BAYER_GAIN_RGB
#define ARTCAM_FILTERTYPE_R_GAIN		ARTCAM_FILTERTYPE_BAYER_GAIN_R
#define ARTCAM_FILTERTYPE_G_GAIN		ARTCAM_FILTERTYPE_BAYER_GAIN_G
#define ARTCAM_FILTERTYPE_B_GAIN		ARTCAM_FILTERTYPE_BAYER_GAIN_B
#define ARTCAM_FILTERTYPE_AUTO_GAIN		ARTCAM_FILTERTYPE_BAYER_GAIN_AUTO


// Pixel skipping transfer mode
enum {
	SUBSAMPLE_1 = 0,	// Full size
	SUBSAMPLE_2,		// 1/2
	SUBSAMPLE_4,		// 1/4
	SUBSAMPLE_8,		// 1/8
};


// Auto iris mode
enum {
	AI_NONE = 0,		// Invalid
	AI_EXPOSURE,		// Exposure time
	AI_GAIN,			// Gain
};


// Video format
enum {
	VIDEOFORMAT_NTSC = 0,	// NTSC
	VIDEOFORMAT_PAL,		// PAL
	VIDEOFORMAT_PALM,		// PALM
	VIDEOFORMAT_SECAM,		// SECAM
};

// Sampling rate
enum {
	WIDE_HISPEED = 0,	// 
	WIDE_LOWSPEED,		// 
	NORMAL_HISPEED,		// 
	NORMAL_LOWSPEED,	// 
};


// Format for image saving
enum {
	FILETYPE_BITMAP = 0,	// Bitmap saving (large size).
	FILETYPE_RAW,			// Only data is saved (large size)
	FILETYPE_JPEG_HIGH,		// Save in high-quality JPEG format (medium size). Characters and lines are distorted.
	FILETYPE_JPEG_NOMAL,	// Save in standard JPEG format (small size). Small patterns are distorted.
	FILETYPE_JPEG_LOW,		// Save in low-quality JPEG (small size). Block noise are seen.
	FILETYPE_PNG,			// Save as PNG. 16 bits image can be saved correctly. Size=Medium
	FILETYPE_TIFF,			// Save as TIFF. 16 bits image can be saved correctly. Size=Large
};


/*
 *Remark*
 16 bit image cannot be read in every image-processing software or in every environment.
 16 bit PNG and TIFF saved by this SDK can be read by our Veiwer Software Version2 or later and Adobe Photoshop6.
 For all other software, deterioration of image or an error may be experienced.
 Since it saves as a 16-bit gray scale picture when saved by 16-bit bitmap   The color palette etc. is not saved. 
 Since it saves as a 16-bit gray scale picture when saved by 16-bit bitmap   The color palette etc. is not saved.
 
 In case it saves by 16 bits which needs to prepare a 16-bit palette by the software side, 
 or needs to prepare the device context of gray scale, we recommend you to save by TIFF or PNG. 
 In JPEG, it cannot save other than a color picture. 
 Especially about a 16-bit picture, data of 8 bits of low ranks is omitted and saved. 
 
 
 
 
*/






typedef DWORD		(*GETDLLVERSION)		(void);
typedef LONG		(*GETLASTERROR)			(HACAM hACam);

typedef HACAM		(*INITIALIZE)			(HWND hWnd);
typedef BOOL		(*RELEASE)				(HACAM hACam);

typedef BOOL		(*PREVIEW)				(HACAM hACam);
typedef BOOL		(*RECORD)				(HACAM hACam, LPCTSTR lpAviName, UINT RecTime, BOOL fShow);
typedef BOOL		(*CALLBACKPREVIEW)		(HACAM hACam, HWND hWnd, LPBYTE lpImage, LONG Size, BOOL TopDown);
typedef BOOL		(*SNAPSHOT)				(HACAM hACam, LPBYTE lpImage, LONG Size, BOOL TopDown);
typedef BOOL		(*CLOSE)				(HACAM hACam);
typedef BOOL		(*CAPTURE)				(HACAM hACam);
typedef BOOL		(*TRIGGER)				(HACAM hACam, HWND hWnd, LPBYTE lpImage, LONG Size, BOOL TopDown);
typedef BOOL		(*SAVEIMAGE)			(HACAM hACam, LPCTSTR lpSaveName, LONG FileType);

typedef BOOL		(*SETPREVIEWWINDOW)		(HACAM hACam, HWND hWnd, LONG Left, LONG Top, LONG Right, LONG Bottom);
typedef BOOL		(*SETCAPTUREWINDOW)		(HACAM hACam, LONG Width, LONG Height, LONG Fps);
typedef BOOL		(*SETCAPTUREWINDOWEX)	(HACAM hACam, LONG HTotal, LONG HStart, LONG HEffective, LONG VTotal, LONG VStart, LONG VEffective);
typedef BOOL		(*GETCAPTUREWINDOWEX)	(HACAM hACam, LONG* HTotal, LONG* HStart, LONG* HEffective, LONG* VTotal, LONG* VStart, LONG* VEffective);

typedef BOOL		(*SETCOLORMODE)			(HACAM hACam, LONG ColorMode);
typedef LONG		(*GETCOLORMODE)			(HACAM hACam);

typedef BOOL		(*SETCROSSBAR)			(HACAM hACam, LONG Output, LONG Input);
typedef BOOL		(*SETDEVICENUMBER)		(HACAM hACam, LONG Number);
typedef LONG		(*ENUMDEVICE)			(HACAM hACam, TCHAR szDeviceName[10][256]);
typedef BOOL		(*GETDEVICENAME)		(HACAM hACam, LONG index, LPSTR szDeviceName, LONG nSize);

typedef BOOL		(*STARTPREVIEW)			(HACAM hACam);
typedef BOOL		(*STOPPREVIEW)			(HACAM hACam);
typedef BOOL		(*GETIMAGE)				(HACAM hACam, LPBYTE lpImage, LONG Size, BOOL TopDown);

typedef LONG		(*WIDTH)				(HACAM hACam);
typedef LONG		(*HEIGHT)				(HACAM hACam);
typedef LONG		(*FPS)					(HACAM hACam);

typedef BOOL		(*SETCAMERADLG)			(HACAM hACam, HWND hWnd);
typedef BOOL		(*SETIMAGEDLG)			(HACAM hACam, HWND hWnd);
typedef BOOL		(*SETANALOGDLG)			(HACAM hACam, HWND hWnd);

typedef BOOL		(*SETIOPORT)			(HACAM hACam, BYTE byteData, LONG longData, DWORD Reserve);
typedef BOOL		(*GETIOPORT)			(HACAM hACam, LPBYTE byteData, LPLONG longData, DWORD Reserve);
typedef BOOL		(*SETFILTERVALUE)		(HACAM hACam, LONG FilterType, LONG Value);
typedef LONG		(*GETFILTERVALUE)		(HACAM hACam, LONG FilterType, LPBOOL Error);

typedef BOOL		(*SETSUBSAMPLE)			(HACAM hACam, LONG SubSampleMode);
typedef LONG		(*GETSUBSAMPLE)			(HACAM hACam);
typedef BOOL		(*SETWAITTIME)			(HACAM hACam, LONG WaitTime);
typedef LONG		(*GETWAITTIME)			(HACAM hACam);


typedef BOOL		(*SETMIRRORV)			(HACAM hACam, BOOL Flg);
typedef BOOL		(*GETMIRRORV)			(HACAM hACam);
typedef BOOL		(*SETMIRRORH)			(HACAM hACam, BOOL Flg);
typedef BOOL		(*GETMIRRORH)			(HACAM hACam);

typedef BOOL		(*SETBRIGHTNESS)		(HACAM hACam, LONG Value);
typedef LONG		(*GETBRIGHTNESS)		(HACAM hACam, LPBOOL Error);
typedef BOOL		(*SETCONTRAST)			(HACAM hACam, LONG Value);
typedef LONG		(*GETCONTRAST)			(HACAM hACam, LPBOOL Error);
typedef BOOL		(*SETHUE)				(HACAM hACam, LONG Value);
typedef LONG		(*GETHUE)				(HACAM hACam, LPBOOL Error);
typedef BOOL		(*SETSATURATION)		(HACAM hACam, LONG Value);
typedef LONG		(*GETSATURATION)		(HACAM hACam, LPBOOL Error);
typedef BOOL		(*SETSHARPNESS)			(HACAM hACam, LONG Value);
typedef LONG		(*GETSHARPNESS)			(HACAM hACam, LPBOOL Error);
typedef BOOL		(*SETBAYERGAINRGB)		(HACAM hACam, LONG Value);
typedef LONG		(*GETBAYERGAINRGB)		(HACAM hACam, LPBOOL Error);
typedef BOOL		(*SETBAYERGAINRED)		(HACAM hACam, LONG Value);
typedef LONG		(*GETBAYERGAINRED)		(HACAM hACam, LPBOOL Error);
typedef BOOL		(*SETBAYERGAINGREEN)	(HACAM hACam, LONG Value);
typedef LONG		(*GETBAYERGAINGREEN)	(HACAM hACam, LPBOOL Error);
typedef BOOL		(*SETBAYERGAINBLUE)		(HACAM hACam, LONG Value);
typedef LONG		(*GETBAYERGAINBLUE)		(HACAM hACam, LPBOOL Error);
typedef BOOL		(*SETBAYERGAINAUTO)		(HACAM hACam, LONG Value);
typedef LONG		(*GETBAYERGAINAUTO)		(HACAM hACam, LPBOOL Error);
typedef BOOL		(*SETGAMMA)				(HACAM hACam, LONG Value);
typedef LONG		(*GETGAMMA)				(HACAM hACam, LPBOOL Error);
typedef BOOL		(*SETBAYERMODE)			(HACAM hACam, LONG Value);
typedef LONG		(*GETBAYERMODE)			(HACAM hACam, LPBOOL Error);

typedef BOOL		(*SETGLOBALGAIN)		(HACAM hACam, LONG Value);
typedef LONG		(*GETGLOBALGAIN)		(HACAM hACam, LPBOOL Error);
typedef BOOL		(*SETCOLORGAINRED)		(HACAM hACam, LONG Value);
typedef LONG		(*GETCOLORGAINRED)		(HACAM hACam, LPBOOL Error);
typedef BOOL		(*SETCOLORGAINGREEN1)	(HACAM hACam, LONG Value);
typedef LONG		(*GETCOLORGAINGREEN1)	(HACAM hACam, LPBOOL Error);
typedef BOOL		(*SETCOLORGAINGREEN2)	(HACAM hACam, LONG Value);
typedef LONG		(*GETCOLORGAINGREEN2)	(HACAM hACam, LPBOOL Error);
typedef BOOL		(*SETCOLORGAINBLUE)		(HACAM hACam, LONG Value);
typedef LONG		(*GETCOLORGAINBLUE)		(HACAM hACam, LPBOOL Error);
typedef BOOL		(*SETEXPOSURETIME)		(HACAM hACam, LONG Value);
typedef LONG		(*GETEXPOSURETIME)		(HACAM hACam, LPBOOL Error);

typedef BOOL		(*SETHALFCLOCK)			(HACAM hACam, LONG Value);
typedef LONG		(*GETHALFCLOCK)			(HACAM hACam, LPBOOL Error);
typedef BOOL		(*SETAUTOIRIS)			(HACAM hACam, LONG Value);
typedef LONG		(*GETAUTOIRIS)			(HACAM hACam, LPBOOL Error);

typedef BOOL		(*SETSAMPLINGRATE)		(HACAM hACam, LONG Value);
typedef LONG		(*GETSAMPLINGRATE)		(HACAM hACam, LPBOOL Error);
typedef LONG		(*GETVIDEOFORMAT)		(HACAM hACam, LPBOOL Error);

typedef BOOL		(*WRITESROMID)			(HACAM hACam, LONG Address, LONG Value);
typedef LONG		(*READSROMID)			(HACAM hACam, LONG Address, LPBOOL Error);


class CArtCamSdk{
private:
	HINSTANCE	m_hArtCamSdk;
	HACAM		m_hACam;


	GETDLLVERSION		m_GetDllVersion;
	GETLASTERROR		m_GetLastError;

	INITIALIZE			m_Initialize;
	RELEASE				m_Release;

	PREVIEW				m_Preview;
	RECORD				m_Record;
	CALLBACKPREVIEW		m_CallBackPreview;
	SNAPSHOT			m_SnapShot;
	CLOSE				m_Close;
	CAPTURE				m_Capture;
	TRIGGER				m_Trigger;
	SAVEIMAGE			m_SaveImage;

	SETPREVIEWWINDOW	m_SetPreviewWindow;
	SETCAPTUREWINDOW	m_SetCaptureWindow;
	SETCAPTUREWINDOWEX	m_SetCaptureWindowEx;
	GETCAPTUREWINDOWEX	m_GetCaptureWindowEx;

	SETCOLORMODE		m_SetColorMode;
	GETCOLORMODE		m_GetColorMode;

	SETCROSSBAR			m_SetCrossbar;
	SETDEVICENUMBER		m_SetDeviceNumber;
	ENUMDEVICE			m_EnumDevice;
	GETDEVICENAME		m_GetDeviceName;

	STARTPREVIEW		m_StartPreview;
	STOPPREVIEW			m_StopPreview;
	GETIMAGE			m_GetImage;

	WIDTH				m_Width;
	HEIGHT				m_Height;
	FPS					m_Fps;

	SETCAMERADLG		m_SetCameraDlg;
	SETIMAGEDLG			m_SetImageDlg;
	SETANALOGDLG		m_SetAnalogDlg;

	SETIOPORT			m_SetIOPort;
	GETIOPORT			m_GetIOPort;
	SETFILTERVALUE		m_SetFilterValue;
	GETFILTERVALUE		m_GetFilterValue;

	SETSUBSAMPLE		m_SetSubSample;
	GETSUBSAMPLE		m_GetSubSample;
	SETWAITTIME			m_SetWaitTime;
	GETWAITTIME			m_GetWaitTime;

	SETMIRRORV			m_SetMirrorV;
	GETMIRRORV			m_GetMirrorV;
	SETMIRRORH			m_SetMirrorH;
	GETMIRRORH			m_GetMirrorH;

	SETBRIGHTNESS		m_SetBrightness;
	GETBRIGHTNESS		m_GetBrightness;
	SETCONTRAST			m_SetContrast;
	GETCONTRAST			m_GetContrast;
	SETHUE				m_SetHue;
	GETHUE				m_GetHue;
	SETSATURATION		m_SetSaturation;
	GETSATURATION		m_GetSaturation;
	SETSHARPNESS		m_SetSharpness;
	GETSHARPNESS		m_GetSharpness;
	SETBAYERGAINRGB		m_SetBayerGainRGB;
	GETBAYERGAINRGB		m_GetBayerGainRGB;
	SETBAYERGAINRED		m_SetBayerGainRed;
	GETBAYERGAINRED		m_GetBayerGainRed;
	SETBAYERGAINGREEN	m_SetBayerGainGreen;
	GETBAYERGAINGREEN	m_GetBayerGainGreen;
	SETBAYERGAINBLUE	m_SetBayerGainBlue;
	GETBAYERGAINBLUE	m_GetBayerGainBlue;
	SETBAYERGAINAUTO	m_SetBayerGainAuto;
	GETBAYERGAINAUTO	m_GetBayerGainAuto;
	SETGAMMA			m_SetGamma;
	GETGAMMA			m_GetGamma;
	SETBAYERMODE		m_SetBayerMode;
	GETBAYERMODE		m_GetBayerMode;

	SETGLOBALGAIN		m_SetGlobalGain;
	GETGLOBALGAIN		m_GetGlobalGain;
	SETCOLORGAINRED		m_SetColorGainRed;
	GETCOLORGAINRED		m_GetColorGainRed;
	SETCOLORGAINGREEN1	m_SetColorGainGreen1;
	GETCOLORGAINGREEN1	m_GetColorGainGreen1;
	SETCOLORGAINGREEN2	m_SetColorGainGreen2;
	GETCOLORGAINGREEN2	m_GetColorGainGreen2;
	SETCOLORGAINBLUE	m_SetColorGainBlue;
	GETCOLORGAINBLUE	m_GetColorGainBlue;
	SETEXPOSURETIME		m_SetExposureTime;
	GETEXPOSURETIME		m_GetExposureTime;

	SETHALFCLOCK		m_SetHalfClock;
	GETHALFCLOCK		m_GetHalfClock;
	SETAUTOIRIS			m_SetAutoIris;
	GETAUTOIRIS			m_GetAutoIris;

	SETSAMPLINGRATE		m_SetSamplingRate;
	GETSAMPLINGRATE		m_GetSamplingRate;
	GETVIDEOFORMAT		m_GetVideoFormat;

	WRITESROMID			m_WriteSromID;
	READSROMID			m_ReadSromID;

public:

	DWORD		GetDllVersion		(void);
	LONG		GetLastError		(void);

	BOOL		Initialize			(HWND hWnd);
	BOOL		Release				(void);

	BOOL		Preview				(void);
	BOOL		Record				(LPCTSTR lpAviName, UINT RecTime, BOOL fShow);
	BOOL		CallBackPreview		(HWND hWnd, LPBYTE lpImage, LONG Size, BOOL TopDown);
	BOOL		SnapShot			(LPBYTE lpImage, LONG Size, BOOL TopDown);		// 1230
	BOOL		Close				(void);

	// 1250
	BOOL		Capture				(void);

	// 1260
	BOOL		Trigger				(HWND hWnd, LPBYTE lpImage, LONG Size, BOOL TopDown);
	BOOL		SaveImage			(LPCTSTR lpSaveName, LONG FileType);

	BOOL		SetPreviewWindow	(HWND hWnd, LONG Left, LONG Top, LONG Right, LONG Bottom);
	BOOL		SetCaptureWindow	(LONG Width, LONG Height, LONG Fps);
	BOOL		SetCaptureWindowEx	(LONG HTotal, LONG HStart, LONG HEffective, LONG VTotal, LONG VStart, LONG VEffective);
	BOOL		GetCaptureWindowEx	(LONG* HTotal, LONG* HStart, LONG* HEffective, LONG* VTotal, LONG* VStart, LONG* VEffective);

	BOOL		SetColorMode		(LONG ColorMode);
	LONG		GetColorMode		(void);

	BOOL		SetCrossbar			(LONG Output, LONG Input);
	BOOL		SetDeviceNumber		(LONG Number);
	LONG		EnumDevice			(TCHAR szDeviceName[10][256]);
	BOOL		GetDeviceName		(LONG index, LPSTR szDeviceName, LONG nSize);

	BOOL		StartPreview		(void);
	BOOL		StopPreview			(void);
	BOOL		GetImage			(LPBYTE lpImage, LONG Size, BOOL TopDown);

	LONG		Width				(void);
	LONG		Height				(void);
	LONG		Fps					(void);

	BOOL		SetCameraDlg		(HWND hWnd);
	BOOL		SetImageDlg			(HWND hWnd);
	BOOL		SetAnalogDlg		(HWND hWnd);

	// 1230
	BOOL		SetIOPort			(BYTE byteData, LONG longData, DWORD Reserve=0);
	BOOL		GetIOPort			(LPBYTE byteData, LPLONG longData, DWORD Reserve=0);
	BOOL		SetFilterValue		(LONG FilterType, LONG Value);
	LONG		GetFilterValue		(LONG FilterType, LPBOOL Error=NULL);

	BOOL		SetSubSample		(LONG SubSampleMode);
	LONG		GetSubSample		(void);
	BOOL		SetWaitTime			(LONG WaitTime);
	LONG		GetWaitTime			(void);

	// 1240
	BOOL		SetMirrorV			(BOOL Flg);
	BOOL		GetMirrorV			(void);
	BOOL		SetMirrorH			(BOOL Flg);
	BOOL		GetMirrorH			(void);

	BOOL		SetBrightness		(LONG Value);
	LONG		GetBrightness		(LPBOOL Error=NULL);
	BOOL		SetContrast			(LONG Value);
	LONG		GetContrast			(LPBOOL Error=NULL);
	BOOL		SetHue				(LONG Value);
	LONG		GetHue				(LPBOOL Error=NULL);
	BOOL		SetSaturation		(LONG Value);
	LONG		GetSaturation		(LPBOOL Error=NULL);
	BOOL		SetSharpness		(LONG Value);
	LONG		GetSharpness		(LPBOOL Error=NULL);
	BOOL		SetBayerGainRGB		(LONG Value);
	LONG		GetBayerGainRGB		(LPBOOL Error=NULL);
	BOOL		SetBayerGainRed		(LONG Value);
	LONG		GetBayerGainRed		(LPBOOL Error=NULL);
	BOOL		SetBayerGainGreen	(LONG Value);
	LONG		GetBayerGainGreen	(LPBOOL Error=NULL);
	BOOL		SetBayerGainBlue	(LONG Value);
	LONG		GetBayerGainBlue	(LPBOOL Error=NULL);
	BOOL		SetBayerGainAuto	(LONG Value);
	LONG		GetBayerGainAuto	(LPBOOL Error=NULL);
	BOOL		SetGamma			(LONG Value);
	LONG		GetGamma			(LPBOOL Error=NULL);
	BOOL		SetBayerMode		(LONG Value);
	LONG		GetBayerMode		(LPBOOL Error=NULL);

	BOOL		SetGlobalGain		(LONG Value);
	LONG		GetGlobalGain		(LPBOOL Error=NULL);
	BOOL		SetColorGainRed		(LONG Value);
	LONG		GetColorGainRed		(LPBOOL Error=NULL);
	BOOL		SetColorGainGreen1	(LONG Value);
	LONG		GetColorGainGreen1	(LPBOOL Error=NULL);
	BOOL		SetColorGainGreen2	(LONG Value);
	LONG		GetColorGainGreen2	(LPBOOL Error=NULL);
	BOOL		SetColorGainBlue	(LONG Value);
	LONG		GetColorGainBlue	(LPBOOL Error=NULL);
	BOOL		SetExposureTime		(LONG Value);
	LONG		GetExposureTime		(LPBOOL Error=NULL);

	// 1260
	BOOL		SetHalfClock		(LONG Value);
	LONG		GetHalfClock		(LPBOOL Error=NULL);
	BOOL		SetAutoIris			(LONG Value);
	LONG		GetAutoIris			(LPBOOL Error=NULL);

	// 1275
	BOOL		SetSamplingRate		(LONG Value);
	LONG		GetSamplingRate		(LPBOOL Error=NULL);
	LONG		GetVideoFormat		(LPBOOL Error=NULL);

	// 1276
	BOOL		WriteSromID			(LONG Address, LONG Value);
	LONG		ReadSromID			(LONG Address, LPBOOL Error=NULL);


public:
	CArtCamSdk(){
		NullSet();
	}

	~CArtCamSdk(){
		FreeLibrary();
	}

	BOOL	LoadLibrary	(LPSTR szDllName);
	void	FreeLibrary	(void);

	BOOL	IsInit		(void) { return m_hArtCamSdk ? TRUE : FALSE; }

protected:
	void	NullSet		(void);
};

inline void CArtCamSdk::NullSet(void)
{
	m_hArtCamSdk			= NULL;
	m_hACam					= NULL;

	m_GetDllVersion			= NULL;
	m_GetLastError			= NULL;
	m_Initialize			= NULL;
	m_Release				= NULL;
	m_Preview				= NULL;
	m_Record				= NULL;
	m_CallBackPreview		= NULL;
	m_SnapShot				= NULL;
	m_Close					= NULL;
	m_Capture				= NULL;
	m_Trigger				= NULL;
	m_SaveImage				= NULL;
	m_SetPreviewWindow		= NULL;
	m_SetCaptureWindow		= NULL;
	m_SetCaptureWindowEx	= NULL;
	m_GetCaptureWindowEx	= NULL;
	m_SetColorMode			= NULL;
	m_GetColorMode			= NULL;
	m_SetCrossbar			= NULL;
	m_SetDeviceNumber		= NULL;
	m_EnumDevice			= NULL;
	m_GetDeviceName			= NULL;
	m_StartPreview			= NULL;
	m_StopPreview			= NULL;
	m_GetImage				= NULL;
	m_Width					= NULL;
	m_Height				= NULL;
	m_Fps					= NULL;
	m_SetCameraDlg			= NULL;
	m_SetImageDlg			= NULL;
	m_SetAnalogDlg			= NULL;
	m_SetIOPort				= NULL;
	m_GetIOPort				= NULL;
	m_SetFilterValue		= NULL;
	m_GetFilterValue		= NULL;
	m_SetSubSample			= NULL;
	m_GetSubSample			= NULL;
	m_SetWaitTime			= NULL;
	m_GetWaitTime			= NULL;
	m_SetMirrorV			= NULL;
	m_GetMirrorV			= NULL;
	m_SetMirrorH			= NULL;
	m_GetMirrorH			= NULL;
	m_SetBrightness			= NULL;
	m_GetBrightness			= NULL;
	m_SetContrast			= NULL;
	m_GetContrast			= NULL;
	m_SetHue				= NULL;
	m_GetHue				= NULL;
	m_SetSaturation			= NULL;
	m_GetSaturation			= NULL;
	m_SetSharpness			= NULL;
	m_GetSharpness			= NULL;
	m_SetBayerGainRGB		= NULL;
	m_GetBayerGainRGB		= NULL;
	m_SetBayerGainRed		= NULL;
	m_GetBayerGainRed		= NULL;
	m_SetBayerGainGreen		= NULL;
	m_GetBayerGainGreen		= NULL;
	m_SetBayerGainBlue		= NULL;
	m_GetBayerGainBlue		= NULL;
	m_SetBayerGainAuto		= NULL;
	m_GetBayerGainAuto		= NULL;
	m_SetGamma				= NULL;
	m_GetGamma				= NULL;
	m_SetBayerMode			= NULL;
	m_GetBayerMode			= NULL;
	m_SetGlobalGain			= NULL;
	m_GetGlobalGain			= NULL;
	m_SetColorGainRed		= NULL;
	m_GetColorGainRed		= NULL;
	m_SetColorGainGreen1	= NULL;
	m_GetColorGainGreen1	= NULL;
	m_SetColorGainGreen2	= NULL;
	m_GetColorGainGreen2	= NULL;
	m_SetColorGainBlue		= NULL;
	m_GetColorGainBlue		= NULL;
	m_SetExposureTime		= NULL;
	m_GetExposureTime		= NULL;
	m_SetHalfClock			= NULL;
	m_GetHalfClock			= NULL;
	m_SetAutoIris			= NULL;
	m_GetAutoIris			= NULL;
	m_SetSamplingRate		= NULL;
	m_GetSamplingRate		= NULL;
	m_GetVideoFormat		= NULL;
	m_WriteSromID			= NULL;
	m_ReadSromID			= NULL;
}

inline BOOL CArtCamSdk::LoadLibrary(LPSTR szDllName)
{
	if(m_hArtCamSdk)
		return TRUE;

	m_hArtCamSdk = ::LoadLibrary(szDllName);

	if(NULL == m_hArtCamSdk){
		return FALSE;
	}


	// DLL version check
	m_GetDllVersion				= (GETDLLVERSION)		::GetProcAddress(m_hArtCamSdk, "ArtCam_GetDllVersion");
	if(!m_GetDllVersion){
		FreeLibrary();
		return FALSE;
	}


	// Obtain version
	WORD Version = LOWORD(GetDllVersion());

	// Standard function
	if(1000 <= Version){
		m_GetLastError				= (GETLASTERROR)		::GetProcAddress(m_hArtCamSdk, "ArtCam_GetLastError");
		m_Initialize				= (INITIALIZE)			::GetProcAddress(m_hArtCamSdk, "ArtCam_Initialize");
		m_Release					= (RELEASE)				::GetProcAddress(m_hArtCamSdk, "ArtCam_Release");
		m_Preview					= (PREVIEW)				::GetProcAddress(m_hArtCamSdk, "ArtCam_Preview");
		m_Record					= (RECORD)				::GetProcAddress(m_hArtCamSdk, "ArtCam_Record");
		m_CallBackPreview			= (CALLBACKPREVIEW)		::GetProcAddress(m_hArtCamSdk, "ArtCam_CallBackPreview");
		m_Close						= (CLOSE)				::GetProcAddress(m_hArtCamSdk, "ArtCam_Close");
		m_SetPreviewWindow			= (SETPREVIEWWINDOW)	::GetProcAddress(m_hArtCamSdk, "ArtCam_SetPreviewWindow");
		m_SetCaptureWindow			= (SETCAPTUREWINDOW)	::GetProcAddress(m_hArtCamSdk, "ArtCam_SetCaptureWindow");
		m_SetCaptureWindowEx		= (SETCAPTUREWINDOWEX)	::GetProcAddress(m_hArtCamSdk, "ArtCam_SetCaptureWindowEx");
		m_GetCaptureWindowEx		= (GETCAPTUREWINDOWEX)	::GetProcAddress(m_hArtCamSdk, "ArtCam_GetCaptureWindowEx");
		m_SetColorMode				= (SETCOLORMODE)		::GetProcAddress(m_hArtCamSdk, "ArtCam_SetColorMode");
		m_GetColorMode				= (GETCOLORMODE)		::GetProcAddress(m_hArtCamSdk, "ArtCam_GetColorMode");
		m_SetCrossbar				= (SETCROSSBAR)			::GetProcAddress(m_hArtCamSdk, "ArtCam_SetCrossbar");
		m_SetDeviceNumber			= (SETDEVICENUMBER)		::GetProcAddress(m_hArtCamSdk, "ArtCam_SetDeviceNumber");
		m_EnumDevice				= (ENUMDEVICE)			::GetProcAddress(m_hArtCamSdk, "ArtCam_EnumDevice");
		m_GetDeviceName				= (GETDEVICENAME)		::GetProcAddress(m_hArtCamSdk, "ArtCam_GetDeviceName");
		m_StartPreview				= (STARTPREVIEW)		::GetProcAddress(m_hArtCamSdk, "ArtCam_StartPreview");
		m_StopPreview				= (STOPPREVIEW)			::GetProcAddress(m_hArtCamSdk, "ArtCam_StopPreview");
		m_GetImage					= (GETIMAGE)			::GetProcAddress(m_hArtCamSdk, "ArtCam_GetImage");
		m_Width						= (WIDTH)				::GetProcAddress(m_hArtCamSdk, "ArtCam_Width");
		m_Height					= (HEIGHT)				::GetProcAddress(m_hArtCamSdk, "ArtCam_Height");
		m_Fps						= (FPS)					::GetProcAddress(m_hArtCamSdk, "ArtCam_Fps");
		m_SetCameraDlg				= (SETCAMERADLG)		::GetProcAddress(m_hArtCamSdk, "ArtCam_SetCameraDlg");
		m_SetImageDlg				= (SETIMAGEDLG)			::GetProcAddress(m_hArtCamSdk, "ArtCam_SetImageDlg");
		m_SetAnalogDlg				= (SETANALOGDLG)		::GetProcAddress(m_hArtCamSdk, "ArtCam_SetAnalogDlg");


		if(	!m_GetLastError || !m_Initialize || !m_Release || !m_Preview || !m_Record || !m_CallBackPreview || !m_Close || 
			!m_SetPreviewWindow || !m_SetCaptureWindow || !m_SetCaptureWindowEx || !m_GetCaptureWindowEx || !m_SetColorMode || !m_GetColorMode || 
			!m_SetCrossbar || !m_SetDeviceNumber || !m_EnumDevice || !m_GetDeviceName || !m_StartPreview || !m_StopPreview || !m_GetImage ||
			!m_Width || !m_Height || !m_Fps || !m_SetCameraDlg || !m_SetImageDlg || !m_SetAnalogDlg)
		{
			FreeLibrary();
			return FALSE;
		}
	}


	// Functions available with V1.23 or later
	if(1230 <= Version){
		m_SetIOPort			= (SETIOPORT)		::GetProcAddress(m_hArtCamSdk, "ArtCam_SetIOPort");
		m_GetIOPort			= (GETIOPORT)		::GetProcAddress(m_hArtCamSdk, "ArtCam_GetIOPort");
		m_SetFilterValue	= (SETFILTERVALUE)	::GetProcAddress(m_hArtCamSdk, "ArtCam_SetFilterValue");
		m_GetFilterValue	= (GETFILTERVALUE)	::GetProcAddress(m_hArtCamSdk, "ArtCam_GetFilterValue");
		m_SnapShot			= (SNAPSHOT)		::GetProcAddress(m_hArtCamSdk, "ArtCam_SnapShot");
		m_SetSubSample		= (SETSUBSAMPLE)	::GetProcAddress(m_hArtCamSdk, "ArtCam_SetSubSample");
		m_GetSubSample		= (GETSUBSAMPLE)	::GetProcAddress(m_hArtCamSdk, "ArtCam_GetSubSample");
		m_SetWaitTime		= (SETWAITTIME)		::GetProcAddress(m_hArtCamSdk, "ArtCam_SetWaitTime");
		m_GetWaitTime		= (GETWAITTIME)		::GetProcAddress(m_hArtCamSdk, "ArtCam_GetWaitTime");

		if(	!m_SetIOPort || !m_GetIOPort || !m_SetFilterValue || !m_GetFilterValue || !m_SnapShot ||
			!m_SetSubSample || !m_GetSubSample || !m_SetWaitTime || !m_GetWaitTime	)
		{
			FreeLibrary();
			return FALSE;
		}
	}


	// Functions available with V1.24 or later
	if(1240 <= Version){
		m_SetMirrorV			= (SETMIRRORV)			::GetProcAddress(m_hArtCamSdk, "ArtCam_SetMirrorV");
		m_GetMirrorV			= (GETMIRRORV)			::GetProcAddress(m_hArtCamSdk, "ArtCam_GetMirrorV");
		m_SetMirrorH			= (SETMIRRORH)			::GetProcAddress(m_hArtCamSdk, "ArtCam_SetMirrorH");
		m_GetMirrorH			= (GETMIRRORH)			::GetProcAddress(m_hArtCamSdk, "ArtCam_GetMirrorH");
		m_SetBrightness			= (SETBRIGHTNESS)		::GetProcAddress(m_hArtCamSdk, "ArtCam_SetBrightness");
		m_GetBrightness			= (GETBRIGHTNESS)		::GetProcAddress(m_hArtCamSdk, "ArtCam_GetBrightness");
		m_SetContrast			= (SETCONTRAST)			::GetProcAddress(m_hArtCamSdk, "ArtCam_SetContrast");
		m_GetContrast			= (GETCONTRAST)			::GetProcAddress(m_hArtCamSdk, "ArtCam_GetContrast");
		m_SetHue				= (SETHUE)				::GetProcAddress(m_hArtCamSdk, "ArtCam_SetHue");
		m_GetHue				= (GETHUE)				::GetProcAddress(m_hArtCamSdk, "ArtCam_GetHue");
		m_SetSaturation			= (SETSATURATION)		::GetProcAddress(m_hArtCamSdk, "ArtCam_SetSaturation");
		m_GetSaturation			= (GETSATURATION)		::GetProcAddress(m_hArtCamSdk, "ArtCam_GetSaturation");
		m_SetSharpness			= (SETSHARPNESS)		::GetProcAddress(m_hArtCamSdk, "ArtCam_SetSharpness");
		m_GetSharpness			= (GETSHARPNESS)		::GetProcAddress(m_hArtCamSdk, "ArtCam_GetSharpness");
		m_SetBayerGainRGB		= (SETBAYERGAINRGB)		::GetProcAddress(m_hArtCamSdk, "ArtCam_SetBayerGainRGB");
		m_GetBayerGainRGB		= (GETBAYERGAINRGB)		::GetProcAddress(m_hArtCamSdk, "ArtCam_GetBayerGainRGB");
		m_SetBayerGainRed		= (SETBAYERGAINRED)		::GetProcAddress(m_hArtCamSdk, "ArtCam_SetBayerGainRed");
		m_GetBayerGainRed		= (GETBAYERGAINRED)		::GetProcAddress(m_hArtCamSdk, "ArtCam_GetBayerGainRed");
		m_SetBayerGainGreen		= (SETBAYERGAINGREEN)	::GetProcAddress(m_hArtCamSdk, "ArtCam_SetBayerGainGreen");
		m_GetBayerGainGreen		= (GETBAYERGAINGREEN)	::GetProcAddress(m_hArtCamSdk, "ArtCam_GetBayerGainGreen");
		m_SetBayerGainBlue		= (SETBAYERGAINBLUE)	::GetProcAddress(m_hArtCamSdk, "ArtCam_SetBayerGainBlue");
		m_GetBayerGainBlue		= (GETBAYERGAINBLUE)	::GetProcAddress(m_hArtCamSdk, "ArtCam_GetBayerGainBlue");
		m_SetBayerGainAuto		= (SETBAYERGAINAUTO)	::GetProcAddress(m_hArtCamSdk, "ArtCam_SetBayerGainAuto");
		m_GetBayerGainAuto		= (GETBAYERGAINAUTO)	::GetProcAddress(m_hArtCamSdk, "ArtCam_GetBayerGainAuto");
		m_SetGamma				= (SETGAMMA)			::GetProcAddress(m_hArtCamSdk, "ArtCam_SetGamma");
		m_GetGamma				= (GETGAMMA)			::GetProcAddress(m_hArtCamSdk, "ArtCam_GetGamma");
		m_SetBayerMode			= (SETBAYERMODE)		::GetProcAddress(m_hArtCamSdk, "ArtCam_SetBayerMode");
		m_GetBayerMode			= (GETBAYERMODE)		::GetProcAddress(m_hArtCamSdk, "ArtCam_GetBayerMode");
		m_SetGlobalGain			= (SETGLOBALGAIN)		::GetProcAddress(m_hArtCamSdk, "ArtCam_SetGlobalGain");
		m_GetGlobalGain			= (GETGLOBALGAIN)		::GetProcAddress(m_hArtCamSdk, "ArtCam_GetGlobalGain");
		m_SetColorGainRed		= (SETCOLORGAINRED)		::GetProcAddress(m_hArtCamSdk, "ArtCam_SetColorGainRed");
		m_GetColorGainRed		= (GETCOLORGAINRED)		::GetProcAddress(m_hArtCamSdk, "ArtCam_GetColorGainRed");
		m_SetColorGainGreen1	= (SETCOLORGAINGREEN1)	::GetProcAddress(m_hArtCamSdk, "ArtCam_SetColorGainGreen1");
		m_GetColorGainGreen1	= (GETCOLORGAINGREEN1)	::GetProcAddress(m_hArtCamSdk, "ArtCam_GetColorGainGreen1");
		m_SetColorGainGreen2	= (SETCOLORGAINGREEN2)	::GetProcAddress(m_hArtCamSdk, "ArtCam_SetColorGainGreen2");
		m_GetColorGainGreen2	= (GETCOLORGAINGREEN2)	::GetProcAddress(m_hArtCamSdk, "ArtCam_GetColorGainGreen2");
		m_SetColorGainBlue		= (SETCOLORGAINBLUE)	::GetProcAddress(m_hArtCamSdk, "ArtCam_SetColorGainBlue");
		m_GetColorGainBlue		= (GETCOLORGAINBLUE)	::GetProcAddress(m_hArtCamSdk, "ArtCam_GetColorGainBlue");
		m_SetExposureTime		= (SETEXPOSURETIME)		::GetProcAddress(m_hArtCamSdk, "ArtCam_SetExposureTime");
		m_GetExposureTime		= (GETEXPOSURETIME)		::GetProcAddress(m_hArtCamSdk, "ArtCam_GetExposureTime");

		if(	!m_SetMirrorV || !m_GetMirrorV || !m_SetMirrorH || !m_GetMirrorH || 
			!m_SetBrightness || !m_GetBrightness || !m_SetContrast || !m_GetContrast || 
			!m_SetHue || !m_GetHue || !m_SetSaturation || !m_GetSaturation || 
			!m_SetSharpness || !m_GetSharpness || !m_SetBayerGainRGB || !m_GetBayerGainRGB || 
			!m_SetBayerGainRed || !m_GetBayerGainRed || !m_SetBayerGainGreen || !m_GetBayerGainGreen || 
			!m_SetBayerGainBlue || !m_GetBayerGainBlue || !m_SetBayerGainAuto || !m_GetBayerGainAuto || 
			!m_SetGamma || !m_GetGamma || !m_SetBayerMode || !m_GetBayerMode || 
			!m_SetGlobalGain || !m_GetGlobalGain || !m_SetColorGainRed || !m_GetColorGainRed || 
			!m_SetColorGainGreen1 || !m_GetColorGainGreen1 || !m_SetColorGainGreen2 || !m_GetColorGainGreen2 || 
			!m_SetColorGainBlue || !m_GetColorGainBlue || !m_SetExposureTime || !m_GetExposureTime)
		{
			FreeLibrary();
			return FALSE;
		}
	}


	// Functions available with V1.25 or later
	if(1250 <= Version){
		m_Capture = (CAPTURE) ::GetProcAddress(m_hArtCamSdk, "ArtCam_Capture");
		if(!m_Capture){
			FreeLibrary();
			return FALSE;
		}
	}



	// Functions available with V1.26 or later
	if(1260 <= Version){
		m_Trigger		= (TRIGGER)			::GetProcAddress(m_hArtCamSdk, "ArtCam_Trigger");
		m_SaveImage		= (SAVEIMAGE)		::GetProcAddress(m_hArtCamSdk, "ArtCam_SaveImage");
		m_SetHalfClock	= (SETHALFCLOCK)	::GetProcAddress(m_hArtCamSdk, "ArtCam_SetHalfClock");
		m_GetHalfClock	= (GETHALFCLOCK)	::GetProcAddress(m_hArtCamSdk, "ArtCam_GetHalfClock");
		m_SetAutoIris	= (SETAUTOIRIS)		::GetProcAddress(m_hArtCamSdk, "ArtCam_SetAutoIris");
		m_GetAutoIris	= (GETAUTOIRIS)		::GetProcAddress(m_hArtCamSdk, "ArtCam_GetAutoIris");

		if(!m_Trigger || !m_SaveImage || !m_SetHalfClock || !m_GetHalfClock || !m_SetAutoIris || !m_GetAutoIris){
			FreeLibrary();
			return FALSE;
		}
	}


	// Functions available with V1.275 or later
	if(1275 <= Version){
		m_SetSamplingRate	= (SETSAMPLINGRATE)	::GetProcAddress(m_hArtCamSdk, "ArtCam_SetSamplingRate");
		m_GetSamplingRate	= (GETSAMPLINGRATE)	::GetProcAddress(m_hArtCamSdk, "ArtCam_GetSamplingRate");
		m_GetVideoFormat	= (GETVIDEOFORMAT)	::GetProcAddress(m_hArtCamSdk, "ArtCam_GetVideoFormat");

		if(!m_SetSamplingRate || !m_GetSamplingRate || !m_GetVideoFormat){
			FreeLibrary();
			return FALSE;
		}
	}


	// Functions available with V1.276 or later
	if(1276 <= Version){
		m_WriteSromID	= (WRITESROMID)	::GetProcAddress(m_hArtCamSdk, "ArtCam_WriteSromID");
		m_ReadSromID	= (READSROMID)	::GetProcAddress(m_hArtCamSdk, "ArtCam_ReadSromID");

		if(!m_WriteSromID || !m_ReadSromID){
			FreeLibrary();
			return FALSE;
		}
	}


	return TRUE;
}

inline void CArtCamSdk::FreeLibrary()
{
	if(m_hACam && m_Release)
		m_Release(m_hACam);

	if(m_hArtCamSdk)
		::FreeLibrary(m_hArtCamSdk);

	NullSet();
}

inline DWORD CArtCamSdk::GetDllVersion(void)
{
	if(m_GetDllVersion){
		return m_GetDllVersion();
	}
	return FALSE;
}

inline LONG CArtCamSdk::GetLastError(void)
{
	if(m_GetLastError){
		return m_GetLastError(m_hACam);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::Initialize(HWND hWnd)
{
	if(m_Initialize){
		m_hACam = m_Initialize(hWnd);
		if(m_hACam)
			return TRUE;
	}
	return FALSE;
}

inline BOOL CArtCamSdk::Release(void)
{
	if(m_Release){
		return m_Release(m_hACam);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::Preview(void)
{
	if(m_Preview){
		return m_Preview(m_hACam);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::Record(LPCTSTR lpAviName, UINT RecTime, BOOL fShow)
{
	if(m_Record){
		return m_Record(m_hACam, lpAviName, RecTime, fShow);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::CallBackPreview(HWND hWnd, LPBYTE lpImage, LONG Size, BOOL TopDown)
{
	if(m_CallBackPreview){
		return m_CallBackPreview(m_hACam, hWnd, lpImage, Size, TopDown);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::SnapShot(LPBYTE lpImage, LONG Size, BOOL TopDown)
{
	if(m_SnapShot){
		return m_SnapShot(m_hACam, lpImage, Size, TopDown);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::Close(void)
{
	if(m_Close){
		return m_Close(m_hACam);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::Capture(void)
{
	if(m_Capture){
		return m_Capture(m_hACam);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::Trigger(HWND hWnd, LPBYTE lpImage, LONG Size, BOOL TopDown)
{
	if(m_Trigger){
		return m_Trigger(m_hACam, hWnd, lpImage, Size, TopDown);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::SaveImage(LPCTSTR lpSaveName, LONG FileType)
{
	if(m_SaveImage){
		return m_SaveImage(m_hACam, lpSaveName, FileType);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::SetPreviewWindow(HWND hWnd, LONG Left, LONG Top, LONG Right, LONG Bottom)
{
	if(m_SetPreviewWindow){
		return m_SetPreviewWindow(m_hACam, hWnd, Left, Top, Right, Bottom);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::SetCaptureWindow(LONG Width, LONG Height, LONG Fps)
{
	if(m_SetCaptureWindow){
		return m_SetCaptureWindow(m_hACam, Width, Height, Fps);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::SetCaptureWindowEx(LONG HTotal, LONG HStart, LONG HEffective, LONG VTotal, LONG VStart, LONG VEffective)
{
	if(m_SetCaptureWindowEx){
		return m_SetCaptureWindowEx(m_hACam, HTotal, HStart, HEffective, VTotal, VStart, VEffective);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::GetCaptureWindowEx(LONG* HTotal, LONG* HStart, LONG* HEffective, LONG* VTotal, LONG* VStart, LONG* VEffective)
{
	if(m_GetCaptureWindowEx){
		return m_GetCaptureWindowEx(m_hACam, HTotal, HStart, HEffective, VTotal, VStart, VEffective);
	}
	return FALSE;
}


inline BOOL CArtCamSdk::SetColorMode(LONG ColorMode)
{
	if(m_SetColorMode){
		return m_SetColorMode(m_hACam, ColorMode);
	}
	return FALSE;
}

inline LONG CArtCamSdk::GetColorMode(void)
{
	if(m_GetColorMode){
		return m_GetColorMode(m_hACam);
	}
	return -1;
}


inline BOOL CArtCamSdk::SetCrossbar(LONG Output, LONG Input)
{
	if(m_SetCrossbar){
		return m_SetCrossbar(m_hACam, Output, Input);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::SetDeviceNumber(LONG Number)
{
	if(m_SetDeviceNumber){
		return m_SetDeviceNumber(m_hACam, Number);
	}
	return FALSE;
}

inline LONG CArtCamSdk::EnumDevice(TCHAR szDeviceName[10][256])
{
	if(m_EnumDevice){
		return m_EnumDevice(m_hACam, szDeviceName);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::GetDeviceName(LONG index, LPSTR szDeviceName, LONG nSize)
{
	if(m_GetDeviceName){
		return m_GetDeviceName(m_hACam, index, szDeviceName, nSize);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::StartPreview(void)
{
	if(m_StartPreview){
		return m_StartPreview(m_hACam);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::StopPreview(void)
{
	if(m_StopPreview){
		return m_StopPreview(m_hACam);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::GetImage(LPBYTE lpImage, LONG Size, BOOL TopDown)
{
	if(m_GetImage){
		return m_GetImage(m_hACam, lpImage, Size, TopDown);
	}
	return FALSE;
}

inline LONG CArtCamSdk::Width(void)
{
	if(m_Width){
		return m_Width(m_hACam);
	}
	return FALSE;
}

inline LONG CArtCamSdk::Height(void)
{
	if(m_Height){
		return m_Height(m_hACam);
	}
	return FALSE;
}

inline LONG CArtCamSdk::Fps(void)
{
	if(m_Fps){
		return m_Fps(m_hACam);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::SetCameraDlg(HWND hWnd)
{
	if(m_SetCameraDlg){
		return m_SetCameraDlg(m_hACam, hWnd);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::SetImageDlg(HWND hWnd)
{
	if(m_SetImageDlg){
		return m_SetImageDlg(m_hACam, hWnd);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::SetAnalogDlg(HWND hWnd)
{
	if(m_SetAnalogDlg){
		return m_SetAnalogDlg(m_hACam, hWnd);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::SetIOPort(BYTE byteData, LONG longData, DWORD Reserve)
{
	if(m_SetIOPort){
		return m_SetIOPort(m_hACam, byteData, longData, Reserve);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::GetIOPort(LPBYTE byteData, LPLONG longData, DWORD Reserve)
{
	if(m_GetIOPort){
		return m_GetIOPort(m_hACam, byteData, longData, Reserve);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::SetFilterValue(LONG FilterType, LONG Value)
{
	if(m_SetFilterValue){
		return m_SetFilterValue(m_hACam, FilterType, Value);
	}
	return FALSE;
}

inline LONG CArtCamSdk::GetFilterValue(LONG FilterType, LPBOOL Error)
{
	if(m_GetFilterValue){
		return m_GetFilterValue(m_hACam, FilterType, Error);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::SetSubSample(LONG SubSampleMode)
{
	if(m_SetSubSample){
		return m_SetSubSample(m_hACam, SubSampleMode);
	}
	return FALSE;
}

inline LONG CArtCamSdk::GetSubSample(void)
{
	if(m_GetSubSample){
		return m_GetSubSample(m_hACam);
	}
	return -1;
}

inline BOOL CArtCamSdk::SetWaitTime(LONG WaitTime)
{
	if(m_SetWaitTime){
		return m_SetWaitTime(m_hACam, WaitTime);
	}
	return FALSE;
}

inline LONG CArtCamSdk::GetWaitTime(void)
{
	if(m_GetWaitTime){
		return m_GetWaitTime(m_hACam);
	}
	return -1;
}

inline BOOL CArtCamSdk::SetMirrorV(BOOL Flg)
{
	if(m_SetMirrorV){
		return m_SetMirrorV(m_hACam, Flg);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::GetMirrorV(void)
{
	if(m_GetMirrorV){
		return m_GetMirrorV(m_hACam);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::SetMirrorH(BOOL Flg)
{
	if(m_SetMirrorH){
		return m_SetMirrorH(m_hACam, Flg);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::GetMirrorH(void)
{
	if(m_GetMirrorH){
		return m_GetMirrorH(m_hACam);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::SetBrightness(LONG Value)
{
	if(m_SetBrightness){
		return m_SetBrightness(m_hACam, Value);
	}
	return FALSE;
}

inline LONG CArtCamSdk::GetBrightness(LPBOOL Error)
{
	if(m_GetBrightness){
		return m_GetBrightness(m_hACam, Error);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::SetContrast(LONG Value)
{
	if(m_SetContrast){
		return m_SetContrast(m_hACam, Value);
	}
	return FALSE;
}

inline LONG CArtCamSdk::GetContrast(LPBOOL Error)
{
	if(m_GetContrast){
		return m_GetContrast(m_hACam, Error);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::SetHue(LONG Value)
{
	if(m_SetHue){
		return m_SetHue(m_hACam, Value);
	}
	return FALSE;
}

inline LONG CArtCamSdk::GetHue(LPBOOL Error)
{
	if(m_GetHue){
		return m_GetHue(m_hACam, Error);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::SetSaturation(LONG Value)
{
	if(m_SetSaturation){
		return m_SetSaturation(m_hACam, Value);
	}
	return FALSE;
}

inline LONG CArtCamSdk::GetSaturation(LPBOOL Error)
{
	if(m_GetSaturation){
		return m_GetSaturation(m_hACam, Error);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::SetSharpness(LONG Value)
{
	if(m_SetSharpness){
		return m_SetSharpness(m_hACam, Value);
	}
	return FALSE;
}

inline LONG CArtCamSdk::GetSharpness(LPBOOL Error)
{
	if(m_GetSharpness){
		return m_GetSharpness(m_hACam, Error);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::SetBayerGainRGB(LONG Value)
{
	if(m_SetBayerGainRGB){
		return m_SetBayerGainRGB(m_hACam, Value);
	}
	return FALSE;
}

inline LONG CArtCamSdk::GetBayerGainRGB(LPBOOL Error)
{
	if(m_GetBayerGainRGB){
		return m_GetBayerGainRGB(m_hACam, Error);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::SetBayerGainRed(LONG Value)
{
	if(m_SetBayerGainRed){
		return m_SetBayerGainRed(m_hACam, Value);
	}
	return FALSE;
}

inline LONG CArtCamSdk::GetBayerGainRed(LPBOOL Error)
{
	if(m_GetBayerGainRed){
		return m_GetBayerGainRed(m_hACam, Error);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::SetBayerGainGreen(LONG Value)
{
	if(m_SetBayerGainGreen){
		return m_SetBayerGainGreen(m_hACam, Value);
	}
	return FALSE;
}

inline LONG CArtCamSdk::GetBayerGainGreen(LPBOOL Error)
{
	if(m_GetBayerGainGreen){
		return m_GetBayerGainGreen(m_hACam, Error);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::SetBayerGainBlue(LONG Value)
{
	if(m_SetBayerGainBlue){
		return m_SetBayerGainBlue(m_hACam, Value);
	}
	return FALSE;
}

inline LONG CArtCamSdk::GetBayerGainBlue(LPBOOL Error)
{
	if(m_GetBayerGainBlue){
		return m_GetBayerGainBlue(m_hACam, Error);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::SetBayerGainAuto(LONG Value)
{
	if(m_SetBayerGainAuto){
		return m_SetBayerGainAuto(m_hACam, Value);
	}
	return FALSE;
}

inline LONG CArtCamSdk::GetBayerGainAuto(LPBOOL Error)
{
	if(m_GetBayerGainAuto){
		return m_GetBayerGainAuto(m_hACam, Error);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::SetGamma(LONG Value)
{
	if(m_SetGamma){
		return m_SetGamma(m_hACam, Value);
	}
	return FALSE;
}

inline LONG CArtCamSdk::GetGamma(LPBOOL Error)
{
	if(m_GetGamma){
		return m_GetGamma(m_hACam, Error);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::SetBayerMode(LONG Value)
{
	if(m_SetBayerMode){
		return m_SetBayerMode(m_hACam, Value);
	}
	return FALSE;
}

inline LONG CArtCamSdk::GetBayerMode(LPBOOL Error)
{
	if(m_GetBayerMode){
		return m_GetBayerMode(m_hACam, Error);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::SetGlobalGain(LONG Value)
{
	if(m_SetGlobalGain){
		return m_SetGlobalGain(m_hACam, Value);
	}
	return FALSE;
}

inline LONG CArtCamSdk::GetGlobalGain(LPBOOL Error)
{
	if(m_GetGlobalGain){
		return m_GetGlobalGain(m_hACam, Error);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::SetColorGainRed(LONG Value)
{
	if(m_SetColorGainRed){
		return m_SetColorGainRed(m_hACam, Value);
	}
	return FALSE;
}

inline LONG CArtCamSdk::GetColorGainRed(LPBOOL Error)
{
	if(m_GetColorGainRed){
		return m_GetColorGainRed(m_hACam, Error);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::SetColorGainGreen1(LONG Value)
{
	if(m_SetColorGainGreen1){
		return m_SetColorGainGreen1(m_hACam, Value);
	}
	return FALSE;
}

inline LONG CArtCamSdk::GetColorGainGreen1(LPBOOL Error)
{
	if(m_GetColorGainGreen1){
		return m_GetColorGainGreen1(m_hACam, Error);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::SetColorGainGreen2(LONG Value)
{
	if(m_SetColorGainGreen2){
		return m_SetColorGainGreen2(m_hACam, Value);
	}
	return FALSE;
}

inline LONG CArtCamSdk::GetColorGainGreen2(LPBOOL Error)
{
	if(m_GetColorGainGreen2){
		return m_GetColorGainGreen2(m_hACam, Error);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::SetColorGainBlue(LONG Value)
{
	if(m_SetColorGainBlue){
		return m_SetColorGainBlue(m_hACam, Value);
	}
	return FALSE;
}

inline LONG CArtCamSdk::GetColorGainBlue(LPBOOL Error)
{
	if(m_GetColorGainBlue){
		return m_GetColorGainBlue(m_hACam, Error);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::SetExposureTime(LONG Value)
{
	if(m_SetExposureTime){
		return m_SetExposureTime(m_hACam, Value);
	}
	return FALSE;
}

inline LONG CArtCamSdk::GetExposureTime(LPBOOL Error)
{
	if(m_GetExposureTime){
		return m_GetExposureTime(m_hACam, Error);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::SetHalfClock(LONG Value)
{
	if(m_SetHalfClock){
		return m_SetHalfClock(m_hACam, Value);
	}
	return FALSE;
}

inline LONG CArtCamSdk::GetHalfClock(LPBOOL Error)
{
	if(m_GetHalfClock){
		return m_GetHalfClock(m_hACam, Error);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::SetAutoIris(LONG Value)
{
	if(m_SetAutoIris){
		return m_SetAutoIris(m_hACam, Value);
	}
	return FALSE;
}

inline LONG CArtCamSdk::GetAutoIris(LPBOOL Error)
{
	if(m_GetAutoIris){
		return m_GetAutoIris(m_hACam, Error);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::SetSamplingRate(LONG Value)
{
	if(m_SetSamplingRate){
		return m_SetSamplingRate(m_hACam, Value);
	}
	return FALSE;
}

inline LONG CArtCamSdk::GetSamplingRate(LPBOOL Error)
{
	if(m_GetSamplingRate){
		return m_GetSamplingRate(m_hACam, Error);
	}
	return FALSE;
}

inline LONG CArtCamSdk::GetVideoFormat(LPBOOL Error)
{
	if(m_GetVideoFormat){
		return m_GetVideoFormat(m_hACam, Error);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::WriteSromID(LONG Address, LONG Value)
{
	if(m_SetSamplingRate){
		return m_WriteSromID(m_hACam, Address, Value);
	}
	return FALSE;
}

inline LONG CArtCamSdk::ReadSromID(LONG Address, LPBOOL Error)
{
	if(m_GetSamplingRate){
		return m_ReadSromID(m_hACam, Address, Error);
	}
	return FALSE;
}


#endif	// #ifndef _CARTCAMSDK_
