unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, Menus, ArtCam, AppEvnts, ExtCtrls;

type
  TForm1 = class(TForm)
    MainMenu1: TMainMenu;
    PopupFile: TMenuItem;
    PopupShow: TMenuItem;
    PopupSet: TMenuItem;
    PopupDll: TMenuItem;
    FileSave: TMenuItem;
    FileExit: TMenuItem;
    ShowPreview: TMenuItem;
    ShowCallBack: TMenuItem;
    ShowSnapShot: TMenuItem;
    ShowCapture: TMenuItem;
    ShowTrigger: TMenuItem;
    SetCamera1: TMenuItem;
    SetCamera2: TMenuItem;
    SetFilter1: TMenuItem;
    SetFilter2: TMenuItem;
    SetAnalog1: TMenuItem;
    SetAnalog2: TMenuItem;
    DIRECTSHOW: TMenuItem;
    ARTUST: TMenuItem;
    ARTCNV: TMenuItem;
    ARTCAM130MI: TMenuItem;
    ARTCAM200MI: TMenuItem;
    ARTCAM300MI: TMenuItem;
    ARTCAM150P: TMenuItem;
    ARTCAM320P: TMenuItem;
    ARTCAM200SH: TMenuItem;
    ARTCAM098: TMenuItem;
    ARTCAM036MI: TMenuItem;
    ARTCAM500P: TMenuItem;
    ARTCAM150P2: TMenuItem;
    Timer1: TTimer;
    N1: TMenuItem;
    N2: TMenuItem;
    Image1: TImage;
    Image2: TImage;

    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SaveClick(Sender: TObject);
    procedure ExitClick(Sender: TObject);
    procedure PreviewClick(Sender: TObject);
    procedure CallBackClick(Sender: TObject);
    procedure SnapShotClick(Sender: TObject);
    procedure CaptureClick(Sender: TObject);
    procedure TriggerClick(Sender: TObject);
    procedure SetCameraClick1(Sender: TObject);
    procedure SetCameraClick2(Sender: TObject);
    procedure SetFilterClick1(Sender: TObject);
    procedure SetFilterClick2(Sender: TObject);
    procedure SetAnalogClick1(Sender: TObject);
    procedure SetAnalogClick2(Sender: TObject);
    procedure DSCameraClick(Sender: TObject);
    procedure ARTUSTClick(Sender: TObject);
    procedure ARTCNVClick(Sender: TObject);
    procedure ARTCAM130MIClick(Sender: TObject);
    procedure ARTCAM200MIClick(Sender: TObject);
    procedure ARTCAM300MIClick(Sender: TObject);
    procedure ARTCAM150PClick(Sender: TObject);
    procedure ARTCAM320PClick(Sender: TObject);
    procedure ARTCAM200SHClick(Sender: TObject);
    procedure ARTCAM098Click(Sender: TObject);
    procedure ARTCAM036MIClick(Sender: TObject);
    procedure ARTCAM500PClick(Sender: TObject);
    procedure ARTCAM150P2Click(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    { Private declaration }
  public
    procedure CreateBitmap;
    procedure DrawImage;
    procedure DllChange(Sender: TObject; Number: Integer);
    function ArtCamGetWidth: Integer;
    function ArtCamGetHeight: Integer;
    function ArtCamGetColor: Integer;
    function ArtCamGetBpp: Integer;
    function ArtCamGetSize: Integer;
  end;

  function MsgProc(code: integer; wparam: WPARAM; lparam: LPARAM): LRESULT; stdcall;

var
  Form1: TForm1;
  BmpDraw: array[0..1] of TBitmap;
  ArtCam1: array[0..1] of TArtCam;
  Hook: HHOOK;
  PreviewMode: Integer;

const
  SUB_SAMPLE: array[0..3] of Integer = (1, 2, 4, 8);


implementation

{$R *.dfm}


//////////////////////////////////////////////////
// Receive capture message
//////////////////////////////////////////////////
function MsgProc(code: integer; wparam: WPARAM; lparam: LPARAM): LRESULT; stdcall;
var
  Msg: PMsg;
begin
  if code < 0 then begin
    result := CallNextHookEx(Hook, code, wparam, lparam);
    exit;
  end;

  if code = HC_ACTION then begin
    Msg := PMsg(lparam);

    case Msg.message of
      // Display code. Sent from SDK when the new image is captured
      UINT(WM_GRAPHPAINT): begin
        Form1.DrawImage;
      end;

      // Error Code
      UINT(WM_ERROR): begin
        case ARTCAMSDK_ERROR(Msg.lParam) of
          ARTCAMSDK_NOERROR:        Form1.Caption := 'Sample';
          ARTCAMSDK_NOT_INITIALIZE: Form1.Caption := 'Sample Not initialized';
          ARTCAMSDK_DISABLEDDEVICE: Form1.Caption := 'Sample Access to unavailable device was attempted';
          ARTCAMSDK_CREATETHREAD:   Form1.Caption := 'Sample Failure of thread creation for capturing';
          ARTCAMSDK_CREATEWINDOW:   Form1.Caption := 'Sample Window creation failed';
          ARTCAMSDK_OUTOFMEMORY:    Form1.Caption := 'Sample Not enough memory for image transfer/Failure for securing memory';
          ARTCAMSDK_CAMERASET:      Form1.Caption := 'Sample Error for camera (device) settings';
          ARTCAMSDK_CAPTURE:        Form1.Caption := 'Sample Capturing failed';
          ARTCAMSDK_PARAM:          Form1.Caption := 'Sample Wrong argument';
          ARTCAMSDK_DIRECTSHOW:     Form1.Caption := 'Sample Directshow initialization error';
          ARTCAMSDK_UNSUPPORTED:    Form1.Caption := 'Sample Not supported';
          ARTCAMSDK_UNKNOWN:        Form1.Caption := 'Sample Unknow error';
          ARTCAMSDK_CAPTURELOST:    Form1.Caption := 'Sample Device lost';
        end;
      end;
    end;
  end;

  result := CallNextHookEx(Hook, code, wparam, lparam);
end;


//////////////////////////////////////////////////
// Initialization of camera
//////////////////////////////////////////////////
procedure TForm1.FormCreate(Sender: TObject);
begin
  // Build class
  ArtCam1[0] := TArtCam.Create;
  ArtCam1[1] := TArtCam.Create;

  // Receive DLL message by hook
  Hook := SetWindowsHookEx(WH_GETMESSAGE, MsgProc, 0, GetCurrentThreadID);

  PreviewMode := 1;
  Timer1.Enabled := false;
  DllChange(Sender, 5);
end;


//////////////////////////////////////////////////
// Release camera
//////////////////////////////////////////////////
procedure TForm1.FormDestroy(Sender: TObject);
begin
  ArtCam1[0].Release();
  ArtCam1[1].Release();

  UnHookWindowsHookEx(Hook);
end;


//////////////////////////////////////////////////
// Save image
//////////////////////////////////////////////////
procedure TForm1.SaveClick(Sender: TObject);
begin
  if not ArtCam1[0].IsInit then begin ShowMessage('Select DLL of camera which is connected'); exit; end;
  ArtCam1[0].SaveImage('Image1.png', FILETYPE_PNG);
  ArtCam1[1].SaveImage('Image2.png', FILETYPE_PNG);
end;


//////////////////////////////////////////////////
// End
//////////////////////////////////////////////////
procedure TForm1.ExitClick(Sender: TObject);
begin
  CLose;
end;


//////////////////////////////////////////////////
// Preview
//////////////////////////////////////////////////
procedure TForm1.PreviewClick(Sender: TObject);
var
  rc:  TRect;
begin
  if not ArtCam1[0].IsInit then begin ShowMessage('Select DLL of camera which is connected'); exit; end;
  Timer1.Enabled := false;

  rc := GetClientRect();

  ArtCam1[0].Close;
  ArtCam1[1].Close;
  ArtCam1[0].SetPreviewWindow(Form1.Handle, 0, 0, rc.Right div 2, rc.Bottom);
  ArtCam1[1].SetPreviewWindow(Form1.Handle, rc.Right div 2, 0, rc.Right, rc.Bottom);
  ArtCam1[0].Preview;
  ArtCam1[1].Preview;

  // Check menu
  ShowPreview.Checked := true;
  ShowCallBack.Checked := false;
  ShowCapture.Checked := false;
  ShowTrigger.Checked := false;

  Image1.Hide;
  Image2.Hide;

  Invalidate;
  PreviewMode := 0;
end;


//////////////////////////////////////////////////
// Callback
//////////////////////////////////////////////////
procedure TForm1.CallBackClick(Sender: TObject);
begin
  if not ArtCam1[0].IsInit then begin ShowMessage('Select DLL of camera which is connected'); exit; end;
  Timer1.Enabled := false;

  ArtCam1[0].Close;
  ArtCam1[1].Close;
  CreateBitmap;
  ArtCam1[0].SetPreviewWindow(Form1.Handle, 0, 0, 0, 0);
  ArtCam1[1].SetPreviewWindow(Form1.Handle, 0, 0, 0, 0);
  ArtCam1[0].CallBackPreview(Form1.Handle, BmpDraw[0].ScanLine[BmpDraw[0].Height-1], ArtCamGetSize, FALSE);
  ArtCam1[1].CallBackPreview(Form1.Handle, BmpDraw[1].ScanLine[BmpDraw[1].Height-1], ArtCamGetSize, FALSE);

  // Check menu
  ShowPreview.Checked := false;
  ShowCallBack.Checked := true;
  ShowCapture.Checked := false;
  ShowTrigger.Checked := false;

  Image1.Show;
  Image2.Show;

  Invalidate;
  PreviewMode := 1;
end;


//////////////////////////////////////////////////
// Snapshot
//////////////////////////////////////////////////
procedure TForm1.SnapShotClick(Sender: TObject);
begin
  if not ArtCam1[0].IsInit then begin ShowMessage('Select DLL of camera which is connected'); exit; end;
  Timer1.Enabled := false;

  ArtCam1[0].Close;
  ArtCam1[1].Close;
  CreateBitmap;
  ArtCam1[0].SnapShot(BmpDraw[0].ScanLine[BmpDraw[0].Height-1], ArtCamGetSize, FALSE);
  ArtCam1[1].SnapShot(BmpDraw[1].ScanLine[BmpDraw[1].Height-1], ArtCamGetSize, FALSE);
  DrawImage;

  // Check menu
  ShowPreview.Checked := false;
  ShowCallBack.Checked := false;
  ShowCapture.Checked := false;
  ShowTrigger.Checked := false;

  Image1.Show;
  Image2.Show;

//Invalidate;
  PreviewMode := -1;
end;


//////////////////////////////////////////////////
// Capture
//////////////////////////////////////////////////
procedure TForm1.CaptureClick(Sender: TObject);
begin
  if not ArtCam1[0].IsInit then begin ShowMessage('Select DLL of camera which is connected'); exit; end;
  Timer1.Enabled := false;

  ArtCam1[0].Close;
  ArtCam1[1].Close;
  CreateBitmap;
  ArtCam1[0].Capture;
  ArtCam1[1].Capture;

  Timer1.Interval := 200;
  Timer1.Enabled := true;

  // Check menu
  ShowPreview.Checked := false;
  ShowCallBack.Checked := false;
  ShowCapture.Checked := true;
  ShowTrigger.Checked := false;

  Image1.Show;
  Image2.Show;

  Invalidate;
  PreviewMode := 3;
end;


//////////////////////////////////////////////////
// Trigger
//////////////////////////////////////////////////
procedure TForm1.TriggerClick(Sender: TObject);
begin
  if not ArtCam1[0].IsInit then begin ShowMessage('Select DLL of camera which is connected'); exit; end;
  Timer1.Enabled := false;

  ArtCam1[0].Close;
  ArtCam1[1].Close;
  CreateBitmap;
  ArtCam1[0].SetPreviewWindow(Form1.Handle, 0, 0, 0, 0);
  ArtCam1[1].SetPreviewWindow(Form1.Handle, 0, 0, 0, 0);
  ArtCam1[0].Trigger(Form1.Handle, BmpDraw[0].ScanLine[BmpDraw[0].Height-1], ArtCamGetSize, FALSE);
  ArtCam1[1].Trigger(Form1.Handle, BmpDraw[1].ScanLine[BmpDraw[1].Height-1], ArtCamGetSize, FALSE);

  // Check menu
  ShowPreview.Checked := false;
  ShowCallBack.Checked := false;
  ShowCapture.Checked := false;
  ShowTrigger.Checked := true;

  Image1.Show;
  Image2.Show;

  Invalidate;
  PreviewMode := 4;
end;


//////////////////////////////////////////////////
// Create bit-map
//////////////////////////////////////////////////
procedure TForm1.CreateBitmap;
var
  pal: TMaxLogPalette;
  i: Integer;
begin
  // Space for display
  BmpDraw[0] := TBitmap.Create;
  BmpDraw[0].Width := ArtCamGetWidth;
  BmpDraw[0].Height := ArtCamGetHeight;
  BmpDraw[1] := TBitmap.Create;
  BmpDraw[1].Width := ArtCamGetWidth;
  BmpDraw[1].Height := ArtCamGetHeight;

  case ArtCamGetBpp of
    1: begin
      BmpDraw[0].PixelFormat := pf8bit;
      BmpDraw[1].PixelFormat := pf8bit;

      // In case of 8 bit monochro, create pallet.
      for i := 0 to 255 do begin
        pal.palPalEntry[i].peRed   := i;
        pal.palPalEntry[i].peGreen := i;
        pal.palPalEntry[i].peBlue  := i;
        pal.palPalEntry[i].peFlags := 0;
      end;

      pal.palNumEntries := 256;
      pal.palVersion := $0300;
      BmpDraw[0].Palette := CreatePalette(PLogPalette(@pal)^);
      BmpDraw[1].Palette := CreatePalette(PLogPalette(@pal)^);
    end;

    // 24 bit color
    3: begin
      BmpDraw[0].PixelFormat := pf24bit;
      BmpDraw[1].PixelFormat := pf24bit;
    end;
  end;

  // To use other color number,
  // Sample_Delphi7_Graphic&Allbpp
  // please refer to **

end;


//////////////////////////////////////////////////
// Display
//////////////////////////////////////////////////
procedure TForm1.DrawImage;
begin
  if BmpDraw[0]=nil then begin exit; end;
  if BmpDraw[0].Empty then begin exit; end;

  // Draw image
  Image1.Canvas.Draw(0, 0, BmpDraw[0]);
  Image2.Canvas.Draw(0, 0, BmpDraw[1]);
end;


//////////////////////////////////////////////////
// Obtain camera size etc.
//////////////////////////////////////////////////
function TForm1.ArtCamGetWidth: Integer;
begin
  result := ArtCam1[0].Width div SUB_SAMPLE[LONG(ArtCam1[0].GetSubSample)];
end;

function TForm1.ArtCamGetHeight: Integer;
begin
  result := ArtCam1[0].Height div SUB_SAMPLE[LONG(ArtCam1[0].GetSubSample)];
end;

function TForm1.ArtCamGetColor: Integer;
begin
  result := ((ArtCam1[0].GetColorMode + 7) div 8) * 8;
end;

function TForm1.ArtCamGetBpp: Integer;
begin
  result := ((ArtCam1[0].GetColorMode + 7) div 8);
end;

function TForm1.ArtCamGetSize: Integer;
begin
  result := ArtCamGetWidth * ArtCamGetHeight * ArtCamGetBpp;
end;


//////////////////////////////////////////////////
// Camera settings
//////////////////////////////////////////////////
procedure TForm1.SetCameraClick1(Sender: TObject);
begin
  if not ArtCam1[0].IsInit then begin ShowMessage('Select DLL of camera which is connected'); exit; end;
  ArtCam1[0].SetCameraDlg(Form1.Handle);

  case PreviewMode of
  0: PreviewClick(Sender);
  1: CallBackClick(Sender);
  3: CaptureClick(Sender);
  4: TriggerClick(Sender);
  end;
end;

procedure TForm1.SetCameraClick2(Sender: TObject);
begin
  if not ArtCam1[1].IsInit then begin ShowMessage('Select DLL of camera which is connected'); exit; end;
  ArtCam1[1].SetCameraDlg(Form1.Handle);

  case PreviewMode of
  0: PreviewClick(Sender);
  1: CallBackClick(Sender);
  3: CaptureClick(Sender);
  4: TriggerClick(Sender);
  end;
end;


//////////////////////////////////////////////////
// Filter settings
//////////////////////////////////////////////////
procedure TForm1.SetFilterClick1(Sender: TObject);
begin
  if not ArtCam1[0].IsInit then begin ShowMessage('Select DLL of camera which is connected'); exit; end;
  ArtCam1[0].SetImageDlg(Form1.Handle);
end;

procedure TForm1.SetFilterClick2(Sender: TObject);
begin
  if not ArtCam1[1].IsInit then begin ShowMessage('Select DLL of camera which is connected'); exit; end;
  ArtCam1[1].SetImageDlg(Form1.Handle);
end;


//////////////////////////////////////////////////
// Analog settings
//////////////////////////////////////////////////
procedure TForm1.SetAnalogClick1(Sender: TObject);
begin
  if not ArtCam1[0].IsInit then begin ShowMessage('Select DLL of camera which is connected'); exit; end;
  ArtCam1[0].SetAnalogDlg(Form1.Handle);
end;

procedure TForm1.SetAnalogClick2(Sender: TObject);
begin
  if not ArtCam1[1].IsInit then begin ShowMessage('Select DLL of camera which is connected'); exit; end;
  ArtCam1[1].SetAnalogDlg(Form1.Handle);
end;


//////////////////////////////////////////////////
// Change DLL
//////////////////////////////////////////////////
procedure TForm1.DllChange(Sender: TObject; Number: Integer);
const
  DllNameList: array[0..14, 0..63] of AnsiChar = (
      'ArtCamSdk.dll',
      '',
      'ArtCamSdk_USTC.dll',
      'ArtCnvSdk.dll',
      '',
      'ArtCamSdk_130MI.dll',
      'ArtCamSdk_200MI.dll',
      'ArtCamSdk_300MI.dll',
      'ArtCamSdk_150P.dll',
      'ArtCamSdk_320P.dll',
      'ArtCamSdk_200SH.dll',
      'ArtCamSdk_098.dll',
      'ArtCamSdk_036MI.dll',
      'ArtCamSdk_500P.dll',
      'ArtCamSdk_150P2.dll'
    );

var
  i: Integer;
  Temp: THandle;
begin
  // Release all once
  if ArtCam1[0].IsInit then begin
    ArtCam1[0].Release;
    Timer1.Enabled := false;
  end;

  if ArtCam1[1].IsInit then begin
    ArtCam1[1].Release;
  end;


  // Check availability of DLL 
  for i := 0 to 14 do begin
    Temp := Windows.LoadLibrary(DllNameList[i]);
    if Temp<>0 then begin
      Windows.FreeLibrary(Temp);
      PopupDll[i].Enabled := true;
    end
    else begin
      PopupDll[i].Enabled := false;
    end;

    PopupDll[i].Checked := false;
  end;


  // Read DLL
  if ArtCam1[0].LoadLibrary(DllNameList[LONG(Number)]) then
    // With DLL
    begin
      ArtCam1[1].LoadLibrary(DllNameList[LONG(Number)]);

      ArtCam1[0].Initialize(Form1.Handle);
      ArtCam1[1].Initialize(Form1.Handle);

      ArtCam1[0].SetDeviceNumber(0);
      ArtCam1[1].SetDeviceNumber(1);

      // To operate 2 cameras simultenously, set camera clock to half.
      ArtCam1[0].SetHalfClock(1);
      ArtCam1[1].SetHalfClock(1);


      case PreviewMode of
      0: PreviewClick(Sender);
      1: CallBackClick(Sender);
      3: CaptureClick(Sender);
      4: TriggerClick(Sender);
      end;

    end;
  end;


//////////////////////////////////////////////////
// Select DLL
//////////////////////////////////////////////////
procedure TForm1.DSCameraClick   (Sender: TObject); begin DllChange(Sender, 0);  end;
procedure TForm1.ARTUSTClick     (Sender: TObject); begin DllChange(Sender, 2);  end;
procedure TForm1.ARTCNVClick     (Sender: TObject); begin DllChange(Sender, 3);  end;
procedure TForm1.ARTCAM130MIClick(Sender: TObject); begin DllChange(Sender, 5);  end;
procedure TForm1.ARTCAM200MIClick(Sender: TObject); begin DllChange(Sender, 6);  end;
procedure TForm1.ARTCAM300MIClick(Sender: TObject); begin DllChange(Sender, 7);  end;
procedure TForm1.ARTCAM150PClick (Sender: TObject); begin DllChange(Sender, 8);  end;
procedure TForm1.ARTCAM320PClick (Sender: TObject); begin DllChange(Sender, 9);  end;
procedure TForm1.ARTCAM200SHClick(Sender: TObject); begin DllChange(Sender, 10); end;
procedure TForm1.ARTCAM098Click  (Sender: TObject); begin DllChange(Sender, 11); end;
procedure TForm1.ARTCAM036MIClick(Sender: TObject); begin DllChange(Sender, 12); end;
procedure TForm1.ARTCAM500PClick (Sender: TObject); begin DllChange(Sender, 13); end;
procedure TForm1.ARTCAM150P2Click(Sender: TObject); begin DllChange(Sender, 14); end;


//////////////////////////////////////////////////
// For capture
//////////////////////////////////////////////////
procedure TForm1.Timer1Timer(Sender: TObject);
begin
  ArtCam1[0].SnapShot(BmpDraw[0].ScanLine[ArtCam1[0].Height-1], ArtCamGetSize, FALSE);
  ArtCam1[1].SnapShot(BmpDraw[1].ScanLine[ArtCam1[1].Height-1], ArtCamGetSize, FALSE);
  DrawImage;
end;


//////////////////////////////////////////////////
// Change window size                                       *
//////////////////////////////////////////////////
procedure TForm1.FormResize(Sender: TObject);
var
  Child1: HWND;
  Child2: HWND;
  rc:  TRect;
begin
  // In Preview Mode, display size varies when size of child window is changed.
  if PreviewMode = 0 then begin
    Child1 := GetWindow(Form1.Handle, GW_CHILD);
    Child2 := GetWindow(Child1, GW_HWNDNEXT);
    rc := GetClientRect();

    SetWindowPos(Child1, HWND_TOP, 0, 0, rc.Right div 2, rc.Bottom, SWP_NOMOVE or SWP_NOZORDER or SWP_SHOWWINDOW);
    SetWindowPos(Child2, HWND_TOP, rc.Right div 2, 0, rc.Right div 2, rc.Bottom, SWP_NOZORDER or SWP_SHOWWINDOW);
  end;
end;

end.

