unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, Menus, ArtCam, AppEvnts, ExtCtrls;

type
  TForm1 = class(TForm)
    MainMenu1: TMainMenu;
    PopupFile: TMenuItem;
    PopupShow: TMenuItem;
    PopupSet: TMenuItem;
    PopupDevice: TMenuItem;
    PopupDll: TMenuItem;
    FileSave: TMenuItem;
    FileExit: TMenuItem;
    ShowPreview: TMenuItem;
    ShowCallBack: TMenuItem;
    ShowSnapShot: TMenuItem;
    ShowCapture: TMenuItem;
    ShowTrigger: TMenuItem;
    SetCamera: TMenuItem;
    SetFilter: TMenuItem;
    SetAnalog: TMenuItem;
    Device0: TMenuItem;
    Device1: TMenuItem;
    Device2: TMenuItem;
    Device3: TMenuItem;
    Device4: TMenuItem;
    Device5: TMenuItem;
    Device6: TMenuItem;
    Device7: TMenuItem;
    Device8: TMenuItem;
    Device9: TMenuItem;
    DIRECTSHOW: TMenuItem;
    ARTUST: TMenuItem;
    ARTCNV: TMenuItem;
    ARTCAM130MI: TMenuItem;
    ARTCAM200MI: TMenuItem;
    ARTCAM300MI: TMenuItem;
    ARTCAM150P: TMenuItem;
    ARTCAM320P: TMenuItem;
    ARTCAM200SH: TMenuItem;
    ARTCAM098: TMenuItem;
    ARTCAM036MI: TMenuItem;
    ARTCAM500P: TMenuItem;
    ARTCAM150P2: TMenuItem;
    Timer1: TTimer;
    N1: TMenuItem;
    N2: TMenuItem;

    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SaveClick(Sender: TObject);
    procedure ExitClick(Sender: TObject);
    procedure PreviewClick(Sender: TObject);
    procedure CallBackClick(Sender: TObject);
    procedure SnapShotClick(Sender: TObject);
    procedure CaptureClick(Sender: TObject);
    procedure TriggerClick(Sender: TObject);
    procedure SetCameraClick(Sender: TObject);
    procedure SetFilterClick(Sender: TObject);
    procedure SetAnalogClick(Sender: TObject);
    procedure Device0Click(Sender: TObject);
    procedure Device1Click(Sender: TObject);
    procedure Device2Click(Sender: TObject);
    procedure Device3Click(Sender: TObject);
    procedure Device4Click(Sender: TObject);
    procedure Device5Click(Sender: TObject);
    procedure Device6Click(Sender: TObject);
    procedure Device7Click(Sender: TObject);
    procedure Device8Click(Sender: TObject);
    procedure Device9Click(Sender: TObject);
    procedure DSCameraClick(Sender: TObject);
    procedure ARTUSTClick(Sender: TObject);
    procedure ARTCNVClick(Sender: TObject);
    procedure ARTCAM130MIClick(Sender: TObject);
    procedure ARTCAM200MIClick(Sender: TObject);
    procedure ARTCAM300MIClick(Sender: TObject);
    procedure ARTCAM150PClick(Sender: TObject);
    procedure ARTCAM320PClick(Sender: TObject);
    procedure ARTCAM200SHClick(Sender: TObject);
    procedure ARTCAM098Click(Sender: TObject);
    procedure ARTCAM036MIClick(Sender: TObject);
    procedure ARTCAM500PClick(Sender: TObject);
    procedure ARTCAM150P2Click(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    { Private declaration }
  public
    procedure CreateBitmap;
    procedure DrawImage;
    procedure DeviceChange(Sender: TObject; Number: Integer);
    procedure DllChange(Sender: TObject; Number: Integer);
    function ArtCamGetWidth: Integer;
    function ArtCamGetHeight: Integer;
    function ArtCamGetColor: Integer;
    function ArtCamGetBpp: Integer;
    function ArtCamGetSize: Integer;
  end;

  function MsgProc(code: integer; wparam: WPARAM; lparam: LPARAM): LRESULT; stdcall;

var
  Form1: TForm1;
  BmpDraw: TBitmap;
  BmpCapture: TBitmap;
  ArtCam1: TArtCam;
  Hook: HHOOK;
  PreviewMode: Integer;

const
  SUB_SAMPLE: array[0..3] of Integer = (1, 2, 4, 8);


implementation

{$R *.dfm}


//////////////////////////////////////////////////
// Receive capture message
//////////////////////////////////////////////////
function MsgProc(code: integer; wparam: WPARAM; lparam: LPARAM): LRESULT; stdcall;
var
  Msg: PMsg;
begin
  if code < 0 then begin
    result := CallNextHookEx(Hook, code, wparam, lparam);
    exit;
  end;

  if code = HC_ACTION then begin
    Msg := PMsg(lparam);

    case Msg.message of
      // Display code. Sent from SDK when the new image is captured
      UINT(WM_GRAPHPAINT): begin
        Form1.DrawImage;
      end;

      // Error Code
      UINT(WM_ERROR): begin
        case ARTCAMSDK_ERROR(Msg.lParam) of
          ARTCAMSDK_NOERROR:        Form1.Caption := 'Sample';
          ARTCAMSDK_NOT_INITIALIZE: Form1.Caption := 'Sample Not initialized';
          ARTCAMSDK_DISABLEDDEVICE: Form1.Caption := 'Sample Access to unavailable device was attempted';
          ARTCAMSDK_CREATETHREAD:   Form1.Caption := 'Sample Failure of thread creation for capturing';
          ARTCAMSDK_CREATEWINDOW:   Form1.Caption := 'Sample Window creation failed';
          ARTCAMSDK_OUTOFMEMORY:    Form1.Caption := 'Sample Not enough memory for image transfer/Failure for securing memory';
          ARTCAMSDK_CAMERASET:      Form1.Caption := 'Sample Error for camera (device) settings';
          ARTCAMSDK_CAPTURE:        Form1.Caption := 'Sample Capturing failed';
          ARTCAMSDK_PARAM:          Form1.Caption := 'Sample Wrong argument';
          ARTCAMSDK_DIRECTSHOW:     Form1.Caption := 'Sample Directshow initialization error';
          ARTCAMSDK_UNSUPPORTED:    Form1.Caption := 'Sample Not supported';
          ARTCAMSDK_UNKNOWN:        Form1.Caption := 'Sample Unknow error';
          ARTCAMSDK_CAPTURELOST:    Form1.Caption := 'Sample Device lost';
        end;
      end;
    end;
  end;

  result := CallNextHookEx(Hook, code, wparam, lparam);
end;


//////////////////////////////////////////////////
// Initialization of camera
//////////////////////////////////////////////////
procedure TForm1.FormCreate(Sender: TObject);
begin
  // Build class
  ArtCam1 := TArtCam.Create;

  // Receive DLL message by hook
  Hook := SetWindowsHookEx(WH_GETMESSAGE, MsgProc, 0, GetCurrentThreadID);

  PreviewMode := 1;
  Timer1.Enabled := false;
  DllChange(Sender, 5);
end;


//////////////////////////////////////////////////
// Release camera
//////////////////////////////////////////////////
procedure TForm1.FormDestroy(Sender: TObject);
begin
  ArtCam1.Release();

  UnHookWindowsHookEx(Hook);
end;


//////////////////////////////////////////////////
// Save image
//////////////////////////////////////////////////
procedure TForm1.SaveClick(Sender: TObject);
begin
  if not ArtCam1.IsInit then begin ShowMessage('Select DLL of camera which is connected'); exit; end;
  ArtCam1.SaveImage('image.png', FILETYPE_PNG);
end;


//////////////////////////////////////////////////
// End
//////////////////////////////////////////////////
procedure TForm1.ExitClick(Sender: TObject);
begin
  CLose;
end;


//////////////////////////////////////////////////
// Preview
//////////////////////////////////////////////////
procedure TForm1.PreviewClick(Sender: TObject);
begin
  if not ArtCam1.IsInit then begin ShowMessage('Select DLL of camera which is connected'); exit; end;
  Timer1.Enabled := false;

  ArtCam1.Close;
  ArtCam1.SetPreviewWindow(Form1.Handle, 0, 0, Form1.Width, Form1.Height);
  ArtCam1.Preview;

  // Check menu
  ShowPreview.Checked := true;
  ShowCallBack.Checked := false;
  ShowCapture.Checked := false;
  ShowTrigger.Checked := false;

  Invalidate;
  PreviewMode := 0;
end;


//////////////////////////////////////////////////
// Callback
//////////////////////////////////////////////////
procedure TForm1.CallBackClick(Sender: TObject);
begin
  if not ArtCam1.IsInit then begin ShowMessage('Select DLL of camera which is connected'); exit; end;
  Timer1.Enabled := false;

  ArtCam1.Close;
  CreateBitmap;
  ArtCam1.SetPreviewWindow(Form1.Handle, 0, 0, 0, 0);
  ArtCam1.CallBackPreview(Form1.Handle, BmpCapture.ScanLine[BmpDraw.Height-1], ArtCamGetSize, FALSE);

  // Check menu
  ShowPreview.Checked := false;
  ShowCallBack.Checked := true;
  ShowCapture.Checked := false;
  ShowTrigger.Checked := false;

  Invalidate;
  PreviewMode := 1;
end;


//////////////////////////////////////////////////
// Snapshot
//////////////////////////////////////////////////
procedure TForm1.SnapShotClick(Sender: TObject);
begin
  if not ArtCam1.IsInit then begin ShowMessage('Select DLL of camera which is connected'); exit; end;
  Timer1.Enabled := false;

  ArtCam1.Close;
  CreateBitmap;
  ArtCam1.SnapShot(BmpCapture.ScanLine[BmpDraw.Height-1], ArtCamGetSize, FALSE);
  DrawImage;

  // Check menu
  ShowPreview.Checked := false;
  ShowCallBack.Checked := false;
  ShowCapture.Checked := false;
  ShowTrigger.Checked := false;

//Invalidate;
  PreviewMode := -1;
end;


//////////////////////////////////////////////////
// Capture
//////////////////////////////////////////////////
procedure TForm1.CaptureClick(Sender: TObject);
begin
  if not ArtCam1.IsInit then begin ShowMessage('Select DLL of camera which is connected'); exit; end;
  Timer1.Enabled := false;

  ArtCam1.Close;
  CreateBitmap;
  ArtCam1.Capture;

  Timer1.Interval := 200;
  Timer1.Enabled := true;

  // Check menu
  ShowPreview.Checked := false;
  ShowCallBack.Checked := false;
  ShowCapture.Checked := true;
  ShowTrigger.Checked := false;

  Invalidate;
  PreviewMode := 3;
end;


//////////////////////////////////////////////////
// Trigger
//////////////////////////////////////////////////
procedure TForm1.TriggerClick(Sender: TObject);
begin
  if not ArtCam1.IsInit then begin ShowMessage('Select DLL of camera which is connected'); exit; end;
  Timer1.Enabled := false;

  ArtCam1.Close;
  CreateBitmap;
  ArtCam1.SetPreviewWindow(Form1.Handle, 0, 0, 0, 0);
  ArtCam1.Trigger(Form1.Handle, BmpCapture.ScanLine[BmpDraw.Height-1], ArtCamGetSize, FALSE);

  // Check menu
  ShowPreview.Checked := false;
  ShowCallBack.Checked := false;
  ShowCapture.Checked := false;
  ShowTrigger.Checked := true;

  Invalidate;
  PreviewMode := 4;
end;


//////////////////////////////////////////////////
// Create bit-map
//////////////////////////////////////////////////
procedure TForm1.CreateBitmap;
var
  pal: TMaxLogPalette;
  i: Integer;
begin
  // Space for display
  BmpDraw := TBitmap.Create;
  BmpDraw.Width := ArtCamGetWidth;
  BmpDraw.Height := ArtCamGetHeight;
  BmpDraw.PixelFormat := pf24bit;

  // Appoint red pen
  BmpDraw.Canvas.Pen.Color := clRed;

  // Space for taking in
  BmpCapture := TBitmap.Create;
  BmpCapture.Width := ArtCamGetWidth;
  BmpCapture.Height := ArtCamGetHeight;

  case ArtCamGetBpp of
    1: begin
      BmpCapture.PixelFormat := pf8bit;

      // In case of 8 bit monochro, create pallet.
      for i := 0 to 255 do begin
        pal.palPalEntry[i].peRed   := i;
        pal.palPalEntry[i].peGreen := i;
        pal.palPalEntry[i].peBlue  := i;
        pal.palPalEntry[i].peFlags := 0;
      end;

      pal.palNumEntries := 256;
      pal.palVersion := $0300;
      BmpCapture.Palette := CreatePalette(PLogPalette(@pal)^);
    end;

    2: begin BmpCapture.PixelFormat := pf16bit; end;
    3: begin BmpCapture.PixelFormat := pf24bit; end;
    4: begin BmpCapture.PixelFormat := pf32bit; end;

    6: begin
      // You cannot appoint pf48bit. So increase data volume by doubling horizontal size.
      // By aligning size, there will be no problem with alignment length of area for capturing.
      BmpCapture.Width := ArtCamGetWidth * 2;
      BmpCapture.Height := ArtCamGetHeight;
      BmpCapture.PixelFormat := pf24bit;
    end;

    8: begin
      // You cannot appoint pf64bit. So increase data volume by doubling horizontal size.
      // By aligning size, there will be no problem with alignment length of area for capturing.
      BmpCapture.Width := ArtCamGetWidth * 2;
      BmpCapture.Height := ArtCamGetHeight;
      BmpCapture.PixelFormat := pf32bit;
    end;
  end;

end;


//////////////////////////////////////////////////
// Display
//////////////////////////////////////////////////
procedure TForm1.DrawImage;
var
  pt: TPoint;
  x: Integer;
  y: Integer;
  dx: Integer;
  sx: Integer;
  pDest: PByteArray;
  pSrc : PByteArray;
  SrcBpp: Integer;
begin
  if BmpDraw=nil then begin exit; end;
  if BmpDraw.Empty then begin exit; end;

  SrcBpp := ArtCamGetBpp;


  // For capturing->Draw for display
  case SrcBpp of
    // Draw when bit calculation is not required
    1, 3, 4: begin
      BmpDraw.Canvas.Draw(0, 0, BmpCapture);
    end;

    // In 16 bit, shift calculation is required in the unit of pixel.
    2: begin
      for y := 0 to BmpDraw.Height-1 do begin
        pDest := BmpDraw.ScanLine[y];
        pSrc  := BmpCapture.ScanLine[y];
        dx := 0;
        sx := 0;

        for x := 0 to BmpDraw.Width-1 do begin
          pDest[dx  ] := (pSrc[sx+1] shl 6) or (pSrc[sx] shr 2);
          pDest[dx+1] := (pSrc[sx+1] shl 6) or (pSrc[sx] shr 2);
          pDest[dx+2] := (pSrc[sx+1] shl 6) or (pSrc[sx] shr 2);

          dx := dx + 3;
          sx := sx + SrcBpp;
        end;
      end;
    end;

    6, 8: begin
      for y := 0 to BmpDraw.Height-1 do begin
        pDest := BmpDraw.ScanLine[y];
        pSrc  := BmpCapture.ScanLine[y];
        dx := 0;
        sx := 0;

        for x := 0 to BmpDraw.Width-1 do begin
          pDest[dx  ] := (pSrc[sx+1] shl 6) or (pSrc[sx  ] shr 2);
          pDest[dx+1] := (pSrc[sx+3] shl 6) or (pSrc[sx+2] shr 2);
          pDest[dx+2] := (pSrc[sx+5] shl 6) or (pSrc[sx+4] shr 2);

          dx := dx + 3;
          sx := sx + SrcBpp;
        end;
      end;
    end;
  end;


  // Line drawing
  GetCursorPos(pt);
  pt := ScreenToClient(pt);

  // Draw the line to bitmap not to flicker
  BmpDraw.Canvas.MoveTo(0, pt.y);
  BmpDraw.Canvas.LineTo(BmpDraw.Width, pt.y);
  BmpDraw.Canvas.MoveTo(pt.x, 0);
  BmpDraw.Canvas.LineTo(pt.x, BmpDraw.Height);

  // Draw image
  Canvas.Draw(0, 0, BmpDraw);
end;


//////////////////////////////////////////////////
// Obtain camera size etc.
//////////////////////////////////////////////////
function TForm1.ArtCamGetWidth: Integer;
begin
  result := ArtCam1.Width div SUB_SAMPLE[LONG(ArtCam1.GetSubSample)];
end;

function TForm1.ArtCamGetHeight: Integer;
begin
  result := ArtCam1.Height div SUB_SAMPLE[LONG(ArtCam1.GetSubSample)];
end;

function TForm1.ArtCamGetColor: Integer;
begin
  result := ((ArtCam1.GetColorMode + 7) div 8) * 8;
end;

function TForm1.ArtCamGetBpp: Integer;
begin
  result := ((ArtCam1.GetColorMode + 7) div 8);
end;

function TForm1.ArtCamGetSize: Integer;
begin
  result := ArtCamGetWidth * ArtCamGetHeight * ArtCamGetBpp;
end;


//////////////////////////////////////////////////
// Camera settings
//////////////////////////////////////////////////
procedure TForm1.SetCameraClick(Sender: TObject);
begin
  if not ArtCam1.IsInit then begin ShowMessage('Select DLL of camera which is connected'); exit; end;
  ArtCam1.SetCameraDlg(Form1.Handle);

  case PreviewMode of
  0: PreviewClick(Sender);
  1: CallBackClick(Sender);
  3: CaptureClick(Sender);
  4: TriggerClick(Sender);
  end;
end;


//////////////////////////////////////////////////
// Filter settings
//////////////////////////////////////////////////
procedure TForm1.SetFilterClick(Sender: TObject);
begin
  if not ArtCam1.IsInit then begin ShowMessage('Select DLL of camera which is connected'); exit; end;
  ArtCam1.SetImageDlg(Form1.Handle);
end;


//////////////////////////////////////////////////
// Analog settings
//////////////////////////////////////////////////
procedure TForm1.SetAnalogClick(Sender: TObject);
begin
  if not ArtCam1.IsInit then begin ShowMessage('Select DLL of camera which is connected'); exit; end;
  ArtCam1.SetAnalogDlg(Form1.Handle);
end;


//////////////////////////////////////////////////
// Change device
//////////////////////////////////////////////////
procedure TForm1.DeviceChange(Sender: TObject; Number: Integer);
begin
  ArtCam1.Close;
  ArtCam1.SetDeviceNumber(Number);
  Timer1.Enabled := false;

  case PreviewMode of
  0: PreviewClick(Sender);
  1: CallBackClick(Sender);
  3: CaptureClick(Sender);
  4: TriggerClick(Sender);
  end;
end;


//////////////////////////////////////////////////
// Select device
//////////////////////////////////////////////////
procedure TForm1.Device0Click(Sender: TObject); begin DeviceChange(Sender, 0); end;
procedure TForm1.Device1Click(Sender: TObject); begin DeviceChange(Sender, 1); end;
procedure TForm1.Device2Click(Sender: TObject); begin DeviceChange(Sender, 2); end;
procedure TForm1.Device3Click(Sender: TObject); begin DeviceChange(Sender, 3); end;
procedure TForm1.Device4Click(Sender: TObject); begin DeviceChange(Sender, 4); end;
procedure TForm1.Device5Click(Sender: TObject); begin DeviceChange(Sender, 5); end;
procedure TForm1.Device6Click(Sender: TObject); begin DeviceChange(Sender, 6); end;
procedure TForm1.Device7Click(Sender: TObject); begin DeviceChange(Sender, 7); end;
procedure TForm1.Device8Click(Sender: TObject); begin DeviceChange(Sender, 8); end;
procedure TForm1.Device9Click(Sender: TObject); begin DeviceChange(Sender, 9); end;


//////////////////////////////////////////////////
// Change DLL
//////////////////////////////////////////////////
procedure TForm1.DllChange(Sender: TObject; Number: Integer);
const
  DllNameList: array[0..14, 0..63] of AnsiChar = (
      'ArtCamSdk.dll',
      '',
      'ArtCamSdk_USTC.dll',
      'ArtCnvSdk.dll',
      '',
      'ArtCamSdk_130MI.dll',
      'ArtCamSdk_200MI.dll',
      'ArtCamSdk_300MI.dll',
      'ArtCamSdk_150P.dll',
      'ArtCamSdk_320P.dll',
      'ArtCamSdk_200SH.dll',
      'ArtCamSdk_098.dll',
      'ArtCamSdk_036MI.dll',
      'ArtCamSdk_500P.dll',
      'ArtCamSdk_150P2.dll'
    );

var
  DevName: DeviceNameList;
  DevCount: LONG;
  i: Integer;
  Temp: THandle;
begin
  // Release all once
  if ArtCam1.IsInit then begin
    ArtCam1.Release;
    Timer1.Enabled := false;
  end;


  // Check availability of DLL
  for i := 0 to 14 do begin
    Temp := Windows.LoadLibrary(DllNameList[i]);
    if Temp<>0 then begin
      Windows.FreeLibrary(Temp);
      PopupDll[i].Enabled := true;
    end
    else begin
      PopupDll[i].Enabled := false;
    end;

    PopupDll[i].Checked := false;
  end;


  // Read DLL
  if ArtCam1.LoadLibrary(DllNameList[LONG(Number)]) then
    // With DLL
    begin
      PopupDll[LONG(Number)].Checked := true;

      ArtCam1.Initialize(Form1.Handle);
      DevCount := ArtCam1.EnumDevice(DevName);

      for i := 0 to DevCount-1 do begin
        ArtCam1.GetDeviceName(i, DevName[i], 256);
        PopupDevice[i].Caption := DevName[i];
        PopupDevice[i].Enabled := true;
      end;

      for i := DevCount to 9 do begin
        PopupDevice[i].Caption := IntToStr(i);
        PopupDevice[i].Enabled := false;
      end;

      case PreviewMode of
      0: PreviewClick(Sender);
      1: CallBackClick(Sender);
      3: CaptureClick(Sender);
      4: TriggerClick(Sender);
      end;

    end
  else
    // Without DLL
    begin
      for i := 0 to 9 do begin
        PopupDevice[i].Caption := IntToStr(i);
        PopupDevice[i].Enabled := false;
      end;
    end
end;


//////////////////////////////////////////////////
// Select DLL
//////////////////////////////////////////////////
procedure TForm1.DSCameraClick   (Sender: TObject); begin DllChange(Sender, 0);  end;
procedure TForm1.ARTUSTClick     (Sender: TObject); begin DllChange(Sender, 2);  end;
procedure TForm1.ARTCNVClick     (Sender: TObject); begin DllChange(Sender, 3);  end;
procedure TForm1.ARTCAM130MIClick(Sender: TObject); begin DllChange(Sender, 5);  end;
procedure TForm1.ARTCAM200MIClick(Sender: TObject); begin DllChange(Sender, 6);  end;
procedure TForm1.ARTCAM300MIClick(Sender: TObject); begin DllChange(Sender, 7);  end;
procedure TForm1.ARTCAM150PClick (Sender: TObject); begin DllChange(Sender, 8);  end;
procedure TForm1.ARTCAM320PClick (Sender: TObject); begin DllChange(Sender, 9);  end;
procedure TForm1.ARTCAM200SHClick(Sender: TObject); begin DllChange(Sender, 10); end;
procedure TForm1.ARTCAM098Click  (Sender: TObject); begin DllChange(Sender, 11); end;
procedure TForm1.ARTCAM036MIClick(Sender: TObject); begin DllChange(Sender, 12); end;
procedure TForm1.ARTCAM500PClick (Sender: TObject); begin DllChange(Sender, 13); end;
procedure TForm1.ARTCAM150P2Click(Sender: TObject); begin DllChange(Sender, 14); end;


//////////////////////////////////////////////////
// For capture
//////////////////////////////////////////////////
procedure TForm1.Timer1Timer(Sender: TObject);
begin
  ArtCam1.SnapShot(BmpCapture.ScanLine[ArtCam1.Height-1], ArtCamGetSize, FALSE);
  DrawImage;
end;


//////////////////////////////////////////////////
// Change window size
//////////////////////////////////////////////////
procedure TForm1.FormResize(Sender: TObject);
var
  Child: HWND;
  rc:  TRect;
{
  winW: Double;
  winH: Double;
  capW: Double;
  capH: Double;
}
begin
  // In Preview Mode, display size varies when size of child window is changed.
  if PreviewMode = 0 then begin
    Child := GetWindow(Form1.Handle, GW_CHILD);
    rc := GetClientRect();

    // To avoiding changing aspect ratio, remove comment
{
    // Use the real number once because calculation turns to the decimal fraction unit.
    winW := rc.Right;
    winH := rc.Bottom;
    capW := ArtCamGetWidth;
    capH := ArtCamGetHeight;

    if (winW * capH) > (winH * capW) then begin
      winW := winH * capW / capH;
      rc.Right := Trunc(winW);
    end
    else
    if (winW * capH) < (winH * capW) then begin
      winH := winW * capH / capW;
      rc.Bottom := Trunc(winH);
    end;
 }

    SetWindowPos(Child, HWND_TOP, 0, 0, rc.Right, rc.Bottom, SWP_NOMOVE or SWP_NOZORDER or SWP_SHOWWINDOW);
  end;
end;

end.

