//==========================================================
//	ARTRAY Camera / Capture Module Software Developer Kit
//						Preview/Callback function sample
//						VC++.NET Win32 Project
//
//									(C) 2002-2005 Artray
//==========================================================




//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Include file
//

#include <windows.h>
#include <stdio.h>

// Library include
#include "CArtCamSdk.hpp"



// Various menu ID 
enum {
	IDM_SAVE = WM_USER + 1,
	IDM_EXIT,

	IDM_PREVIEW,
	IDM_CALLBACK,
	IDM_SNAPSHOT,
	IDM_CAPTURE,
	IDM_TRIGGER,

	IDM_FORMAT1,
	IDM_FORMAT2,
	IDM_FILTER1,
	IDM_FILTER2,
	IDM_ANALOG1,
	IDM_ANALOG2,

	IDM_CAM,
	IDM_USTC,
	IDM_CNV,
	IDM_130MI,
	IDM_200MI,
	IDM_300MI,
	IDM_150P,
	IDM_320P,
	IDM_200SH,
	IDM_098,
	IDM_036MI,
	IDM_500P,
	IDM_150P2,
};



#define TIMER_ID	100
#define TIMER_TIME	50



CArtCamSdk	g_CArtCamSdk[2];
LPBYTE		g_pImage[2] = {NULL,NULL};		// Space for image data

BOOL		g_Preview = -1;			// Preview display/Callback display
LONG		g_DllType = -1;			// DLL to be connected
LONG		g_DeviceCount = 0;		// Number of detected device


const int SUB_SAMPLE[] = { 1, 2, 4, 8 };
#define ArtCamGetWidth(n)	(g_CArtCamSdk[n].Width()  / SUB_SAMPLE[g_CArtCamSdk[n].GetSubSample()])
#define ArtCamGetHeight(n)	(g_CArtCamSdk[n].Height() / SUB_SAMPLE[g_CArtCamSdk[n].GetSubSample()])
#define ArtCamGetColor(n)	WORD(((g_CArtCamSdk[n].GetColorMode() + 7) / 8) * 8)
#define ArtCamGetBpp(n)		(((g_CArtCamSdk[n].GetColorMode() + 7) / 8))
#define ArtCamGetPitch(n)	((ArtCamGetWidth(n) * ArtCamGetBpp(n) + 3) & ~3)
#define ArtCamGetSize(n)	(ArtCamGetPitch(n) * ArtCamGetHeight(n))
#define SAFE_DELETE(x)  { if(x) delete[] x; x=NULL; }


struct MENU_LIST{
	HMENU	File;
	HMENU	Preview;
	HMENU	Set;
	HMENU	Dll;
};

MENU_LIST g_Menu;



TCHAR g_DllName[][256] = {
	"ArtCamSdk.dll",
	"",
	"ArtCamSdk_USTC.dll",
	"ArtCnvSdk.dll",
	"",
	"ArtCamSdk_130MI.dll",
	"ArtCamSdk_200MI.dll",
	"ArtCamSdk_300MI.dll",
	"ArtCamSdk_150P.dll",
	"ArtCamSdk_320P.dll",
	"ArtCamSdk_200SH.dll",
	"ArtCamSdk_098.dll",
	"ArtCamSdk_036MI.dll",
	"ArtCamSdk_500P.dll",
	"ArtCamSdk_150P2.dll"
};

const int DLLCOUNT = sizeof(g_DllName)/256;
LONG DefProc = NULL;


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Draw image
//

struct BITMAPINFO2 {
	BITMAPINFOHEADER   bmiHeader;
	RGBQUAD            bmiColors[256];
};

LRESULT DrawImage(HWND hWnd, HDC hDC, LONG Number)
{
	LPBYTE pImage = g_pImage[Number];


	if(NULL == pImage){
		return 0;
	}

	BOOL Flg = FALSE;
	if(NULL == hDC){
		hDC = GetDC(hWnd);
		Flg = TRUE;
	}

	if(hDC){
		LONG Width = ArtCamGetWidth(Number);
		LONG Height = ArtCamGetHeight(Number);
		LONG Bpp = ArtCamGetBpp(Number);

		BITMAPINFO2 bm;
		bm.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
		bm.bmiHeader.biPlanes = 1;
		bm.bmiHeader.biCompression = BI_RGB;
		bm.bmiHeader.biSizeImage = 0;
		bm.bmiHeader.biXPelsPerMeter = 0;
		bm.bmiHeader.biYPelsPerMeter = 0;
		bm.bmiHeader.biClrUsed = 0;
		bm.bmiHeader.biClrImportant = 0;

		bm.bmiHeader.biWidth = Width;
		bm.bmiHeader.biHeight = Height;


		RECT rc;
		GetClientRect(hWnd, &rc);
		rc.right = rc.right < Width*2 ? rc.right : Width*2;

		rc.left   = 0 + (rc.right/2) * Number;
		rc.top    = 0;
		rc.right  = rc.right / 2;
		rc.bottom = Height;

		UINT Line = Height;



		// Monochrome 8 bit picture transfer
		if(1 == Bpp){
			bm.bmiHeader.biBitCount = 8;

			for(int i=0; i<256; i++){
				bm.bmiColors[i].rgbRed = i;
				bm.bmiColors[i].rgbGreen = i;
				bm.bmiColors[i].rgbBlue = i;
				bm.bmiColors[i].rgbReserved = 0;
			}
			::SetDIBColorTable(hDC, 0, 256, bm.bmiColors);

			SetDIBitsToDevice(hDC, rc.left, rc.top, rc.right, rc.bottom, 0, 0, 0, Line, pImage, (BITMAPINFO*)&bm, DIB_RGB_COLORS);
		}

		// Monochromatic 10 bit/16 bit picture transfer
		else if(2 == Bpp){
			bm.bmiHeader.biBitCount = 8;

			for(int i=0; i<256; i++){
				bm.bmiColors[i].rgbRed = i;
				bm.bmiColors[i].rgbGreen = i;
				bm.bmiColors[i].rgbBlue = i;
				bm.bmiColors[i].rgbReserved = 0;
			}
			::SetDIBColorTable(hDC, 0, 256, bm.bmiColors);

			LPBYTE pTemp = new BYTE[bm.bmiHeader.biWidth * bm.bmiHeader.biHeight];
			if(pTemp){
				for(int y=0; y<bm.bmiHeader.biHeight; y++){

					LPBYTE pDest =        pTemp   + (y * bm.bmiHeader.biWidth);
					LPWORD pSrc  = LPWORD(pImage) + (y * bm.bmiHeader.biWidth);

					for(int x=0; x<bm.bmiHeader.biWidth; x++){
						pDest[0] = BYTE(pSrc[0] >> 2);

						pDest++;
						pSrc ++;
					}
				}


				SetDIBitsToDevice(hDC, rc.left, rc.top, rc.right, rc.bottom, 0, 0, 0, Line, pTemp, (BITMAPINFO*)&bm, DIB_RGB_COLORS);
				delete[] pTemp;
			}
		}

		// Color 24 bit transfer
		else if(3 == Bpp){
			bm.bmiHeader.biBitCount = 24;
			SetDIBitsToDevice(hDC, rc.left, rc.top, rc.right, rc.bottom, 0, 0, 0, Line, pImage, (BITMAPINFO*)&bm, DIB_RGB_COLORS);
		}

		// Color 32 bit transfer
		else if(4 == Bpp){
			bm.bmiHeader.biBitCount = 24;

			LPBYTE pTemp = new BYTE[bm.bmiHeader.biWidth * bm.bmiHeader.biHeight * 3];
			if(pTemp){
				for(int y=0; y<bm.bmiHeader.biHeight; y++){

					LPBYTE pDest = pTemp    + (y * bm.bmiHeader.biWidth * 3);
					LPBYTE pSrc  = pImage + (y * bm.bmiHeader.biWidth * 4);

					for(int x=0; x<bm.bmiHeader.biWidth; x++){
						pDest[0] = pSrc[0];
						pDest[1] = pSrc[1];
						pDest[2] = pSrc[2];

						pDest+=3;
						pSrc +=4;
					}
				}


				SetDIBitsToDevice(hDC, rc.left, rc.top, rc.right, rc.bottom, 0, 0, 0, Line, pTemp, (BITMAPINFO*)&bm, DIB_RGB_COLORS);
				delete[] pTemp;
			}
		}

		// Color 48, 64 bit transfer
		else if(6 == Bpp || 8 == Bpp){
			bm.bmiHeader.biBitCount = 24;
			Bpp /= 2;

			LPBYTE pTemp = new BYTE[bm.bmiHeader.biWidth * bm.bmiHeader.biHeight * 3];
			if(pTemp){
				for(int y=0; y<bm.bmiHeader.biHeight; y++){

					LPBYTE pDest =        pTemp     + (y * bm.bmiHeader.biWidth * 3);
					LPWORD pSrc  = LPWORD(pImage) + (y * bm.bmiHeader.biWidth * Bpp);

					for(int x=0; x<bm.bmiHeader.biWidth; x++){
						pDest[0] = BYTE(pSrc[0] >> 2);
						pDest[1] = BYTE(pSrc[1] >> 2);
						pDest[2] = BYTE(pSrc[2] >> 2);

						pDest+=3;
						pSrc +=Bpp;
					}
				}


				SetDIBitsToDevice(hDC, rc.left, rc.top, rc.right, rc.bottom, 0, 0, 0, Line, pTemp, (BITMAPINFO*)&bm, DIB_RGB_COLORS);
				delete[] pTemp;
			}
		}


		if(Flg){
			ReleaseDC(hWnd, hDC);
		}

		return 1;
	}

	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Draw line with SDK
//

BOOL OnPreview(HWND hWnd)
{
	// Not selected DLL
	if(-1 == g_DllType){
		MessageBox(hWnd, "Select DLL for the camera connected", "Error", MB_OK);
		return FALSE;
	}

	// Delete timer
	if(3 == g_Preview){
		KillTimer(hWnd, TIMER_ID);
	}


	// Release device
	g_CArtCamSdk[0].Close();
	g_CArtCamSdk[1].Close();


	// Release area for image
	SAFE_DELETE(g_pImage[0]);
	SAFE_DELETE(g_pImage[1]);


	// Set window to be displayed
	// When setting NULL to hWnd,it is possible to create new window and show it.
	RECT rc;
	GetClientRect(hWnd, &rc);
	g_CArtCamSdk[0].SetPreviewWindow(hWnd, 0, 0, rc.right/2, rc.bottom);
	g_CArtCamSdk[1].SetPreviewWindow(hWnd, rc.right/2, 0, rc.right, rc.bottom);


	// Display image
	g_CArtCamSdk[0].Preview();
	g_CArtCamSdk[1].Preview();


	// Check menu
	::CheckMenuItem(g_Menu.Preview, 0, MF_BYPOSITION | MF_CHECKED);
	::CheckMenuItem(g_Menu.Preview, 1, MF_BYPOSITION | MF_UNCHECKED);
	::CheckMenuItem(g_Menu.Preview, 3, MF_BYPOSITION | MF_UNCHECKED);
	::CheckMenuItem(g_Menu.Preview, 4, MF_BYPOSITION | MF_UNCHECKED);

	g_Preview = 0;

	InvalidateRect(hWnd, NULL, TRUE);

	return TRUE;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Obtain image pointer and draw with application
//

BOOL OnCallBack(HWND hWnd)
{
	// Not selected DLL
	if(-1 == g_DllType){
		MessageBox(hWnd, "Select DLL for the camera connected", "Error", MB_OK);
		return FALSE;
	}

	// Delete timer
	if(3 == g_Preview){
		KillTimer(hWnd, TIMER_ID);
	}


	// Release device
	g_CArtCamSdk[0].Close();
	g_CArtCamSdk[1].Close();


	// Release area for image
	SAFE_DELETE(g_pImage[0]);
	SAFE_DELETE(g_pImage[1]);


	// If drawing by yourself, set all window size to 0.
	// An automatic display can be performed, if window size is set up even when using CallBackPreview
	g_CArtCamSdk[0].SetPreviewWindow(hWnd, 0, 0, 0, 0);
	g_CArtCamSdk[1].SetPreviewWindow(hWnd, 0, 0, 0, 0);


	// Secure the space for taking in
	g_pImage[0] = new BYTE[ArtCamGetSize(0)];
	g_pImage[1] = new BYTE[ArtCamGetSize(1)];

	g_CArtCamSdk[0].CallBackPreview(hWnd, g_pImage[0], ArtCamGetSize(0), 0);
	g_CArtCamSdk[1].CallBackPreview(hWnd, g_pImage[1], ArtCamGetSize(1), 0);


	// Check menu
	::CheckMenuItem(g_Menu.Preview, 0, MF_BYPOSITION | MF_UNCHECKED);
	::CheckMenuItem(g_Menu.Preview, 1, MF_BYPOSITION | MF_CHECKED);
	::CheckMenuItem(g_Menu.Preview, 3, MF_BYPOSITION | MF_UNCHECKED);
	::CheckMenuItem(g_Menu.Preview, 4, MF_BYPOSITION | MF_UNCHECKED);

	g_Preview = 1;

	InvalidateRect(hWnd, NULL, TRUE);

	return TRUE;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Display one frame
//

BOOL OnSnapShot(HWND hWnd)
{
	// Not selected DLL
	if(-1 == g_DllType){
		MessageBox(hWnd, "Select DLL for the camera connected", "Error", MB_OK);
		return FALSE;
	}

	// Delete timer
	if(3 == g_Preview){
		KillTimer(hWnd, TIMER_ID);
	}


	// Release device
	g_CArtCamSdk[0].Close();
	g_CArtCamSdk[1].Close();


	// Release area for image
	SAFE_DELETE(g_pImage[0]);
	SAFE_DELETE(g_pImage[1]);


	// Secure the space for taking in
	g_pImage[0] = new BYTE[ArtCamGetSize(0)];
	g_pImage[1] = new BYTE[ArtCamGetSize(1)];

	g_CArtCamSdk[0].SnapShot(g_pImage[0], ArtCamGetSize(0), 0);
	g_CArtCamSdk[1].SnapShot(g_pImage[1], ArtCamGetSize(1), 0);


	// Draw image
	DrawImage(hWnd, NULL, 0);
	DrawImage(hWnd, NULL, 1);

	// Check menu
	::CheckMenuItem(g_Menu.Preview, 0, MF_BYPOSITION | MF_UNCHECKED);
	::CheckMenuItem(g_Menu.Preview, 1, MF_BYPOSITION | MF_UNCHECKED);
	::CheckMenuItem(g_Menu.Preview, 3, MF_BYPOSITION | MF_UNCHECKED);
	::CheckMenuItem(g_Menu.Preview, 4, MF_BYPOSITION | MF_UNCHECKED);

	g_Preview = -1;

	return TRUE;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Draw line with SDK
//

LRESULT OnCapture(HWND hWnd)
{
	// Not selected DLL
	if(-1 == g_DllType){
		MessageBox(hWnd, "Select DLL for the camera connected", "Error", MB_OK);
		return FALSE;
	}


	// Release device
	g_CArtCamSdk[0].Close();
	g_CArtCamSdk[1].Close();


	// Release area for image
	SAFE_DELETE(g_pImage[0]);
	SAFE_DELETE(g_pImage[1]);

	// Secure the space for taking in
	g_pImage[0] = new BYTE[ArtCamGetSize(0)];
	g_pImage[1] = new BYTE[ArtCamGetSize(1)];

	g_CArtCamSdk[0].Capture();
	g_CArtCamSdk[1].Capture();


	// Check menu
	::CheckMenuItem(g_Menu.Preview, 0, MF_BYPOSITION | MF_UNCHECKED);
	::CheckMenuItem(g_Menu.Preview, 1, MF_BYPOSITION | MF_UNCHECKED);
	::CheckMenuItem(g_Menu.Preview, 3, MF_BYPOSITION | MF_CHECKED);
	::CheckMenuItem(g_Menu.Preview, 4, MF_BYPOSITION | MF_UNCHECKED);

	g_Preview = 3;

	SetTimer(hWnd, TIMER_ID, TIMER_TIME, NULL);

	return TRUE;
}


LRESULT OnTimer(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	if(3 == g_Preview && TIMER_ID == wParam){
		KillTimer(hWnd, TIMER_ID);

		g_CArtCamSdk[0].SnapShot(g_pImage[0], ArtCamGetSize(0), 0);
		g_CArtCamSdk[1].SnapShot(g_pImage[1], ArtCamGetSize(1), 0);

		// Draw image
		DrawImage(hWnd, NULL, 0);
		DrawImage(hWnd, NULL, 1);

		SetTimer(hWnd, TIMER_ID, TIMER_TIME, NULL);

		return 1;
	}

	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Trigger mode
//

BOOL OnTrigger(HWND hWnd)
{
	// Not selected DLL
	if(-1 == g_DllType){
		MessageBox(hWnd, "Select DLL for the camera connected", "Error", MB_OK);
		return FALSE;
	}

	// Delete timer
	if(3 == g_Preview){
		KillTimer(hWnd, TIMER_ID);
	}


	// Release device
	g_CArtCamSdk[0].Close();
	g_CArtCamSdk[1].Close();


	// Release area for image
	SAFE_DELETE(g_pImage[0]);
	SAFE_DELETE(g_pImage[1]);


	// If drawing by yourself, set all window size to 0.
	// An automatic display can be performed, if window size is set up even when using CallBackPreview
	g_CArtCamSdk[0].SetPreviewWindow(hWnd, 0, 0, 0, 0);
	g_CArtCamSdk[1].SetPreviewWindow(hWnd, 0, 0, 0, 0);


	// Secure the space for taking in
	g_pImage[0] = new BYTE[ArtCamGetSize(0)];
	g_pImage[1] = new BYTE[ArtCamGetSize(1)];

	g_CArtCamSdk[0].Trigger(hWnd, g_pImage[0], ArtCamGetSize(0), 0);
	g_CArtCamSdk[1].Trigger(hWnd, g_pImage[1], ArtCamGetSize(1), 0);


	// Check menu
	::CheckMenuItem(g_Menu.Preview, 0, MF_BYPOSITION | MF_UNCHECKED);
	::CheckMenuItem(g_Menu.Preview, 1, MF_BYPOSITION | MF_UNCHECKED);
	::CheckMenuItem(g_Menu.Preview, 3, MF_BYPOSITION | MF_UNCHECKED);
	::CheckMenuItem(g_Menu.Preview, 4, MF_BYPOSITION | MF_CHECKED);

	g_Preview = 4;

	InvalidateRect(hWnd, NULL, TRUE);

	return TRUE;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Change DLL
//

BOOL OnDllChange(HWND hWnd, LONG DllType)
{
	// Release all devices before switching DLL
	// To display multiple images simulteously, you need to define CArtCamSdk plurally
	g_CArtCamSdk[0].Release();
	g_CArtCamSdk[1].Release();
	g_CArtCamSdk[0].FreeLibrary();
	g_CArtCamSdk[1].FreeLibrary();


	if(!g_CArtCamSdk[0].LoadLibrary(g_DllName[DllType])){
		TCHAR Error[256]="";
		wsprintf(Error, "%s is not found.\nDLL may have been relocated after executing", g_DllName[DllType]);
		MessageBox(hWnd, Error, "Error", MB_OK);
		return FALSE;
	}
	if(!g_CArtCamSdk[1].LoadLibrary(g_DllName[DllType])){
		TCHAR Error[256]="";
		wsprintf(Error, "%s is not found.\nDLL may have been relocated after executing", g_DllName[DllType]);
		MessageBox(hWnd, Error, "Error", MB_OK);
		return FALSE;
	}

	// In case of showing two cameras at the same time,
	// please follow the flow as Initialize -> SetDeviceNumber
	// When try to initialize the other camera after initialize one camera and display,
	// there is possibility of one camera take infuluence.
	if(!g_CArtCamSdk[0].Initialize(hWnd)){
		MessageBox(hWnd, "Failed to initialize SDK", "Error", MB_OK);
		return FALSE;
	}
	if(!g_CArtCamSdk[1].Initialize(hWnd)){
		MessageBox(hWnd, "Failed to initialize SDK", "Error", MB_OK);
		return FALSE;
	}


	// It is possible to know if device is connected after obtain device name.
	TCHAR szDeviceName[256]="";
	if(!g_CArtCamSdk[0].GetDeviceName(0, szDeviceName, 256)){
		g_CArtCamSdk[0].FreeLibrary();
	}
	if(!g_CArtCamSdk[1].GetDeviceName(1, szDeviceName, 256)){
		// No device
		// Please note that there is possibility of duplication access to one device
		// if there is access from two classes when only one device detected.
		// In this case, please release class or do not call function.

		// If you release class, it comes to be safe failure even when call function.
		g_CArtCamSdk[1].FreeLibrary();
	}


	g_CArtCamSdk[0].SetDeviceNumber(0);
	g_CArtCamSdk[1].SetDeviceNumber(1);

	// To operate 2 cameras simultenously, set camera clock to half.
	g_CArtCamSdk[0].SetHalfClock(TRUE);
	g_CArtCamSdk[1].SetHalfClock(TRUE);




	// Check menu
	for(int i=0; i<DLLCOUNT; i++){
		::CheckMenuItem(g_Menu.Dll, i, MF_BYPOSITION | MF_UNCHECKED);
	}

	::CheckMenuItem(g_Menu.Dll, DllType, MF_BYPOSITION | MF_CHECKED);


	// Switch to ON or OFF of setting window
	switch(DllType){
	// CNV
	case 3:
		::EnableMenuItem(g_Menu.Set, 0, MF_BYPOSITION | MF_ENABLED);
		::EnableMenuItem(g_Menu.Set, 1, MF_BYPOSITION | MF_ENABLED);
		::EnableMenuItem(g_Menu.Set, 2, MF_BYPOSITION | MF_GRAYED);
	break;

	// Other
	default:
		::EnableMenuItem(g_Menu.Set, 0, MF_BYPOSITION | MF_ENABLED);
		::EnableMenuItem(g_Menu.Set, 1, MF_BYPOSITION | MF_ENABLED);
		::EnableMenuItem(g_Menu.Set, 2, MF_BYPOSITION | MF_ENABLED);
	break;
	}


	switch(g_Preview){
	case 0:	return OnPreview(hWnd);
	case 1:	return OnCallBack(hWnd);
	case 3:	return OnCapture(hWnd);
	case 4:	return OnTrigger(hWnd);
	}

	return FALSE;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	WM_CREATE
//

LRESULT OnCreate(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	TCHAR szFilePath[MAX_PATH]="";
	TCHAR szExePath[3][MAX_PATH]={"","",""};
	GetModuleFileName(::GetModuleHandle(NULL), szExePath[0], MAX_PATH);
	_splitpath(szExePath[0], szExePath[1], szExePath[2], NULL, NULL);
	wsprintf(szFilePath, "%s%sSample.ini", szExePath[1], szExePath[2]);

	g_DllType = ::GetPrivateProfileInt("Sample", "DllType", -1, szFilePath);

	return 1;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	WM_COMMAND
//

LRESULT OnCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	switch (LOWORD(wParam)){
	// Save file
	case IDM_SAVE:
		g_CArtCamSdk[0].SaveImage("Image1.jpg", FILETYPE_JPEG_HIGH);
		g_CArtCamSdk[1].SaveImage("Image2.jpg", FILETYPE_JPEG_HIGH);
	return 1;

	// End
	case IDM_EXIT:
		::PostMessage(hWnd, WM_CLOSE, 0, 0);
	return 1;

	// Draw with SDK
	case IDM_PREVIEW:
		OnPreview(hWnd);
	return 1;

	// Obtain pointer of image and draw
	case IDM_CALLBACK:
		OnCallBack(hWnd);
	return 1;

	// Obtain A image
	case IDM_SNAPSHOT:
		OnSnapShot(hWnd);
	return 1;

	// Capture snapshot in a low
	case IDM_CAPTURE:
		OnCapture(hWnd);
	return 1;

	// Trigger mode
	case IDM_TRIGGER:
		OnTrigger(hWnd);
	return 1;

	// Camera settingsCamera setting
	case IDM_FORMAT1:
		// When show camera setting dialog,recommend to release display once.
		// Stop image refresh with SDK even when not stop with application.
		// In case of stop with SDK,it restarts when close dialog
		// There is possibility of illegality processing 
		// if you don't correspond setting with application when you change setting and close dialog.
		if(3 == g_Preview)	KillTimer(hWnd, TIMER_ID);

		g_CArtCamSdk[0].Close();
		g_CArtCamSdk[1].Close();
		g_CArtCamSdk[0].SetCameraDlg(hWnd);

		switch(g_Preview){
		case 0:	return OnPreview(hWnd);
		case 1:	return OnCallBack(hWnd);
		case 3:	return OnCapture(hWnd);
		case 4:	return OnTrigger(hWnd);
		}
	return 1;

	case IDM_FORMAT2:
		// When show camera setting dialog,recommend to release display once.
		// Stop image refresh with SDK even when not stop with application.
		// In case of stop with SDK,it restarts when close dialog
		// There is possibility of illegality processing 
		// if you don't correspond setting with application when you change setting and close dialog.
		if(3 == g_Preview)	KillTimer(hWnd, TIMER_ID);

		g_CArtCamSdk[0].Close();
		g_CArtCamSdk[1].Close();
		g_CArtCamSdk[1].SetCameraDlg(hWnd);

		switch(g_Preview){
		case 0:	return OnPreview(hWnd);
		case 1:	return OnCallBack(hWnd);
		case 3:	return OnCapture(hWnd);
		case 4:	return OnTrigger(hWnd);
		}
	return 1;

	// Filter settings
	case IDM_FILTER1:
		g_CArtCamSdk[0].SetImageDlg(hWnd);
	return 1;

	case IDM_FILTER2:
		g_CArtCamSdk[1].SetImageDlg(hWnd);
	return 1;

	// Analog settings
	case IDM_ANALOG1:
		g_CArtCamSdk[0].SetAnalogDlg(hWnd);
	return 1;

	case IDM_ANALOG2:
		g_CArtCamSdk[1].SetAnalogDlg(hWnd);
	return 1;

	// Switch to DLL
	case IDM_CAM:		g_DllType = 0;		OnDllChange(hWnd, g_DllType);	return 1;
	case IDM_USTC:		g_DllType = 2;		OnDllChange(hWnd, g_DllType);	return 1;
	case IDM_CNV:		g_DllType = 3;		OnDllChange(hWnd, g_DllType);	return 1;
	case IDM_130MI:		g_DllType = 5;		OnDllChange(hWnd, g_DllType);	return 1;
	case IDM_200MI:		g_DllType = 6;		OnDllChange(hWnd, g_DllType);	return 1;
	case IDM_300MI:		g_DllType = 7;		OnDllChange(hWnd, g_DllType);	return 1;
	case IDM_150P:		g_DllType = 8;		OnDllChange(hWnd, g_DllType);	return 1;
	case IDM_320P:		g_DllType = 9;		OnDllChange(hWnd, g_DllType);	return 1;
	case IDM_200SH:		g_DllType = 10;		OnDllChange(hWnd, g_DllType);	return 1;
	case IDM_098:		g_DllType = 11;		OnDllChange(hWnd, g_DllType);	return 1;
	case IDM_036MI:		g_DllType = 12;		OnDllChange(hWnd, g_DllType);	return 1;
	case IDM_500P:		g_DllType = 13;		OnDllChange(hWnd, g_DllType);	return 1;
	case IDM_150P2:		g_DllType = 14;		OnDllChange(hWnd, g_DllType);	return 1;
	}

	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	WM_PAINT
//

LRESULT OnPaint(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	PAINTSTRUCT psPaint;
	HDC hDC = BeginPaint(hWnd, &psPaint);
	if(hDC){
		DrawImage(hWnd, hDC, 0);
		DrawImage(hWnd, hDC, 1);
		EndPaint(hWnd, &psPaint);
	}

	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	WM_ERROR
//

LRESULT OnError(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	switch(lParam){
	case ARTCAMSDK_NOERROR:		break;
	case ARTCAMSDK_NOT_INITIALIZE:	SetWindowText(hWnd, "Not initialized");	break;
	case ARTCAMSDK_DISABLEDDEVICE:	SetWindowText(hWnd, "Access to unavailable device was attempted");	break;
	case ARTCAMSDK_CREATETHREAD:	SetWindowText(hWnd, "Failure of thread creation for capturing");	break;
	case ARTCAMSDK_CREATEWINDOW:	SetWindowText(hWnd, "Window creation failed");	break;
	case ARTCAMSDK_OUTOFMEMORY:		SetWindowText(hWnd, "Not enough memory for image transfer/Failure for securing memory");	break;
	case ARTCAMSDK_CAMERASET:		SetWindowText(hWnd, "Error for camera (device) settings");	break;
	case ARTCAMSDK_CAPTURE:			SetWindowText(hWnd, "Capturing failed");	break;
	case ARTCAMSDK_PARAM:			SetWindowText(hWnd, "Wrong argument");	break;
	case ARTCAMSDK_DIRECTSHOW:		SetWindowText(hWnd, "Directshow initialization error");	break;
	case ARTCAMSDK_UNSUPPORTED:		SetWindowText(hWnd, "Not supported");	break;
	case ARTCAMSDK_UNKNOWN:			SetWindowText(hWnd, "Unknow error");	break;
	case ARTCAMSDK_CAPTURELOST:		SetWindowText(hWnd, "Device lost");	break;
	default:						SetWindowText(hWnd, "Unknow error");	break;
	}

	return 1;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	WM_GRAPHPAINT
//

LRESULT OnGraphPaint(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	if(NULL == wParam){
		return OnError(hWnd, wParam, lParam);
	}


	// Once this message is sent, latest image is stored at "g_pImage"
	LPGP_INFO pGpInfo = (LPGP_INFO)wParam;
	DrawImage(hWnd, NULL, 0);
	DrawImage(hWnd, NULL, 1);


	TCHAR Fps[256]="";
	sprintf(Fps, "Sample  %.1f Frame", double(pGpInfo->lFps) / 10.0);
	SetWindowText(hWnd, Fps);

	return 1;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	WM_DESTROY
//

LRESULT OnDestroy(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	// Call "Release" when you end use of camera. (also called "destructor").
	// You need to call "Initialise" when you re-use camera after "Release".
	g_CArtCamSdk[0].Release();
	g_CArtCamSdk[1].Release();
	SAFE_DELETE(g_pImage[0]);
	SAFE_DELETE(g_pImage[1]);


	TCHAR szFilePath[MAX_PATH]="";
	TCHAR szExePath[3][MAX_PATH]={"","",""};
	GetModuleFileName(::GetModuleHandle(NULL), szExePath[0], MAX_PATH);
	_splitpath(szExePath[0], szExePath[1], szExePath[2], NULL, NULL);
	wsprintf(szFilePath, "%s%sSample.ini", szExePath[1], szExePath[2]);


	TCHAR Data[32]="";
	wsprintf(Data, "%d", g_DllType);

	::WritePrivateProfileString("Sample", "DllType", Data, szFilePath);


	::PostQuitMessage(0);
	return 1;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	WM_SIZE
//

HWND ChildList[2] = {NULL, NULL};

static BOOL CALLBACK EnumChildProc(HWND hWnd, LPARAM lParam)
{
	for(int i=0; i<2; i++){
		if(NULL == ChildList[i]){
			ChildList[i] = hWnd;
			break;
		}
	}

	return 1;
}

LRESULT OnSize(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	if(0 == g_Preview){
		ChildList[0] = NULL;
		ChildList[1] = NULL;

		// Child window enumeration 
		::LockWindowUpdate(hWnd);
		::EnumChildWindows(hWnd, ::EnumChildProc, NULL);
		::LockWindowUpdate(NULL);

		RECT rc;
		GetClientRect(hWnd, &rc);

		LONG Width[2]  = { rc.right/2, rc.right/2 };
		LONG Height[2] = { rc.bottom, rc.bottom };


		// To avoiding changing aspect ratio, remove comment
/*
		// Calculation results in decimal figure. So apply floating-point calculations.
		for(int i=0; i<2; i++){
			double winW = rc.right/2;
			double winH = rc.bottom;
			double capW = ArtCamGetWidth(i);
			double capH = ArtCamGetHeight(i);

			if( (winW * capH) > (winH * capW) ){
				winW = winH * capW / capH;
				Width[i] = LONG(winW);
			}
			else if( (winW * capH) < (winH * capW) ){
				winH = winW * capH / capW;
				Height[i] = LONG(winH);
			}
		}
*/
		SetWindowPos(ChildList[0], HWND_TOP,          0, 0, Width[0], Height[0], SWP_NOZORDER | SWP_SHOWWINDOW);
		SetWindowPos(ChildList[1], HWND_TOP, rc.right/2, 0, Width[0], Height[0], SWP_NOZORDER | SWP_SHOWWINDOW);
	}

	return 1;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Window procedure
//

LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg){
	case WM_CREATE:		return OnCreate			(hWnd, wParam, lParam);
	case WM_COMMAND:	return OnCommand		(hWnd, wParam, lParam);
	case WM_PAINT:		return OnPaint			(hWnd, wParam, lParam);
	case WM_GRAPHPAINT:	return OnGraphPaint		(hWnd, wParam, lParam);
	case WM_ERROR:		return OnError			(hWnd, wParam, lParam);
	case WM_DESTROY:	return OnDestroy		(hWnd, wParam, lParam);
	case WM_SIZE:		return OnSize			(hWnd, wParam, lParam);
	case WM_TIMER:		return OnTimer			(hWnd, wParam, lParam);
	default:			return ::DefWindowProc	(hWnd, uMsg, wParam, lParam);
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Window initialization
//

HWND InitWindow(HINSTANCE hInstance, int nCmdShow)
{
	// Registrate window class
	WNDCLASSEX wndclassex;
	ZeroMemory(&wndclassex,		sizeof(wndclassex));
	wndclassex.cbSize			= sizeof(wndclassex);
	wndclassex.style			= 0;
	wndclassex.lpfnWndProc		= WndProc;
	wndclassex.cbClsExtra		= 0;
	wndclassex.cbWndExtra		= 0;
	wndclassex.hInstance		= hInstance;
	wndclassex.hIcon			= LoadIcon(NULL, IDI_APPLICATION);
	wndclassex.hCursor			= LoadCursor(NULL, IDC_ARROW);
	wndclassex.hbrBackground	= reinterpret_cast<HBRUSH>(::GetStockObject(BLACK_BRUSH));
	wndclassex.lpszClassName	= "Sample";
	wndclassex.lpszMenuName		= 0;
	wndclassex.hIconSm			= NULL;

	if(0 == ::RegisterClassEx(&wndclassex))
		return NULL;

	// Window size setting
	RECT rcWndSize = {0, 0, 900, 600};
	DWORD dwStyle = WS_POPUPWINDOW | WS_CAPTION | WS_OVERLAPPEDWINDOW;
	if(0 == ::AdjustWindowRectEx(&rcWndSize, dwStyle, TRUE, NULL))
		return NULL;

	// Main window create
	HWND hWnd = ::CreateWindowEx(NULL, "Sample", "Sample", dwStyle,
			(::GetSystemMetrics(SM_CXSCREEN) - (rcWndSize.right - rcWndSize.left)) / 2,
			(::GetSystemMetrics(SM_CYSCREEN) - (rcWndSize.bottom - rcWndSize.top)) / 2,
			rcWndSize.right - rcWndSize.left, rcWndSize.bottom - rcWndSize.top,
			HWND_DESKTOP, NULL, hInstance, NULL);

	if(NULL == hWnd)
		return NULL;


	// Create menu

	// File menu
	g_Menu.File = ::CreateMenu();
	::InsertMenu(g_Menu.File, 0, MF_BYPOSITION | MF_STRING, IDM_SAVE, "Save(&S)");
	::InsertMenu(g_Menu.File, 1, MF_BYPOSITION | MF_STRING, IDM_EXIT, "End(&X)");

	// Display type menu
	g_Menu.Preview = ::CreateMenu();
	::InsertMenu(g_Menu.Preview, 0, MF_BYPOSITION | MF_STRING, IDM_PREVIEW, "Preview(&P)");
	::InsertMenu(g_Menu.Preview, 1, MF_BYPOSITION | MF_STRING, IDM_CALLBACK, "Callback(&B)");
	::InsertMenu(g_Menu.Preview, 2, MF_BYPOSITION | MF_STRING, IDM_SNAPSHOT, "Snapshot(&S)");
	::InsertMenu(g_Menu.Preview, 3, MF_BYPOSITION | MF_STRING, IDM_CAPTURE, "Capture(&C)");
	::InsertMenu(g_Menu.Preview, 4, MF_BYPOSITION | MF_STRING, IDM_TRIGGER, "Trigger(&T)");

	// Settings menu
	g_Menu.Set = ::CreateMenu();
	::InsertMenu(g_Menu.Set, 0, MF_BYPOSITION | MF_STRING, IDM_FORMAT1, "Format settings1(&C)");
	::InsertMenu(g_Menu.Set, 1, MF_BYPOSITION | MF_STRING, IDM_FORMAT2, "Format settings2(&C)");
	::InsertMenu(g_Menu.Set, 2, MF_BYPOSITION | MF_STRING, IDM_FILTER1, "Filter settings1(&F)");
	::InsertMenu(g_Menu.Set, 3, MF_BYPOSITION | MF_STRING, IDM_FILTER2, "Filter settings2(&F)");
	::InsertMenu(g_Menu.Set, 4, MF_BYPOSITION | MF_STRING, IDM_ANALOG1, "Analog settings1(&A)");
	::InsertMenu(g_Menu.Set, 5, MF_BYPOSITION | MF_STRING, IDM_ANALOG2, "Analog settings2(&A)");

	// DLL selective menu
	g_Menu.Dll = ::CreateMenu();
	::InsertMenu(g_Menu.Dll,  0, MF_BYPOSITION | MF_STRING, IDM_CAM,   "DirectShow Camera");
	::InsertMenu(g_Menu.Dll,  1, MF_BYPOSITION | MF_SEPARATOR, NULL, NULL );
	::InsertMenu(g_Menu.Dll,  2, MF_BYPOSITION | MF_STRING, IDM_USTC,  "ARTUST-0209C");
	::InsertMenu(g_Menu.Dll,  3, MF_BYPOSITION | MF_STRING, IDM_CNV,   "ARTCNV");
	::InsertMenu(g_Menu.Dll,  4, MF_BYPOSITION | MF_SEPARATOR, NULL, NULL );
	::InsertMenu(g_Menu.Dll,  5, MF_BYPOSITION | MF_STRING, IDM_130MI, "ARTCAM-130MI");
	::InsertMenu(g_Menu.Dll,  6, MF_BYPOSITION | MF_STRING, IDM_200MI, "ARTCAM-200MI");
	::InsertMenu(g_Menu.Dll,  7, MF_BYPOSITION | MF_STRING, IDM_300MI, "ARTCAM-300MI");
	::InsertMenu(g_Menu.Dll,  8, MF_BYPOSITION | MF_STRING, IDM_150P,  "ARTCAM-150P");
	::InsertMenu(g_Menu.Dll,  9, MF_BYPOSITION | MF_STRING, IDM_320P,  "ARTCAM-320P");
	::InsertMenu(g_Menu.Dll, 10, MF_BYPOSITION | MF_STRING, IDM_200SH, "ARTCAM-200SH");
	::InsertMenu(g_Menu.Dll, 11, MF_BYPOSITION | MF_STRING, IDM_098,   "ARTCAM-098");
	::InsertMenu(g_Menu.Dll, 12, MF_BYPOSITION | MF_STRING, IDM_036MI, "ARTCAM-036MI");
	::InsertMenu(g_Menu.Dll, 13, MF_BYPOSITION | MF_STRING, IDM_500P,  "ARTCAM-500P");
	::InsertMenu(g_Menu.Dll, 14, MF_BYPOSITION | MF_STRING, IDM_500P,  "ARTCAM-150P2");

	// Check if there is DLL
	for(int i=0; i<DLLCOUNT; i++){
		CArtCamSdk ArtCam;
		if(!ArtCam.LoadLibrary(g_DllName[i])){
			::EnableMenuItem(g_Menu.Dll, i, MF_BYPOSITION | MF_GRAYED);
		}
	}

	// Main menu
	HMENU hMenu = ::CreateMenu();
	::InsertMenu(hMenu, 0, MF_POPUP | MF_STRING, (UINT)g_Menu.File,    "File(&F)");
	::InsertMenu(hMenu, 1, MF_POPUP | MF_STRING, (UINT)g_Menu.Preview, "Display(&V)");
	::InsertMenu(hMenu, 2, MF_POPUP | MF_STRING, (UINT)g_Menu.Set,     "Settings(&S)");
	::InsertMenu(hMenu, 3, MF_POPUP | MF_STRING, (UINT)g_Menu.Dll,     "DLL(&L)");


	::SetMenu(hWnd, hMenu);


	// Focus settings
	::SetFocus(hWnd);
	::ShowWindow(hWnd, nCmdShow);
	if(0 == ::UpdateWindow(hWnd))
		return NULL;

	// Initialize camera
	if(-1 != g_DllType)
		OnDllChange(hWnd, g_DllType);

	SetTimer(hWnd, 1, 100, NULL);
	return hWnd;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	WinMain
//

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmLine, int nCmdShow)
{
	HWND hWnd = InitWindow(hInstance, nCmdShow);

	// Message roop
	MSG msg;
	ZeroMemory(&msg, sizeof(msg));
	while(::GetMessage(&msg, NULL, 0, 0)){
		::TranslateMessage(&msg);
		::DispatchMessage(&msg);
	}

	return msg.wParam;
}



