//==========================================================
//	ARTRAY Camera / Capture Module Software Developer Kit
//						Preview/Callback function sample
//						VC++.NET Win32 Project
//
//									(C) 2002-2009 Artray
//==========================================================




//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Include file
//

#include <windows.h>
#include <stdio.h>

// Library include
#include "CArtCamSdk.hpp"



// Various menu ID 
enum {
	IDM_SAVE = WM_USER + 1,
	IDM_EXIT,

	IDM_PREVIEW,
	IDM_CALLBACK,
	IDM_SNAPSHOT,
	IDM_CAPTURE,
	IDM_TRIGGER,

	IDM_FORMAT,
	IDM_FILTER,
	IDM_ANALOG,

	IDM_CAM,
	IDM_USTC,
	IDM_CNV,
	IDM_130MI,
	IDM_200MI,
	IDM_300MI,
	IDM_150P,
	IDM_320P,
	IDM_200SH,
	IDM_098,
	IDM_036MI,
	IDM_500P,
	IDM_150P2,
	IDM_500MI,
	IDM_T111,
	IDM_T112,
	IDM_150P3,
	IDM_130MIV,
	IDM_150P3V,
	IDM_267KY,
	IDM_274KY,
	IDM_625KY,
    IDM_V135MI,
	IDM_SATA,
	IDM_445KY,
	IDM_098II,
	IDM_MV413,
	IDM_OV210,
	IDM_850SH,
	IDM_1251SH,
	IDM_D131,
	IDM_900MI,
	IDM_1000MI,
	IDM_500P2,

	IDM_SATA_LVDS,
	IDM_SATA_300MI,
	IDM_SATA_500MI,
	IDM_SATA_MV413,
	IDM_SATA_800MI,
	IDM_SATA_036MI,
	IDM_SATA_150P3,
	IDM_SATA_267KY,
	IDM_SATA_274KY,
	IDM_SATA_625KY,
	IDM_SATA_130MI,
	IDM_SATA_200MI,


	IDM_WIDEHI,
	IDM_WIDELOW,
	IDM_NORMALHI,
	IDM_NORMALLOW,
};



// WM_USER + 50 - WM_USER + 59 Reserved for device settings
#define IDM_DEVICE		WM_USER + 50


#define TIMER_ID	100
#define TIMER_TIME	50



CArtCamSdk	g_CArtCamSdk;
LPBYTE		g_pImage = NULL;		// Space for image data

BOOL		g_Preview = -1;			// Preview display/Callback display
LONG		g_DllType = -1;			// DLL to be connected
LONG		g_CameraType = -1;		// Camera type to be connected
LONG		g_DeviceCount = 0;		// Number of detected device


const int SUB_SAMPLE[] = { 1, 2, 4, 8 };
#define ArtCamGetWidth()	(g_CArtCamSdk.Width()  / SUB_SAMPLE[g_CArtCamSdk.GetSubSample()])
#define ArtCamGetHeight()	(g_CArtCamSdk.Height() / SUB_SAMPLE[g_CArtCamSdk.GetSubSample()])
#define ArtCamGetColor()	WORD(((g_CArtCamSdk.GetColorMode() + 7) / 8) * 8)
#define ArtCamGetBpp()		(((g_CArtCamSdk.GetColorMode() + 7) / 8))
#define ArtCamGetPitch()	((ArtCamGetWidth() * ArtCamGetBpp() + 3) & ~3)
#define ArtCamGetSize()		(ArtCamGetPitch() * ArtCamGetHeight())
#define SAFE_DELETE(x)  { if(x) delete[] x; x=NULL; }


struct SCROLL_CTRL{
	POINT	Pos;

	BOOL	MoveFlg;
	POINT	Start;
};

SCROLL_CTRL	g_ScrollCtrl;

struct MENU_LIST{
	HMENU	File;
	HMENU	Preview;
	HMENU	Set;
	HMENU	Dll;
	HMENU	Device;
	HMENU	Rate;
};

MENU_LIST g_Menu;



const TCHAR DLLNAMELIST[][32] = {
	"ArtCamSdk.dll",
	"",
	"ArtCamSdk_USTC.dll",
	"ArtCnvSdk.dll",
	"",
	"ArtCamSdk_130MI.dll",
	"ArtCamSdk_200MI.dll",
	"ArtCamSdk_300MI.dll",
	"ArtCamSdk_150P.dll",
	"ArtCamSdk_320P.dll",
	"ArtCamSdk_200SH.dll",
	"ArtCamSdk_098.dll",
	"ArtCamSdk_036MI.dll",
	"ArtCamSdk_500P.dll",
	"ArtCamSdk_150P2.dll",
	"ArtCamSdk_500MI.dll",
	"ArtCamSdk_T111.dll",
	"ArtCamSdk_T112.dll",
	"ArtCamSdk_150P3.dll",
	"ArtCamSdk_130MIV.dll",
	"ArtCamSdk_150P3V.dll",
	"ArtCamSdk_267KY.dll",
	"ArtCamSdk_274KY.dll",
	"ArtCamSdk_625KY.dll",
	"ArtCamSdk_V135MI.dll",
	"",
	"ArtCamSdk_SATA.dll",
	"",
	"ArtCamSdk_445KY.dll",
    "ArtCamSdk_098II.dll",
	"ArtCamSdk_MV413.dll",
	"ArtCamSdk_OV210.dll",
	"ArtCamSdk_850SH.dll",
	"ArtCamSdk_1251SH.dll",
	"ArtCamSdk_D131.dll",
	"ArtCamSdk_900MI.dll",
	"ArtCamSdk_1000MI.dll",
	"ArtCamSdk_500P2.dll",
};

const TCHAR ERRORLIST[][128] = {
	"",
	"Not initialized",
	"Access to unavailable device was attempted",
	"Failure of thread creation for capturing",
	"Window creation failed",
	"Not enough memory for image transfer/Failure for securing memory",
	"Error for camera (device) settings",
	"Error for camera (device) size settings",
	"Capturing failed",
	"Wrong argument",
	"Directshow initialization error",
	"Not supported",
	"Unknow error",
	"Device lost",
	"File not found",
	"FPGA Setting error",
};

const int DLLCOUNT = sizeof(DLLNAMELIST)/( 32 * sizeof(TCHAR));
const int ERRORCOUNT = sizeof(ERRORLIST)/(128 * sizeof(TCHAR));

LONG DefProc = NULL;


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Re-construct scroll bar
//

void ChangeScrollbar(HWND hWnd)
{
	if(0 == g_Preview){
		g_ScrollCtrl.Pos.x = 0;
		g_ScrollCtrl.Pos.y = 0;
		ShowScrollBar(hWnd, SB_HORZ, FALSE);
		ShowScrollBar(hWnd, SB_VERT, FALSE);
		return;
	}

	RECT rc;
	GetClientRect(hWnd, &rc);

	double dwScale = 1;

	LONG lWidth  = LONG(double(double(ArtCamGetWidth() ) * dwScale));
	LONG lHeight = LONG(double(double(ArtCamGetHeight()) * dwScale));

	LONG PosX = LONG(double(double(g_ScrollCtrl.Pos.x) * dwScale));
	LONG PosY = LONG(double(double(g_ScrollCtrl.Pos.y) * dwScale));


	SCROLLINFO sc;
	sc.fMask	= SIF_RANGE | SIF_POS | SIF_PAGE;

	LONG OffsetH = 16;
	LONG OffsetV = 16;


	if(rc.right + OffsetH >= lWidth){
		ShowScrollBar(hWnd, SB_HORZ, FALSE);
	} else {
		sc.nMin		= 0;
		sc.nMax		= lWidth;
		sc.nPage	= rc.right;
		sc.nPos		= PosX;
		SetScrollInfo(hWnd, SB_HORZ, &sc, TRUE);
	}

	if(rc.bottom + OffsetV >= lHeight){
		ShowScrollBar(hWnd, SB_VERT, FALSE);
	} else {
		sc.nMin		= 0;
		sc.nMax		= lHeight;
		sc.nPage	= rc.bottom;
		sc.nPos		= PosY;
		SetScrollInfo(hWnd, SB_VERT, &sc, TRUE);
	}


	double dwWidth  = double(ArtCamGetWidth ());
	double dwHeight = double(ArtCamGetHeight());

	if(dwWidth - (double(rc.right) / dwScale) < g_ScrollCtrl.Pos.x)
		g_ScrollCtrl.Pos.x = LONG(dwWidth - (double(rc.right) / dwScale));
	if(dwHeight - (double(rc.bottom) / dwScale) < g_ScrollCtrl.Pos.y)
		g_ScrollCtrl.Pos.y = LONG(dwHeight - (double(rc.bottom) / dwScale));
	if(0.0 > g_ScrollCtrl.Pos.x)
		g_ScrollCtrl.Pos.x = 0;
	if(0.0 > g_ScrollCtrl.Pos.y)
		g_ScrollCtrl.Pos.y = 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Draw image
//

struct BITMAPINFO2 {
	BITMAPINFOHEADER   bmiHeader;
	RGBQUAD            bmiColors[256];
};

LRESULT DrawImage(HWND hWnd, HDC hDC)
{
	if(NULL == g_pImage){
		return 0;
	}

	BOOL Flg = FALSE;
	if(NULL == hDC){
		hDC = GetDC(hWnd);
		Flg = TRUE;
	}

	if(hDC){
		LONG Width = ArtCamGetWidth();
		LONG Height = ArtCamGetHeight();
		LONG Bpp = ArtCamGetBpp();

		BITMAPINFO2 bm;
		bm.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
		bm.bmiHeader.biPlanes = 1;
		bm.bmiHeader.biCompression = BI_RGB;
		bm.bmiHeader.biSizeImage = 0;
		bm.bmiHeader.biXPelsPerMeter = 0;
		bm.bmiHeader.biYPelsPerMeter = 0;
		bm.bmiHeader.biClrUsed = 0;
		bm.bmiHeader.biClrImportant = 0;

		bm.bmiHeader.biWidth = Width;
		bm.bmiHeader.biHeight = Height;


		// Monochrome 8 bit picture transfer
		if(1 == Bpp){
			bm.bmiHeader.biBitCount = 8;

			for(int i=0; i<256; i++){
				bm.bmiColors[i].rgbRed = i;
				bm.bmiColors[i].rgbGreen = i;
				bm.bmiColors[i].rgbBlue = i;
				bm.bmiColors[i].rgbReserved = 0;
			}
			::SetDIBColorTable(hDC, 0, 256, bm.bmiColors);

			SetDIBitsToDevice(hDC, -g_ScrollCtrl.Pos.x, -g_ScrollCtrl.Pos.y, bm.bmiHeader.biWidth, bm.bmiHeader.biHeight,
								0, 0, 0, bm.bmiHeader.biHeight, g_pImage, (BITMAPINFO*)&bm, DIB_RGB_COLORS);
		}

		// Monochromatic 10 bit/16 bit picture transfer
		else if(2 == Bpp){
			bm.bmiHeader.biBitCount = 8;

			for(int i=0; i<256; i++){
				bm.bmiColors[i].rgbRed = i;
				bm.bmiColors[i].rgbGreen = i;
				bm.bmiColors[i].rgbBlue = i;
				bm.bmiColors[i].rgbReserved = 0;
			}
			::SetDIBColorTable(hDC, 0, 256, bm.bmiColors);

			LPBYTE pTemp = new BYTE[bm.bmiHeader.biWidth * bm.bmiHeader.biHeight];
			if(pTemp){
				for(int y=0; y<bm.bmiHeader.biHeight; y++){

					LPBYTE pDest =        pTemp     + (y * bm.bmiHeader.biWidth);
					LPWORD pSrc  = LPWORD(g_pImage) + (y * bm.bmiHeader.biWidth);

					for(int x=0; x<bm.bmiHeader.biWidth; x++){
						pDest[0] = BYTE(pSrc[0] >> 2);

						pDest++;
						pSrc ++;
					}
				}


				SetDIBitsToDevice(hDC, -g_ScrollCtrl.Pos.x, -g_ScrollCtrl.Pos.y, bm.bmiHeader.biWidth, bm.bmiHeader.biHeight,
									0, 0, 0, bm.bmiHeader.biHeight, pTemp, (BITMAPINFO*)&bm, DIB_RGB_COLORS);
				delete[] pTemp;
			}
		}

		// Color 24 bit transfer
		else if(3 == Bpp){
			bm.bmiHeader.biBitCount = 24;
			SetDIBitsToDevice(hDC, -g_ScrollCtrl.Pos.x, -g_ScrollCtrl.Pos.y, bm.bmiHeader.biWidth, bm.bmiHeader.biHeight,
								0, 0, 0, bm.bmiHeader.biHeight, g_pImage, (BITMAPINFO*)&bm, DIB_RGB_COLORS);
		}

		// Color 32 bit transfer
		else if(4 == Bpp){
			bm.bmiHeader.biBitCount = 24;

			LPBYTE pTemp = new BYTE[bm.bmiHeader.biWidth * bm.bmiHeader.biHeight * 3];
			if(pTemp){
				for(int y=0; y<bm.bmiHeader.biHeight; y++){

					LPBYTE pDest = pTemp    + (y * bm.bmiHeader.biWidth * 3);
					LPBYTE pSrc  = g_pImage + (y * bm.bmiHeader.biWidth * 4);

					for(int x=0; x<bm.bmiHeader.biWidth; x++){
						pDest[0] = pSrc[0];
						pDest[1] = pSrc[1];
						pDest[2] = pSrc[2];

						pDest+=3;
						pSrc +=4;
					}
				}


				SetDIBitsToDevice(hDC, -g_ScrollCtrl.Pos.x, -g_ScrollCtrl.Pos.y, bm.bmiHeader.biWidth, bm.bmiHeader.biHeight,
									0, 0, 0, bm.bmiHeader.biHeight, pTemp, (BITMAPINFO*)&bm, DIB_RGB_COLORS);
				delete[] pTemp;
			}
		}

		// Color 48, 64 bit transfer
		else if(6 == Bpp || 8 == Bpp){
			bm.bmiHeader.biBitCount = 24;
			Bpp /= 2;

			LPBYTE pTemp = new BYTE[bm.bmiHeader.biWidth * bm.bmiHeader.biHeight * 3];
			if(pTemp){
				for(int y=0; y<bm.bmiHeader.biHeight; y++){

					LPBYTE pDest =        pTemp     + (y * bm.bmiHeader.biWidth * 3);
					LPWORD pSrc  = LPWORD(g_pImage) + (y * bm.bmiHeader.biWidth * Bpp);

					for(int x=0; x<bm.bmiHeader.biWidth; x++){
						pDest[0] = BYTE(pSrc[0] >> 2);
						pDest[1] = BYTE(pSrc[1] >> 2);
						pDest[2] = BYTE(pSrc[2] >> 2);

						pDest+=3;
						pSrc +=Bpp;
					}
				}


				SetDIBitsToDevice(hDC, -g_ScrollCtrl.Pos.x, -g_ScrollCtrl.Pos.y, bm.bmiHeader.biWidth, bm.bmiHeader.biHeight,
									0, 0, 0, bm.bmiHeader.biHeight, pTemp, (BITMAPINFO*)&bm, DIB_RGB_COLORS);
				delete[] pTemp;
			}
		}


		if(Flg){
			ReleaseDC(hWnd, hDC);
		}

		return 1;
	}

	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Draw line with SDK
//

BOOL OnPreview(HWND hWnd)
{
	// Not selected DLL
	if(-1 == g_DllType){
		MessageBox(hWnd, "Select DLL for the camera connected", "Error", MB_OK);
		return FALSE;
	}

	// Delete timer
	if(3 == g_Preview){
		KillTimer(hWnd, TIMER_ID);
	}


	// Release device
	g_CArtCamSdk.Close();


	// Release area for image
	SAFE_DELETE(g_pImage);


	// Set window to be displayed
	// When setting NULL to hWnd,it is possible to create new window and show it.
	RECT rc;
	GetClientRect(hWnd, &rc);
	g_CArtCamSdk.SetPreviewWindow(hWnd, 0, 0, rc.right, rc.bottom);


	// Display image
	g_CArtCamSdk.Preview();


	// Check menu
	::CheckMenuItem(g_Menu.Preview, 0, MF_BYPOSITION | MF_CHECKED);
	::CheckMenuItem(g_Menu.Preview, 1, MF_BYPOSITION | MF_UNCHECKED);
	::CheckMenuItem(g_Menu.Preview, 3, MF_BYPOSITION | MF_UNCHECKED);
	::CheckMenuItem(g_Menu.Preview, 4, MF_BYPOSITION | MF_UNCHECKED);

	g_Preview = 0;

	ChangeScrollbar(hWnd);
	InvalidateRect(hWnd, NULL, TRUE);

	return TRUE;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Obtain image pointer and draw with application
//

BOOL OnCallBack(HWND hWnd)
{
	// Not selected DLL
	if(-1 == g_DllType){
		MessageBox(hWnd, "Select DLL for the camera connected", "Error", MB_OK);
		return FALSE;
	}

	// Delete timer
	if(3 == g_Preview){
		KillTimer(hWnd, TIMER_ID);
	}


	// Release device
	g_CArtCamSdk.Close();


	// Release area for image
	SAFE_DELETE(g_pImage);


	// If drawing by yourself, set all window size to 0.
	// An automatic display can be performed, if window size is set up even when using CallBackPreview
	g_CArtCamSdk.SetPreviewWindow(hWnd, 0, 0, 0, 0);


	// Secure the space for taking in
	g_pImage = new BYTE[ArtCamGetSize()];

	g_CArtCamSdk.CallBackPreview(hWnd, g_pImage, ArtCamGetSize(), 0);


	// Check menu
	::CheckMenuItem(g_Menu.Preview, 0, MF_BYPOSITION | MF_UNCHECKED);
	::CheckMenuItem(g_Menu.Preview, 1, MF_BYPOSITION | MF_CHECKED);
	::CheckMenuItem(g_Menu.Preview, 3, MF_BYPOSITION | MF_UNCHECKED);
	::CheckMenuItem(g_Menu.Preview, 4, MF_BYPOSITION | MF_UNCHECKED);

	g_Preview = 1;

	ChangeScrollbar(hWnd);
	InvalidateRect(hWnd, NULL, TRUE);

	return TRUE;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Display one frame
//

BOOL OnSnapShot(HWND hWnd)
{
	// Not selected DLL
	if(-1 == g_DllType){
		MessageBox(hWnd, "Select DLL for the camera connected", "Error", MB_OK);
		return FALSE;
	}

	// Delete timer
	if(3 == g_Preview){
		KillTimer(hWnd, TIMER_ID);
	}


	// Release device
	g_CArtCamSdk.Close();


	// Release area for image
	SAFE_DELETE(g_pImage);


	// Secure the space for taking in
	g_pImage = new BYTE[ArtCamGetSize()];

	g_CArtCamSdk.SnapShot(g_pImage, ArtCamGetSize(), 0);


	// Draw image
	DrawImage(hWnd, NULL);

	// Check menu
	::CheckMenuItem(g_Menu.Preview, 0, MF_BYPOSITION | MF_UNCHECKED);
	::CheckMenuItem(g_Menu.Preview, 1, MF_BYPOSITION | MF_UNCHECKED);
	::CheckMenuItem(g_Menu.Preview, 3, MF_BYPOSITION | MF_UNCHECKED);
	::CheckMenuItem(g_Menu.Preview, 4, MF_BYPOSITION | MF_UNCHECKED);

	g_Preview = -1;

	return TRUE;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Draw line with SDK
//

LRESULT OnCapture(HWND hWnd)
{
	// Not selected DLL
	if(-1 == g_DllType){
		MessageBox(hWnd, "Select DLL for the camera connected", "Error", MB_OK);
		return FALSE;
	}


	// Release device
	g_CArtCamSdk.Close();

	// Release area for image
	SAFE_DELETE(g_pImage);

	// Secure the space for taking in
	g_pImage = new BYTE[ArtCamGetSize()];

	g_CArtCamSdk.Capture();


	// Check menu
	::CheckMenuItem(g_Menu.Preview, 0, MF_BYPOSITION | MF_UNCHECKED);
	::CheckMenuItem(g_Menu.Preview, 1, MF_BYPOSITION | MF_UNCHECKED);
	::CheckMenuItem(g_Menu.Preview, 3, MF_BYPOSITION | MF_CHECKED);
	::CheckMenuItem(g_Menu.Preview, 4, MF_BYPOSITION | MF_UNCHECKED);

	g_Preview = 3;

	SetTimer(hWnd, TIMER_ID, TIMER_TIME, NULL);

	return TRUE;
}


LRESULT OnTimer(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	if(3 == g_Preview && TIMER_ID == wParam){
		KillTimer(hWnd, TIMER_ID);

		g_CArtCamSdk.SnapShot(g_pImage, ArtCamGetSize(), 0);

		// Draw image
		DrawImage(hWnd, NULL);

		SetTimer(hWnd, TIMER_ID, TIMER_TIME, NULL);

		return 1;
	}

	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Trigger mode
//

BOOL OnTrigger(HWND hWnd)
{
	// Not selected DLL
	if(-1 == g_DllType){
		MessageBox(hWnd, "Select DLL for the camera connected", "Error", MB_OK);
		return FALSE;
	}

	// Delete timer
	if(3 == g_Preview){
		KillTimer(hWnd, TIMER_ID);
	}


	// Release device
	g_CArtCamSdk.Close();


	// Release area for image
	SAFE_DELETE(g_pImage);


	// If drawing by yourself, set all window size to 0.
	// An automatic display can be performed, if window size is set up even when using CallBackPreview
	g_CArtCamSdk.SetPreviewWindow(hWnd, 0, 0, 0, 0);


	// Secure the space for taking in
	g_pImage = new BYTE[ArtCamGetSize()];

	g_CArtCamSdk.Trigger(hWnd, g_pImage, ArtCamGetSize(), 0);


	// Check menu
	::CheckMenuItem(g_Menu.Preview, 0, MF_BYPOSITION | MF_UNCHECKED);
	::CheckMenuItem(g_Menu.Preview, 1, MF_BYPOSITION | MF_UNCHECKED);
	::CheckMenuItem(g_Menu.Preview, 3, MF_BYPOSITION | MF_UNCHECKED);
	::CheckMenuItem(g_Menu.Preview, 4, MF_BYPOSITION | MF_CHECKED);

	g_Preview = 4;

	ChangeScrollbar(hWnd);
	InvalidateRect(hWnd, NULL, TRUE);

	return TRUE;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Change DLL
//

BOOL OnDllChange(HWND hWnd, LONG DllType, LONG CameraType)
{
	// Release all devices before switching DLL
	// To display multiple images simulteously, you need to define CArtCamSdk plurally
	// When defined plurally, you do not need to relase to switch cameras
	g_CArtCamSdk.Release();
	g_CArtCamSdk.FreeLibrary();


	if(!g_CArtCamSdk.LoadLibrary(const_cast<LPTSTR>(DLLNAMELIST[DllType]))){
		TCHAR Error[256]="";
		wsprintf(Error, "%s is not found.\nDLL may have been relocated after executing", DLLNAMELIST[DllType]);
		MessageBox(hWnd, Error, "Error", MB_OK);
		return FALSE;
	}

	// Initialize is to be called first
	// By setting Window Handle here, WMLERROR can be obtained
	if(!g_CArtCamSdk.Initialize(hWnd)){
		MessageBox(hWnd, "Failed to initialize SDK", "Error", MB_OK);
		return FALSE;
	}


	// Select SATA camera type when use Sata.dll
	if(-1 != CameraType && ARTCAM_CAMERATYPE_SATA == HIWORD(g_CArtCamSdk.GetDllVersion())){
		g_CArtCamSdk.SetCameraType(CameraType);
		g_CArtCamSdk.SetDeviceNumber(0);
	}


	// Check menu
	for(int i=0; i<DLLCOUNT; i++){
		::CheckMenuItem(g_Menu.Dll, i, MF_BYPOSITION | MF_UNCHECKED);
	}

	::CheckMenuItem(g_Menu.Dll, DllType, MF_BYPOSITION | MF_CHECKED);


	// Switch to ON or OFF of setting window
	switch(DllType){
	// CNV
	case 3:
		::EnableMenuItem(g_Menu.Set, 0, MF_BYPOSITION | MF_ENABLED);
		::EnableMenuItem(g_Menu.Set, 1, MF_BYPOSITION | MF_ENABLED);
		::EnableMenuItem(g_Menu.Set, 2, MF_BYPOSITION | MF_GRAYED);

		::EnableMenuItem(g_Menu.Rate, 0, MF_BYPOSITION | MF_ENABLED);
		::EnableMenuItem(g_Menu.Rate, 1, MF_BYPOSITION | MF_ENABLED);
		::EnableMenuItem(g_Menu.Rate, 2, MF_BYPOSITION | MF_ENABLED);
		::EnableMenuItem(g_Menu.Rate, 3, MF_BYPOSITION | MF_ENABLED);
	break;

	// Other
	default:
		::EnableMenuItem(g_Menu.Set, 0, MF_BYPOSITION | MF_ENABLED);
		::EnableMenuItem(g_Menu.Set, 1, MF_BYPOSITION | MF_ENABLED);
		::EnableMenuItem(g_Menu.Set, 2, MF_BYPOSITION | MF_ENABLED);

		::EnableMenuItem(g_Menu.Rate, 0, MF_BYPOSITION | MF_GRAYED);
		::EnableMenuItem(g_Menu.Rate, 1, MF_BYPOSITION | MF_GRAYED);
		::EnableMenuItem(g_Menu.Rate, 2, MF_BYPOSITION | MF_GRAYED);
		::EnableMenuItem(g_Menu.Rate, 3, MF_BYPOSITION | MF_GRAYED);
	break;
	}


	// Device modification menu
	for(int i=0; i<g_DeviceCount; i++){
		::DeleteMenu(g_Menu.Device, 0, MF_BYPOSITION | MF_STRING);
	}


	TCHAR szName[10][256];
	g_DeviceCount = g_CArtCamSdk.EnumDevice(szName);
	for(i=0; i<g_DeviceCount; i++){
		::InsertMenu(g_Menu.Device, i, MF_BYPOSITION | MF_STRING, IDM_DEVICE + i, szName[i]);
	}
	::CheckMenuItem(g_Menu.Device, 0, MF_BYPOSITION | MF_CHECKED);


//	g_CArtCamSdk.SetHalfClock(0);
//	g_CArtCamSdk.SetAutoIris(1);

	switch(g_Preview){
	case 0:	return OnPreview(hWnd);
	case 1:	return OnCallBack(hWnd);
	case 3:	return OnCapture(hWnd);
	case 4:	return OnTrigger(hWnd);
	}

	return FALSE;
}



//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Change device
//

BOOL OnDeviceChange(HWND hWnd, LONG Number)
{
	// Not selected DLL
	if(-1 == g_DllType){
		MessageBox(hWnd, "Select DLL for the camera connected", "Error", MB_OK);
		return FALSE;
	}


	// Release the operative device before switching device
	g_CArtCamSdk.Close();

	// A device will be changed, if a camera is displayed after changing the number of a device now
	// Notes: A device is not changed in this function simple substance
	//   After calling this function, a device is changed by initializing a device
	g_CArtCamSdk.SetDeviceNumber(Number);

	for(int i=0; i<g_DeviceCount; i++){
		::CheckMenuItem(g_Menu.Device, i, MF_BYPOSITION | MF_UNCHECKED);
	}

	::CheckMenuItem(g_Menu.Device, Number, MF_BYPOSITION | MF_CHECKED);

	switch(g_Preview){
	case 0:	return OnPreview(hWnd);
	case 1:	return OnCallBack(hWnd);
	case 3:	return OnCapture(hWnd);
	case 4:	return OnTrigger(hWnd);
	}

	return FALSE;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Switch to Sampling rate
//

BOOL OnRateChange(HWND hWnd, LONG Number)
{
	// Not selected DLL
	if(-1 == g_DllType){
		MessageBox(hWnd, "Select DLL for the camera connected", "Error", MB_OK);
		return FALSE;
	}


	// Release the operative device before switching device
	g_CArtCamSdk.Close();

	g_CArtCamSdk.SetSamplingRate(Number);

	for(int i=0; i<4; i++){
		::CheckMenuItem(g_Menu.Rate, i, MF_BYPOSITION | MF_UNCHECKED);
	}

	::CheckMenuItem(g_Menu.Rate, Number, MF_BYPOSITION | MF_CHECKED);

	switch(g_Preview){
	case 0:	return OnPreview(hWnd);
	case 1:	return OnCallBack(hWnd);
	case 3:	return OnCapture(hWnd);
	case 4:	return OnTrigger(hWnd);
	}

	return FALSE;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	WM_CREATE
//

LRESULT OnCreate(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	TCHAR szFilePath[MAX_PATH]="";
	TCHAR szExePath[3][MAX_PATH]={"","",""};
	GetModuleFileName(::GetModuleHandle(NULL), szExePath[0], MAX_PATH);
	_splitpath(szExePath[0], szExePath[1], szExePath[2], NULL, NULL);
	wsprintf(szFilePath, "%s%sSample.ini", szExePath[1], szExePath[2]);

	g_DllType = ::GetPrivateProfileInt("Sample", "DllType", -1, szFilePath);
	g_CameraType = ::GetPrivateProfileInt("Sample", "CameraType", -1, szFilePath);

	return 1;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	WM_COMMAND
//

LRESULT OnCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	switch (LOWORD(wParam)){
	// Save file
	case IDM_SAVE:
//		g_CArtCamSdk.SaveImage("Image.bmp", FILETYPE_BITMAP);
//		g_CArtCamSdk.SaveImage("Image.raw", FILETYPE_RAW);
		g_CArtCamSdk.SaveImage("Imageh.jpg", FILETYPE_JPEG_HIGH);
//		g_CArtCamSdk.SaveImage("Imagen.jpg", FILETYPE_JPEG_NOMAL);
//		g_CArtCamSdk.SaveImage("Imagel.jpg", FILETYPE_JPEG_LOW);
//		g_CArtCamSdk.SaveImage("Image.png", FILETYPE_PNG);
//		g_CArtCamSdk.SaveImage("Image.tif", FILETYPE_TIFF);
	return 1;

	// End
	case IDM_EXIT:
		::PostMessage(hWnd, WM_CLOSE, 0, 0);
	return 1;

	// Draw with SDK
	case IDM_PREVIEW:
		OnPreview(hWnd);
	return 1;

	// Obtain pointer of image and draw
	case IDM_CALLBACK:
		OnCallBack(hWnd);
	return 1;

	// Obtain A image
	case IDM_SNAPSHOT:
		OnSnapShot(hWnd);
	return 1;

	// Capture snapshot in a low
	case IDM_CAPTURE:
		OnCapture(hWnd);
	return 1;

	// Trigger mode
	case IDM_TRIGGER:
		OnTrigger(hWnd);
	return 1;

	// Camera settings
	case IDM_FORMAT:
		// When show camera setting dialog,recommend to release display once.
		// Stop image refresh with SDK even when not stop with application.
		// In case of stop with SDK,it restarts when close dialog
		// There is possibility of illegality processing 
		// if you don't correspond setting with application when you change setting and close dialog.
		if(3 == g_Preview)	KillTimer(hWnd, TIMER_ID);

		g_CArtCamSdk.Close();
		g_CArtCamSdk.SetCameraDlg(hWnd);

		switch(g_Preview){
		case 0:	return OnPreview(hWnd);
		case 1:	return OnCallBack(hWnd);
		case 3:	return OnCapture(hWnd);
		case 4:	return OnTrigger(hWnd);
		}
	return 1;

	// Filter settings
	case IDM_FILTER:
		g_CArtCamSdk.SetImageDlg(hWnd);
	return 1;

	// Analog settings
	case IDM_ANALOG:
		g_CArtCamSdk.SetAnalogDlg(hWnd);
	return 1;

	// Switch to DLL
	case IDM_CAM:		g_DllType =  0;	g_CameraType = -1;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_USTC:		g_DllType =  2;	g_CameraType = -1;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_CNV:		g_DllType =  3;	g_CameraType = -1;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_130MI:		g_DllType =  5;	g_CameraType = -1;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_200MI:		g_DllType =  6;	g_CameraType = -1;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_300MI:		g_DllType =  7;	g_CameraType = -1;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_150P:		g_DllType =  8;	g_CameraType = -1;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_320P:		g_DllType =  9;	g_CameraType = -1;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_200SH:		g_DllType = 10;	g_CameraType = -1;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_098:		g_DllType = 11;	g_CameraType = -1;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_036MI:		g_DllType = 12;	g_CameraType = -1;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_500P:		g_DllType = 13;	g_CameraType = -1;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_150P2:		g_DllType = 14;	g_CameraType = -1;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_500MI:		g_DllType = 15;	g_CameraType = -1;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_T111:		g_DllType = 16;	g_CameraType = -1;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_T112:		g_DllType = 17;	g_CameraType = -1;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_150P3:		g_DllType = 18;	g_CameraType = -1;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_130MIV:	g_DllType = 19;	g_CameraType = -1;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_150P3V:	g_DllType = 20;	g_CameraType = -1;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_267KY:		g_DllType = 21;	g_CameraType = -1;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_274KY:		g_DllType = 22;	g_CameraType = -1;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
    case IDM_625KY:		g_DllType = 23;	g_CameraType = -1;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
    case IDM_V135MI:	g_DllType = 24;	g_CameraType = -1;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_445KY:		g_DllType = 28;	g_CameraType = -1;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
    case IDM_098II:		g_DllType = 29;	g_CameraType = -1;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
    case IDM_MV413:		g_DllType = 30;	g_CameraType = -1;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
    case IDM_OV210:		g_DllType = 31;	g_CameraType = -1;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
    case IDM_850SH:		g_DllType = 32;	g_CameraType = -1;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_1251SH:	g_DllType = 33;	g_CameraType = -1;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_D131:   	g_DllType = 34;	g_CameraType = -1;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_900MI:   	g_DllType = 35;	g_CameraType = -1;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_1000MI:   	g_DllType = 36;	g_CameraType = -1;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_500P2:   	g_DllType = 37;	g_CameraType = -1;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;

	case IDM_SATA_LVDS:	    g_DllType = 26;	g_CameraType = 0;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_SATA_300MI:	g_DllType = 26;	g_CameraType = 1;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_SATA_500MI:	g_DllType = 26;	g_CameraType = 2;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_SATA_MV413:	g_DllType = 26;	g_CameraType = 3;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_SATA_800MI:	g_DllType = 26;	g_CameraType = 4;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_SATA_036MI:	g_DllType = 26;	g_CameraType = 5;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_SATA_150P3:	g_DllType = 26;	g_CameraType = 6;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_SATA_267KY:	g_DllType = 26;	g_CameraType = 6;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_SATA_274KY:	g_DllType = 26;	g_CameraType = 7;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_SATA_625KY:	g_DllType = 26;	g_CameraType = 8;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_SATA_130MI:	g_DllType = 26;	g_CameraType = 9;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;
	case IDM_SATA_200MI:	g_DllType = 26;	g_CameraType =10;	OnDllChange(hWnd, g_DllType, g_CameraType);	return 1;

	// 
	case IDM_WIDEHI:		OnRateChange(hWnd, 0);	return 1;
	case IDM_WIDELOW:		OnRateChange(hWnd, 1);	return 1;
	case IDM_NORMALHI:		OnRateChange(hWnd, 2);	return 1;
	case IDM_NORMALLOW:		OnRateChange(hWnd, 3);	return 1;
	}


	// Change device
	for(int i=0; i<g_DeviceCount; i++){
		if(IDM_DEVICE + i == LOWORD(wParam)){
			OnDeviceChange(hWnd, i);
			return 1;
		}
	}

	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	WM_PAINT
//

LRESULT OnPaint(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	PAINTSTRUCT psPaint;
	HDC hDC = BeginPaint(hWnd, &psPaint);
	if(hDC){
		DrawImage(hWnd, hDC);
		EndPaint(hWnd, &psPaint);
	}

	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	WM_ERROR
//

LRESULT OnError(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	if(0 <= lParam && lParam < ERRORCOUNT-1){
		SetWindowText(hWnd, ERRORLIST[lParam]);
	}
	else {
		// Unknown error
		SetWindowText(hWnd, ERRORLIST[ARTCAMSDK_UNKNOWN]);
	}

	return 1;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	WM_GRAPHPAINT
//

LRESULT OnGraphPaint(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	if(NULL == wParam){
		return OnError(hWnd, wParam, lParam);
	}


	// Once this message is sent, latest image is stored at "g_pImage"
	LPGP_INFO pGpInfo = (LPGP_INFO)wParam;
	DrawImage(hWnd, NULL);


	TCHAR Fps[256]="";
	sprintf(Fps, "Sample  %.1f Frame", double(pGpInfo->lFps) / 10.0);
	SetWindowText(hWnd, Fps);

	return 1;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	WM_DESTROY
//

LRESULT OnDestroy(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	// Call "Release" when you end use of camera. (also called "destructor").
	// You need to call "Initialise" when you re-use camera after "Release".
	g_CArtCamSdk.Release();
	SAFE_DELETE(g_pImage);


	TCHAR szFilePath[MAX_PATH]="";
	TCHAR szExePath[3][MAX_PATH]={"","",""};
	GetModuleFileName(::GetModuleHandle(NULL), szExePath[0], MAX_PATH);
	_splitpath(szExePath[0], szExePath[1], szExePath[2], NULL, NULL);
	wsprintf(szFilePath, "%s%sSample.ini", szExePath[1], szExePath[2]);


	TCHAR Data[32]="";
	wsprintf(Data, "%d", g_DllType);
	::WritePrivateProfileString("Sample", "DllType", Data, szFilePath);

	wsprintf(Data, "%d", g_CameraType);
	::WritePrivateProfileString("Sample", "CameraType", Data, szFilePath);


	::PostQuitMessage(0);
	return 1;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	WM_SIZE
//

LRESULT OnSize(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	if(0 == g_Preview){
		HWND hChild = GetWindow(hWnd, GW_CHILD);

		RECT rc;
		GetClientRect(hWnd, &rc);


		// To avoiding changing aspect ratio, remove comment
/*
		// Calculation results in decimal figure. So apply floating-point calculations.
		double winW = rc.right;
		double winH = rc.bottom;
		double capW = ArtCamGetWidth();
		double capH = ArtCamGetHeight();

		if( (winW * capH) > (winH * capW) ){
			winW = winH * capW / capH;
			rc.right = LONG(winW);
		}
		else if( (winW * capH) < (winH * capW) ){
			winH = winW * capH / capW;
			rc.bottom = LONG(winH);
		}
*/
		SetWindowPos(hChild, HWND_TOP, 0, 0, rc.right, rc.bottom, SWP_NOMOVE | SWP_NOZORDER | SWP_SHOWWINDOW);
	}

	if(1 == g_Preview || 3 == g_Preview || 4 == g_Preview){
		ChangeScrollbar(hWnd);
	}

	return 1;
}


/////////////////////////////////////////////////////////////////////////////
//
//	WM_HSCROLL
//

LRESULT OnHScroll(HWND hWnd, UINT nSBCode, UINT nPos, HWND ScrollBar)
{
	SCROLLINFO sc;
	GetScrollInfo(hWnd, SB_HORZ, &sc);

	switch(nSBCode){
	// Scroll to th left end
	case SB_LEFT:
		SetScrollPos(hWnd, SB_HORZ, 0, TRUE);
	break;

	// Finish scrolling
	case SB_ENDSCROLL:
	break;

	// Scroll to left
	case SB_LINELEFT:
		SetScrollPos(hWnd, SB_HORZ, GetScrollPos(hWnd, SB_HORZ) - sc.nPage/4, TRUE);
	break;

	// Scroll to right
	case SB_LINERIGHT:
		SetScrollPos(hWnd, SB_HORZ, GetScrollPos(hWnd, SB_HORZ) + sc.nPage/4, TRUE);
	break;

	// Scroll to the left page
	case SB_PAGELEFT:
		SetScrollPos(hWnd, SB_HORZ, GetScrollPos(hWnd, SB_HORZ) - sc.nPage, TRUE);
	break;

	// Scroll to the right page
	case SB_PAGERIGHT:
		SetScrollPos(hWnd, SB_HORZ, GetScrollPos(hWnd, SB_HORZ) + sc.nPage, TRUE);
	break;

	// Scroll to the right end
	case SB_RIGHT:
		SetScrollPos(hWnd, SB_HORZ, sc.nMax, TRUE);
	break;

	// Scroll to absolute position. The current position is appointed by argument "nPos".
	case SB_THUMBPOSITION:
		nPos &= 0x0000FFFF;
		SetScrollPos(hWnd, SB_HORZ, nPos, TRUE);
	break;

	// Drag scroll box to the current position. The current position is appointed by argument "nPos".
	case SB_THUMBTRACK:
		nPos &= 0x0000FFFF;
		SetScrollPos(hWnd, SB_HORZ, nPos, TRUE);
	break;
	}

	g_ScrollCtrl.Pos.x = GetScrollPos(hWnd, SB_HORZ);
	DrawImage(hWnd, NULL);

	return 1;
}


/////////////////////////////////////////////////////////////////////////////
//
//	WM_VSCROLL
//

LRESULT OnVScroll(HWND hWnd, UINT nSBCode, UINT nPos, HWND ScrollBar)
{
	SCROLLINFO sc;
	GetScrollInfo(hWnd, SB_VERT, &sc);
	LONG pos = GetScrollPos(hWnd, SB_VERT);


	switch(nSBCode){
	// Scroll to th left end
	case SB_LEFT:
		SetScrollPos(hWnd, SB_VERT, 0, TRUE);
	break;

	// Finish scrolling
	case SB_ENDSCROLL:
	break;

	// Scroll to left
	case SB_LINELEFT:
		SetScrollPos(hWnd, SB_VERT, GetScrollPos(hWnd, SB_VERT) - sc.nPage/4, TRUE);
	break;

	// Scroll to right
	case SB_LINERIGHT:
		SetScrollPos(hWnd, SB_VERT, GetScrollPos(hWnd, SB_VERT) + sc.nPage/4, TRUE);
	break;

	// Scroll to the left page
	case SB_PAGELEFT:
		SetScrollPos(hWnd, SB_VERT, GetScrollPos(hWnd, SB_VERT) - sc.nPage, TRUE);
	break;

	// Scroll to the right page
	case SB_PAGERIGHT:
		SetScrollPos(hWnd, SB_VERT, GetScrollPos(hWnd, SB_VERT) + sc.nPage, TRUE);
	break;

	// Scroll to the right end
	case SB_RIGHT:
		SetScrollPos(hWnd, SB_VERT, sc.nMax, TRUE);
	break;

	// Scroll to absolute position. The current position is appointed by argument "nPos".
	case SB_THUMBPOSITION:
		nPos &= 0x0000FFFF;
		SetScrollPos(hWnd, SB_VERT, nPos, TRUE);
	break;

	// Drag scroll box to the current position. The current position is appointed by argument "nPos". 
	case SB_THUMBTRACK:
		nPos &= 0x0000FFFF;
		SetScrollPos(hWnd, SB_VERT, nPos, TRUE);
	break;
	}

	
	g_ScrollCtrl.Pos.y = GetScrollPos(hWnd, SB_VERT);
	DrawImage(hWnd, NULL);

	return 1;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Window procedure
//

LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg){
	case WM_CREATE:		return OnCreate			(hWnd, wParam, lParam);
	case WM_COMMAND:	return OnCommand		(hWnd, wParam, lParam);
	case WM_PAINT:		return OnPaint			(hWnd, wParam, lParam);
	case WM_GRAPHPAINT:	return OnGraphPaint		(hWnd, wParam, lParam);
	case WM_ERROR:		return OnError			(hWnd, wParam, lParam);
	case WM_DESTROY:	return OnDestroy		(hWnd, wParam, lParam);
	case WM_SIZE:		return OnSize			(hWnd, wParam, lParam);
	case WM_TIMER:		return OnTimer			(hWnd, wParam, lParam);
	case WM_HSCROLL:	return OnHScroll		(hWnd, LOWORD(wParam), HIWORD(wParam), (HWND)lParam);
	case WM_VSCROLL:	return OnVScroll		(hWnd, LOWORD(wParam), HIWORD(wParam), (HWND)lParam);
	default:			return ::DefWindowProc	(hWnd, uMsg, wParam, lParam);
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Window initialization
//

HWND InitWindow(HINSTANCE hInstance, int nCmdShow)
{
	// Registrate window class
	WNDCLASSEX wndclassex;
	ZeroMemory(&wndclassex,		sizeof(wndclassex));
	wndclassex.cbSize			= sizeof(wndclassex);
	wndclassex.style			= 0;
	wndclassex.lpfnWndProc		= WndProc;
	wndclassex.cbClsExtra		= 0;
	wndclassex.cbWndExtra		= 0;
	wndclassex.hInstance		= hInstance;
	wndclassex.hIcon			= LoadIcon(NULL, IDI_APPLICATION);
	wndclassex.hCursor			= LoadCursor(NULL, IDC_ARROW);
	wndclassex.hbrBackground	= reinterpret_cast<HBRUSH>(::GetStockObject(BLACK_BRUSH));
	wndclassex.lpszClassName	= "Sample";
	wndclassex.lpszMenuName		= 0;
	wndclassex.hIconSm			= NULL;

	if(0 == ::RegisterClassEx(&wndclassex))
		return NULL;

	// Window size setting
	RECT rcWndSize = {0, 0, 900, 600};
	DWORD dwStyle = WS_POPUPWINDOW | WS_CAPTION | WS_OVERLAPPEDWINDOW | WS_VSCROLL | WS_HSCROLL;
	if(0 == ::AdjustWindowRectEx(&rcWndSize, dwStyle, TRUE, NULL))
		return NULL;

	// Main window create
	HWND hWnd = ::CreateWindowEx(NULL, "Sample", "Sample", dwStyle,
			(::GetSystemMetrics(SM_CXSCREEN) - (rcWndSize.right - rcWndSize.left)) / 2,
			(::GetSystemMetrics(SM_CYSCREEN) - (rcWndSize.bottom - rcWndSize.top)) / 2,
			rcWndSize.right - rcWndSize.left, rcWndSize.bottom - rcWndSize.top,
			HWND_DESKTOP, NULL, hInstance, NULL);

	if(NULL == hWnd)
		return NULL;


	// Create menu

	// File menu
	g_Menu.File = ::CreateMenu();
	::InsertMenu(g_Menu.File, 0, MF_BYPOSITION | MF_STRING, IDM_SAVE, "Save(&S)");
	::InsertMenu(g_Menu.File, 1, MF_BYPOSITION | MF_STRING, IDM_EXIT, "End(&X)");

	// Display type menu
	g_Menu.Preview = ::CreateMenu();
	::InsertMenu(g_Menu.Preview, 0, MF_BYPOSITION | MF_STRING, IDM_PREVIEW, "Preview(&P)");
	::InsertMenu(g_Menu.Preview, 1, MF_BYPOSITION | MF_STRING, IDM_CALLBACK, "Callback(&B)");
	::InsertMenu(g_Menu.Preview, 2, MF_BYPOSITION | MF_STRING, IDM_SNAPSHOT, "Snapshot(&S)");
	::InsertMenu(g_Menu.Preview, 3, MF_BYPOSITION | MF_STRING, IDM_CAPTURE, "Capture(&C)");
	::InsertMenu(g_Menu.Preview, 4, MF_BYPOSITION | MF_STRING, IDM_TRIGGER, "Trigger(&T)");

	// Settings menu
	g_Menu.Set = ::CreateMenu();
	::InsertMenu(g_Menu.Set, 0, MF_BYPOSITION | MF_STRING, IDM_FORMAT, "Format settings(&C)");
	::InsertMenu(g_Menu.Set, 1, MF_BYPOSITION | MF_STRING, IDM_FILTER, "Filter settings(&F)");
	::InsertMenu(g_Menu.Set, 2, MF_BYPOSITION | MF_STRING, IDM_ANALOG, "Analog settings(&A)");

	// DLL selective menu
	g_Menu.Dll = ::CreateMenu();
	::InsertMenu(g_Menu.Dll,  0, MF_BYPOSITION | MF_STRING, IDM_CAM,   "DirectShow Camera");
	::InsertMenu(g_Menu.Dll,  1, MF_BYPOSITION | MF_SEPARATOR, NULL, NULL );
	::InsertMenu(g_Menu.Dll,  2, MF_BYPOSITION | MF_STRING, IDM_USTC,	"ARTUST");
	::InsertMenu(g_Menu.Dll,  3, MF_BYPOSITION | MF_STRING, IDM_CNV,   "ARTCNV");
	::InsertMenu(g_Menu.Dll,  4, MF_BYPOSITION | MF_SEPARATOR, NULL, NULL );
	::InsertMenu(g_Menu.Dll,  5, MF_BYPOSITION | MF_STRING, IDM_130MI, "ARTCAM-130MI");
	::InsertMenu(g_Menu.Dll,  6, MF_BYPOSITION | MF_STRING, IDM_200MI, "ARTCAM-200MI");
	::InsertMenu(g_Menu.Dll,  7, MF_BYPOSITION | MF_STRING, IDM_300MI, "ARTCAM-300MI");
	::InsertMenu(g_Menu.Dll,  8, MF_BYPOSITION | MF_STRING, IDM_150P,  "ARTCAM-150P");
	::InsertMenu(g_Menu.Dll,  9, MF_BYPOSITION | MF_STRING, IDM_320P,  "ARTCAM-320P");
	::InsertMenu(g_Menu.Dll, 10, MF_BYPOSITION | MF_STRING, IDM_200SH, "ARTCAM-200SH");
	::InsertMenu(g_Menu.Dll, 11, MF_BYPOSITION | MF_STRING, IDM_098,   "ARTCAM-098");
	::InsertMenu(g_Menu.Dll, 12, MF_BYPOSITION | MF_STRING, IDM_036MI, "ARTCAM-036MI");
	::InsertMenu(g_Menu.Dll, 13, MF_BYPOSITION | MF_STRING, IDM_500P,  "ARTCAM-500P");
	::InsertMenu(g_Menu.Dll, 14, MF_BYPOSITION | MF_STRING, IDM_150P2,	"ARTCAM-150P2");
	::InsertMenu(g_Menu.Dll, 15, MF_BYPOSITION | MF_STRING, IDM_500MI,  "ARTCAM-500MI");
	::InsertMenu(g_Menu.Dll, 16, MF_BYPOSITION | MF_STRING, IDM_T111,  "ARTCAM-T111");
	::InsertMenu(g_Menu.Dll, 17, MF_BYPOSITION | MF_STRING, IDM_T112,  "ARTCAM-T112");
	::InsertMenu(g_Menu.Dll, 18, MF_BYPOSITION | MF_STRING, IDM_150P3,  "ARTCAM-150P3");
	::InsertMenu(g_Menu.Dll, 19, MF_BYPOSITION | MF_STRING, IDM_130MIV,  "ARTCAM-130MI MOUT");
	::InsertMenu(g_Menu.Dll, 20, MF_BYPOSITION | MF_STRING, IDM_150P3V,  "ARTCAM-150P3 MOUT");
	::InsertMenu(g_Menu.Dll, 21, MF_BYPOSITION | MF_STRING, IDM_267KY,  "ARTCAM-267KY");
	::InsertMenu(g_Menu.Dll, 22, MF_BYPOSITION | MF_STRING, IDM_274KY,  "ARTCAM-274KY");
    ::InsertMenu(g_Menu.Dll, 23, MF_BYPOSITION | MF_STRING, IDM_625KY,	"ARTCAM-625KY");
	::InsertMenu(g_Menu.Dll, 24, MF_BYPOSITION | MF_STRING, IDM_V135MI,	"ARTCAM-V135MI");
	::InsertMenu(g_Menu.Dll, 25, MF_BYPOSITION | MF_SEPARATOR, NULL, NULL );
    ::InsertMenu(g_Menu.Dll, 27, MF_BYPOSITION | MF_SEPARATOR, NULL, NULL );
	::InsertMenu(g_Menu.Dll, 28, MF_BYPOSITION | MF_STRING, IDM_445KY,	"ARTCAM-445KY");
	::InsertMenu(g_Menu.Dll, 29, MF_BYPOSITION | MF_STRING, IDM_098II,	"ARTCAM-098II");
	::InsertMenu(g_Menu.Dll, 30, MF_BYPOSITION | MF_STRING, IDM_MV413,	"ARTCAM-MV413");
	::InsertMenu(g_Menu.Dll, 31, MF_BYPOSITION | MF_STRING, IDM_OV210,	"ARTCAM-OV210");
	::InsertMenu(g_Menu.Dll, 32, MF_BYPOSITION | MF_STRING, IDM_850SH,	"ARTCAM-850SH");
	::InsertMenu(g_Menu.Dll, 33, MF_BYPOSITION | MF_STRING, IDM_1251SH,	"ARTCAM-1251SH");
	::InsertMenu(g_Menu.Dll, 34, MF_BYPOSITION | MF_STRING, IDM_D131,	"ARTCAM-D131");
	::InsertMenu(g_Menu.Dll, 35, MF_BYPOSITION | MF_STRING, IDM_900MI,	"ARTCAM-900MI");
	::InsertMenu(g_Menu.Dll, 36, MF_BYPOSITION | MF_STRING, IDM_1000MI,	"ARTCAM-1000MI");
	::InsertMenu(g_Menu.Dll, 37, MF_BYPOSITION | MF_STRING, IDM_500P2,	"ARTCAM-500P2");


	// SATA Camera
	HMENU MenuSata = ::CreateMenu();
	::InsertMenu(MenuSata, 0, MF_BYPOSITION | MF_STRING, IDM_SATA_LVDS,   "LVDS");
	::InsertMenu(MenuSata, 1, MF_BYPOSITION | MF_STRING, IDM_SATA_300MI,  "300MI");
	::InsertMenu(MenuSata, 2, MF_BYPOSITION | MF_STRING, IDM_SATA_500MI,  "500MI");
	::InsertMenu(MenuSata, 3, MF_BYPOSITION | MF_STRING, IDM_SATA_MV413,  "MV413");
	::InsertMenu(MenuSata, 4, MF_BYPOSITION | MF_STRING, IDM_SATA_800MI,  "800MI");
	::InsertMenu(MenuSata, 5, MF_BYPOSITION | MF_STRING, IDM_SATA_036MI,  "036MI");
	::InsertMenu(MenuSata, 6, MF_BYPOSITION | MF_STRING, IDM_SATA_150P3,  "150P3");
	::InsertMenu(MenuSata, 7, MF_BYPOSITION | MF_STRING, IDM_SATA_267KY,  "267KY");
	::InsertMenu(MenuSata, 8, MF_BYPOSITION | MF_STRING, IDM_SATA_274KY,  "274KY");
	::InsertMenu(MenuSata, 9, MF_BYPOSITION | MF_STRING, IDM_SATA_625KY,  "625KY");
	::InsertMenu(MenuSata,10, MF_BYPOSITION | MF_STRING, IDM_SATA_130MI,  "130MI");
	::InsertMenu(MenuSata,11, MF_BYPOSITION | MF_STRING, IDM_SATA_200MI,  "200MI");
	::InsertMenu(g_Menu.Dll, 26, MF_BYPOSITION | MF_STRING | MF_POPUP, (UINT_PTR)MenuSata,  "SATA");


	// Sampling rate menu
	g_Menu.Rate = ::CreateMenu();
	::InsertMenu(g_Menu.Rate,  0, MF_BYPOSITION | MF_STRING, IDM_WIDEHI, "Wide HiSpeed");
	::InsertMenu(g_Menu.Rate,  1, MF_BYPOSITION | MF_STRING, IDM_WIDELOW, "Wide LowSpeed");
	::InsertMenu(g_Menu.Rate,  2, MF_BYPOSITION | MF_STRING, IDM_NORMALHI, "Normal HiSpeed");
	::InsertMenu(g_Menu.Rate,  3, MF_BYPOSITION | MF_STRING, IDM_NORMALLOW, "Normal LowSpeed");

	// Check if there is DLL
	for(int i=0; i<DLLCOUNT; i++){
		CArtCamSdk ArtCam;
		if(!ArtCam.LoadLibrary(const_cast<LPTSTR>(DLLNAMELIST[i]))){
			::EnableMenuItem(g_Menu.Dll, i, MF_BYPOSITION | MF_GRAYED);
		}
	}



	// Device modification menu
	g_Menu.Device = ::CreateMenu();

	// Main menu
	HMENU hMenu = ::CreateMenu();
	::InsertMenu(hMenu, 0, MF_POPUP | MF_STRING, (UINT)g_Menu.File,    "File(&F)");
	::InsertMenu(hMenu, 1, MF_POPUP | MF_STRING, (UINT)g_Menu.Preview, "Display(&V)");
	::InsertMenu(hMenu, 2, MF_POPUP | MF_STRING, (UINT)g_Menu.Set,     "Settings(&S)");
	::InsertMenu(hMenu, 3, MF_POPUP | MF_STRING, (UINT)g_Menu.Dll,     "DLL(&L)");
	::InsertMenu(hMenu, 4, MF_POPUP | MF_STRING, (UINT)g_Menu.Device,  "Device(&D)");
	::InsertMenu(hMenu, 5, MF_POPUP | MF_STRING, (UINT)g_Menu.Rate,    "Rate(&D)");


	::SetMenu(hWnd, hMenu);


	// Focus settings
	::SetFocus(hWnd);
	::ShowWindow(hWnd, nCmdShow);
	if(0 == ::UpdateWindow(hWnd))
		return NULL;

	// Initialize camera
	if(-1 != g_DllType)
		OnDllChange(hWnd, g_DllType, g_CameraType);

	SetTimer(hWnd, 1, 100, NULL);
	return hWnd;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	WinMain
//

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmLine, int nCmdShow)
{
	HWND hWnd = InitWindow(hInstance, nCmdShow);

	// Message roopMessage loop
	MSG msg;
	ZeroMemory(&msg, sizeof(msg));
	while(::GetMessage(&msg, NULL, 0, 0)){
		::TranslateMessage(&msg);
		::DispatchMessage(&msg);
	}

	return msg.wParam;
}



